/* Project:     HARTIK 3.0 Network Library                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Luca Abeni					*/
/* Date:        4/12/1997                                       */

/* File: 	3COM.C						*/
/* Revision:    2.0						*/

/*
   Low level network driver: it directly interacts with the ethernet
   card (for the moment, only a 3Com 3c59xxx). Provides an hardware
   independent interface to the ethernet layer
*/
extern int netlev;

#include <hartik.h>
#include <vm.h>
#include <mem.h>
#include <string.h>
#include <cons.h>

#include <pci.h>

#include "eth.h"
#include "netbuff.h"
#include "3com.h"

typedef struct pkt {
    int len;
    DWORD *txbuff;
} PKT;

extern struct eth_device eth_dev;
extern struct netbuff rxbuff;
extern void eth_pktArrived(void *pkt, int len);

struct netbuff lowlevel_txbuff;
SEM txmutex_sem;
PID txpid;
PORT txport;

BYTE lostpkt[ETH_MAX_LEN];

/* Get an ethernet frame from the card's hardware buffer */
int vortex_recv(DWORD BaseAddress,DWORD *buffer)
{
    int i = 0;
    int len;
    WORD err;

    err = VM_inw(BaseAddress + STATUS);
    if (err & RXERROR) {
	exc_raise(ETH_RXERROR);
    }
    len = VM_inw(BaseAddress + RXSTATUS) & RXBYTES;
    for(i = 0; i < ((len + 3) >> 2); i++) {
	buffer[i] = VM_ind(BaseAddress + RXDATA);
    }
    VM_outd(BaseAddress + COMMAND,RXDISCARD);
    return(i * 4); 
}


/*
   This is the external process used for receiving ethernet frame. (We use a
   process and not an handler for not jeopardizing the hard-real-time task
   garantee
*/
TASK net_extern_driver(void)
{
    int l;
    WORD status;
    DWORD *packet;

    for(;;) {
/* debug... */
netlev = 1;
	status = VM_inw(eth_dev.BaseAddress + STATUS);
#ifdef __ETH_DBG__
cprintf("enterin' driver\n");
#endif
	if (status & RXCOMPLETE) {
	    /* Frame arrived: try to get it */
	    if((packet = (DWORD *)netbuff_sequentialget(&rxbuff, NON_BLOCK)) !=0) {
		l = vortex_recv(eth_dev.BaseAddress, (DWORD *)packet);
		eth_pktArrived(packet, l);
	    } else {
	 	/* No receive buffers avaible: discard the frame */
		VM_outd(eth_dev.BaseAddress + COMMAND,RXDISCARD);
	    }
	} else if (status & RXEARLY) {
	    /* Idem */
	    if((packet = (DWORD *)netbuff_sequentialget(&rxbuff, NON_BLOCK)) !=0) {
		l = vortex_recv(eth_dev.BaseAddress, (DWORD *)packet);
		eth_pktArrived(packet, l);
	    } else {
		VM_outd(eth_dev.BaseAddress + COMMAND,RXDISCARD);
	    }
	    VM_outw(eth_dev.BaseAddress + COMMAND, ACKINTR | RXEARLY);
	} else {
	    /* This is impossible!!!!! */
	    VM_printf("!!!!!!!!!!!!!!!!!!!!!!Status: %x!!!!!\n",status);
	    sys_end();
	    exit(-1);
	}
	/* Enable the ethernet card to generate interrupts */
	VM_outw(eth_dev.BaseAddress + COMMAND, SETINTRENB | INTLATCH
	    			| RXEARLY | RXCOMPLETE);
#ifdef __ETH_DBG__
cprintf("exitin' \n");
#endif
	task_endcycle();
    }
}

/*
   This task is used to realize the mutual exclusion in sending frames
   (the other possibility is to use a semaphore)
*/
TASK net_sender_task()
{
    PORT inport;
    PKT txpkt;
    int i,j;
    int done;
    DWORD fsh;
    BYTE err;

    /* Create the communication port for the task */
    inport= port_create("txPort",sizeof(PKT),LOWLEV_TX_BUFFERS,MAILBOX,READ);
    task_endcycle();
    for(;;) {
	/* Somewhat activated the task: there should be a frame to send... */
	if (port_receive(inport,&txpkt,NON_BLOCK)){
	    done = 0;
	    fsh = (txpkt.len & 0x00001FFF) | 0x00008000;
#ifdef __ETH_DBG__
eth_printHeader(txpkt.txbuff);
cprintf("task sendin' %d bytes\n",txpkt.len);
#endif
	    while (!done) {
		while(VM_inw(eth_dev.BaseAddress + TXFREE) < txpkt.len);
		VM_outd(eth_dev.BaseAddress + TXDATA, fsh);
		for (i = 0; i < ((txpkt.len + 3 ) >> 2); i++) {
		    VM_outd(eth_dev.BaseAddress + TXDATA, txpkt.txbuff[i]);
		}
		VM_outd(eth_dev.BaseAddress + COMMAND, TXDONE);
		while((VM_inw(eth_dev.BaseAddress + STATUS) & TXCOMPLETE) != TXCOMPLETE);
		done = 1;
		/* If somewhat went wrong, repeat the transmission */
		err = VM_in (eth_dev.BaseAddress + TXSTATUS);
		if (err & 0x3F) {
		    if (err & 0x30) {
			VM_outw(eth_dev.BaseAddress + COMMAND, TXRESET);
			for (j = 20; j >= 0; j--)
			    if (!VM_inw(eth_dev.BaseAddress + STATUS) & CMDINPROGRESS) break;
		    }
		    VM_outw(eth_dev.BaseAddress + COMMAND, TXENABLE);
		    done = 0;
#ifdef __ETH_DBG__
cprintf("TXERR!!!%x\n",err);
#endif
		}
		VM_out(eth_dev.BaseAddress + TXSTATUS, 0);
	    }
	    netbuff_release(&lowlevel_txbuff,txpkt.txbuff);
	}
	/* Sleep until a new frame will be ready to be transimitted */
	task_endcycle();
    }
}


/*
   Gets the card informations from the PCI configuration registers stored
   in a pci_regs structure
*/

void vortex_info(struct pci_regs *r)
{
    DWORD BaseAddress;
    WORD lowaddr,midaddr,highaddr;
    DWORD InternalConfig;
    char addr[20];

    BaseAddress = r->IoBaseAddress;
    if (BaseAddress & 1) cprintf("        I/O mapped interface\n");
    	else cprintf("        Memory mapped interface\n");
    BaseAddress = BaseAddress & 0xFFFFFFF0;
    eth_dev.BaseAddress = BaseAddress;
    eth_dev.IntLine = r->InterruptLine;
    cprintf("        BaseAddress: %lx\n", eth_dev.BaseAddress);
    cprintf("        Interrupt Line: %d\n", eth_dev.IntLine);
    SETWIN(2);
/*    cprintf("STATUS:%x\n",VM_inw(BaseAddress + STATUS)); */
    lowaddr = VM_inw(BaseAddress + STATIONADDRESSLOW);
    midaddr = VM_inw(BaseAddress + STATIONADDRESSMID);
    highaddr = VM_inw(BaseAddress + STATIONADDRESSHIGH);
    sprintf(addr,"%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\n",lowaddr & 0xFF,
							lowaddr >> 8,
							midaddr & 0xFF,
							midaddr >> 8, 
							highaddr & 0xFF,
							highaddr >> 8);
    cprintf("        Ethernet address: %s\n",addr);
    eth_dev.addr.ad[0] = lowaddr & 0xFF;
    eth_dev.addr.ad[1] = lowaddr >> 8;
    eth_dev.addr.ad[2] = midaddr & 0xFF;
    eth_dev.addr.ad[3] = midaddr >> 8;
    eth_dev.addr.ad[4] = highaddr & 0xFF;
    eth_dev.addr.ad[5] = highaddr >> 8;
    SETWIN(3);
    InternalConfig = VM_ind(BaseAddress + INTERNALCONFIG);
}

/*
   Disable the ethernet card to generate interrupts: it is important for
   preventing chrashes at the end of an HARTIK program :-(
*/
void vortex_close(DWORD BaseAddress)
{
    VM_outw(BaseAddress + COMMAND, SETSTATUSENB | 0x00);
    VM_outw(BaseAddress + COMMAND, SETINTRENB | 0x00);
}

/*
   Shows the card's internal registers (it can be important to understand
   how it works
*/
void vortex_readregs(DWORD BaseAddress)
{
    WORD MacControl;
    DWORD InternalConfig;
    WORD MediaStatus;
    WORD RxFilter;

    SETWIN(3);
    MacControl = VM_inw(BaseAddress + MACCONTROL);
    InternalConfig = VM_ind(BaseAddress + INTERNALCONFIG);

    SETWIN(4);
    MediaStatus = VM_inw(BaseAddress + MEDIASTATUS);

    SETWIN(5);
    RxFilter = VM_inw(BaseAddress + RXFILTER);
    cprintf("Mac Control:\t%x\n",MacControl);
    cprintf("Internal Config:\t%lx\n",InternalConfig);
    cprintf("Media Status:\t%x\n",MediaStatus);
    cprintf("RxFilter:\t%x\n",RxFilter);
}

/*
   Init the ethernet card and all the structures used by the low level
   driver
*/
PID vortex_open(unsigned long int BaseAddress, int mode)
{
    DWORD InternalConfig;
    WORD MediaStatus;
    MODEL m = BASE_MODEL;
    
    int i;
    PID p;

    SETWIN(3);
    VM_outw(BaseAddress + MACCONTROL, 0);
    InternalConfig = VM_ind(BaseAddress + INTERNALCONFIG);
/*
	InternalConfig = (InternalConfig & 0xFFFF) | 0x01320000;
	This is for 10base2; let's use 10BaseT
*/		
    InternalConfig = (InternalConfig & 0xFFFF) | 0x01020000;
    VM_outd(BaseAddress + INTERNALCONFIG, InternalConfig);

    /* Reset the transceivers */
    VM_outw(BaseAddress + COMMAND, TXRESET);
    for(i = 20; i >= 0; i--)
    	if(!VM_inw(BaseAddress + STATUS) & CMDINPROGRESS) break;
    VM_outw(BaseAddress + COMMAND, RXRESET);
    for(i = 20; i >= 0; i--)
    	if(!VM_inw(BaseAddress + STATUS) & CMDINPROGRESS) break;
    VM_outw(BaseAddress + COMMAND, SETSTATUSENB | 0x00);
    if(((InternalConfig >> 20) & 7) == 3) {
        VM_outw(BaseAddress + COMMAND,STARTCOAX);
    }
	
    SETWIN(4);
    MediaStatus = (VM_inw(BaseAddress + MEDIASTATUS) & ~(0x00C0 | 0x0008)) | 0x00C0;
    VM_outw(BaseAddress + MEDIASTATUS, MediaStatus);

    VM_outw(BaseAddress + COMMAND, SETRXFILTER | 0x05);
    VM_outw(BaseAddress + COMMAND, RXENABLE);
    VM_outw(BaseAddress + COMMAND, TXENABLE);
    VM_outw(BaseAddress + COMMAND, SETSTATUSENB | 0xff);
    /* Enable to generate interrupts */
    VM_outw(BaseAddress + COMMAND, ACKINTR | INTLATCH | TXAVAILABLE |
    				RXEARLY | INTREQ);
    VM_outw(BaseAddress + COMMAND, SETINTRENB | INTLATCH | TXAVAILABLE |
				     RXCOMPLETE);
    /* Create the driver process (for receiving frames)	*/
    task_def_wcet(m, 100);
    p = task_create("rxProc", net_extern_driver, SOFT, APERIODIC, 20, &m);
    if (p == NIL) {
	VM_printf("Cannot create rxProc\n");
	sys_end();
	exit(-1);
    }
    if (mode == TXTASK) {
	/* Create a transmission task, if it is needed */
	netbuff_init(&lowlevel_txbuff, LOWLEV_TX_BUFFERS, ETH_MAX_LEN);
	txpid = task_create("txProc", net_sender_task, SOFT, APERIODIC, 20, &m);
	if (txpid == NIL) {
	    VM_printf("Cannot create txProc\n");
	    sys_end();
	    exit(-1);
	}
	task_activate(txpid);
	txport = port_connect("txPort", sizeof(PKT), MAILBOX,WRITE);
    /* Otherwise, create the mutual exclusion semaphore */
    } else txmutex_sem = sem_create(1);
    return p;
}

void vortex_pktdiscard(void)
{
    VM_outd(eth_dev.BaseAddress + COMMAND,RXDISCARD);
}

/*
   Send a frame using a semaphore to realize the mutual exclusion
   (shared memory)
*/
void vortex_send_mem(DWORD BaseAddress, DWORD *txbuff, int len)
{
    int i,j;
    int done = 0;
    DWORD fsh;
    BYTE err;
    
    fsh = (len & 0x00001FFF) | 0x00008000;

    sem_wait(txmutex_sem, BLOCK);
#ifdef __ETH_DBG__
eth_printHeader(txbuff);
cprintf("sendin' %d bytes\n",len);
#endif
    while (!done) {
	while(VM_inw(BaseAddress + TXFREE) < len);
	VM_outd(BaseAddress + TXDATA, fsh);
	for (i = 0; i < ((len + 3 ) >> 2); i++) { 
	    VM_outd(BaseAddress + TXDATA, txbuff[i]);
	}
	VM_outd(BaseAddress + COMMAND, TXDONE);
	while((VM_inw(BaseAddress + STATUS) & TXCOMPLETE) != TXCOMPLETE);
	done = 1;
	err = VM_in (BaseAddress + TXSTATUS);
	if (err & 0x3F) {
	    if (err & 0x30) {
		VM_outw(BaseAddress + COMMAND, TXRESET);
		for (j = 20; j >= 0; j--)
		    if (!VM_inw(BaseAddress + STATUS) & CMDINPROGRESS) break;
	    }
	    VM_outw(BaseAddress + COMMAND, TXENABLE);
	    done = 0;
#ifdef __ETH_DBG__
cprintf("TXERR!!!%x\n",err);
#endif
	}
	VM_out(BaseAddress + TXSTATUS, 0);
    }
    sem_signal(txmutex_sem);
}

/* Send a frame using a tx task to realize the mutual exclusion */
void vortex_send_msg(DWORD BaseAddress, DWORD *txbuff, int len)
{
    PKT pkt;

    pkt.len = len;
    pkt.txbuff = netbuff_get(&lowlevel_txbuff, NON_BLOCK);
    memcpy(pkt.txbuff, txbuff, len);
    port_send(txport, &pkt,NON_BLOCK);
    task_activate(txpid);
}

/* Fast Handler (it only disables the network card to generate interrupts) */
void net_handler(void)
{
    VM_outw(eth_dev.BaseAddress + COMMAND, SETINTRENB);
    VM_outw(eth_dev.BaseAddress + COMMAND, ACKINTR | INTLATCH | INTREQ);
#ifdef __ETH_DBG__
cprintf("disablin' irq...");
#endif
}
