/*--------------------------------------------------------------*/
/*		      TEST ON CABS				*/
/*--------------------------------------------------------------*/

/* CVS $Id: cabs.c,v 1.2 1999/07/16 12:27:00 massy Exp $ */

#include <x86/xdos.h>
#include <sys/kern.h>
#include <sys/cabs.h>
#include <cons.h>
#include <string.h>
#include <glib.h>
#include <keyb.h>

#define	NCAB	4		/* max number of CABs	*/
#define	NCAR	26		/* generated characters	*/

#define	YP	32		/* level of arrows	*/
#define	R	20		/* task radius		*/
#define	YY	(YP+R+32)	/* level of writing	*/
#define	DELTA	(2*R+72)	/* total channel hight	*/
#define	X1	120		/* start column for P1	*/
#define	X2	360		/* start column for P2	*/

#define	XP1	(X1+64)		/* X position of task 1	*/
#define	XP2	(X2+64)		/* X position of task 2	*/
#define	XC	(XP1+96)	/* X position of CAB	*/
#define	L	52		/* CAB rectangle length	*/

void	my_exit(KEY_EVT *k);
void	draw_channel(int i);
void	create_channel(int i);
void	get_data();

MODEL	m = BASE_MODEL;
TASK	producer(), consumer();

char	*cname[NCAB] = {"cab1", "cab2", "cab3", "cab4"};
char	*pname1[NCAB] = {"wr1", "wr2", "wr3", "wr4"};
char	*pname2[NCAB] = {"rd1", "rd2", "rd3", "rd4"};

CAB	cid[NCAB];		/* CAB identifiers	*/
PID	p1[NCAB], p2[NCAB];	/* task identifiers	*/
int	t1[NCAB], t2[NCAB];	/* task periods		*/
int	tick;			/* system tick		*/

/*--------------------------------------------------------------*/
/* User exit function						*/
/*--------------------------------------------------------------*/

void	my_exit(KEY_EVT *k)
{
	grx_close();
	sys_end();
}

/*--------------------------------------------------------------*/
/* Main task							*/
/*--------------------------------------------------------------*/

TASK	main(int argc, char *argv[])
{
char	c = 0;
int	i;
SYS_PARMS	par = BASE_SYS;
KEYB_PARMS	keyb = BASE_KEYB;

	get_data();			/* get data from file */

	sys_def_nocheck(par);
	sys_def_tick(par, tick, mSec);
	sys_init(&par);

	keyb_def_ctrlC(keyb, my_exit);
	keyb_init(&keyb);

	grx_open(640,480,8);
	grx_clear(BLACK);

	grx_text("Press a key [1-4]", 10, 16, 7, 0);
	grx_text("to create a pair",  10, 24, 7, 0);
	grx_text("ESC to exit demo",  10, 48, 7, 0);

	while (c != 27) {
		c = keyb_getch(BLOCK);
		if ((c >= '1') && (c <= '1'+NCAB-1))
			create_channel(c-'1');
	}
  
	sys_end();
	grx_close();

	return 0;
}

/*--------------------------------------------------------------*/
/* write data in a cab						*/
/*--------------------------------------------------------------*/

TASK	producer(int i)
{
char	c;			/* message character		*/
char	*p;			/* pointer to a cab buffer	*/
char	s[2];			/* string to display		*/
int	k = 0;
int	x, y;
int	col = 13;
int	ybase = YY + i*DELTA;

	x = X1;
	y = ybase;
	s[1] = 0;

	k = 0;
	while (1) {
		c = 'A' + k;
		p = cab_reserve(cid[i]);
		*p = c;
		cab_putmes(cid[i], p);

		s[0] = c;
		k = (k + 1) % NCAR;
		grx_text(s,x,y,col,0);

		x += 8;
		if (x >= (X1 + NCAR*8)) {
			x = X1;
			y = y + 8;
			if (y >= ybase+16) {
				y = ybase;
				col = col % 15 + 1;
			}
		}

		task_endcycle();
	}
}

/*--------------------------------------------------------------*/
/* read data from a cab						*/
/*--------------------------------------------------------------*/

TASK	consumer(int i)
{
char	*p;
char	s[2];
int	x, y;
int	col = 13;
int	ybase = YY + i*DELTA;

	x = X2;
	y = ybase;
	s[1] = 0;

	while (1) {
		p = cab_getmes(cid[i]);
		s[0] = *p - 'A' + 'a';
		cab_unget(cid[i], p);

		grx_text(s,x,y,col,0);
		x += 8;

		if (x >= (X2 + NCAR*8)) {
			x = X2;
			y = y + 8;
			if (y >= ybase+16) {
				y = ybase;
				col = col % 15 + 1;
			}
		}
		task_endcycle();
	}
}

/*--------------------------------------------------------------*/
/* create the two tasks and a channel				*/
/*--------------------------------------------------------------*/

void	create_channel(int i)
{
	draw_channel(i);
	cid[i] = cab_create(cname[i], 1, 2);

	task_def_arg(m, i);
	p1[i] = task_create(pname1[i], producer, HARD, PERIODIC, t1[i], &m);
	task_activate(p1[i]);

	task_def_arg(m, i);
	p2[i] = task_create(pname2[i], consumer, HARD, PERIODIC, t2[i], &m);
	task_activate(p2[i]);
}

/*--------------------------------------------------------------*/
/* Disegna i processi e il canale di comunicazione		*/
/*--------------------------------------------------------------*/

void	draw_channel(int i)
{
char	buffer[32];			/* buffer per sprintf	*/
int	yc = YP + i*DELTA;		/* altezza del canale	*/

	grx_circle(XP1,yc,R,2);
	grx_text("P1",XP1-8,yc-4,12,0);

	grx_circle(XP2,yc,R,2);
	grx_text("P2",XP2-8,yc-4,12,0);

	grx_rect(XC,yc-R,XC+L,yc+R,3);
	grx_text("CAB",XC+16,yc-4,12,0);

	grx_line(XP1+R,yc,XC,yc,4);
	grx_line(XC+L,yc,XP2-R,yc,4);

	grx_text("T1 =       ms",X1+40,yc+R+16,14,0);
	sprintf(buffer,"%d", t1[i]);
	grx_text(buffer,X1+88,yc+R+16,14,0);

	grx_text("T2 =       ms",X2+40,yc+R+16,14,0);
	sprintf(buffer,"%d", t2[i]);
	grx_text(buffer,X2+88,yc+R+16,14,0);
}

/*--------------------------------------------------------------*/

void	get_data(void)
{
char		c;		/* carattere letto              */
int		i;		/* indice array			*/
DOS_FILE	*fp;
char buffer[256];
int  num;        
char *s;
   
   fp = DOS_fopen("cabs.dat","r");
   DOS_getline(buffer,sizeof(buffer),'\n',fp);

   DOS_getline(buffer,sizeof(buffer),'\n',fp);
   s=strchr(buffer,':');
   if (!s) exit(-1);
   num=sscanf(s+1,"%i",&tick);
   if (num!=1) exit(-1);
   
   DOS_getline(buffer,sizeof(buffer),'\n',fp);
   DOS_getline(buffer,sizeof(buffer),'\n',fp);
   DOS_getline(buffer,sizeof(buffer),'\n',fp);
   DOS_getline(buffer,sizeof(buffer),'\n',fp);
   
   for (i=0;i<NCAB;i++)  {
      DOS_getline(buffer,sizeof(buffer),'\n',fp);
      s=strchr(buffer,':');
      if (!s) exit(-1);
      num=sscanf(s+1,"%i",t1+i);
      if (num!=1) exit(-1);
      s=strchr(s+1,':');
      if (!s) exit(-1);
      num=sscanf(s+1,"%i",t2+i);
      if (num!=1) exit(-1);
   }
   
   DOS_fclose(fp);      
   return ;
}

/*--------------------------------------------------------------*/
