/*
 * Grx.c
 *
 * Author: Massimiliano Giorgi
 * Date: 10/5/99
 */

/* CVS $Id: grx.c,v 1.4 1999/07/16 09:16:24 massy Exp $ */

#include <sys/kern.h>
#include <sys/sys.h>
#include <glib.h>
#include <keyb.h>
#include <mouse.h>

extern long int srand(long int seed);
extern long int rand(void);

/*
 *
 */

int mode;
int w;
int h;
int d;

/*
 *
 */

#define CMDLINE    0
#define MODEERR    1
#define SETERROR   2
#define GRXINITERR 3

char *err_msg[]={
  "command line error!",
  "mode not supported!",
  "can't set this graphic mode!",
  "can't init GRX module!"
};

void exit_error(int index)
{
  cprintf("%s\n",err_msg[index]);
  if (index==CMDLINE) {
    cprintf("example: x grx 800x600x16\n");
  }
  sys_end();
  exit(-1); 
}

void press_a_key(void)
{
  cprintf("Press a key to continue...");
  keyb_getch(BLOCK);
  cprintf("\n");
}

/*
 *
 */

void enter_graph_mode(void)
{
  int result;
  void realize_palette(void);
  
  result=grx_setmode(mode);
  if (result==-1) exit_error(SETERROR);
  
  if (d==8) realize_palette();
}

void leave_graph_mode(void)
{
  grx_close();
}

/* color management */
/* (to do better!) */

DWORD color(BYTE red, BYTE green, BYTE blue)
{
  switch(d) {
    case 15: return rgb15(red,green,blue);
    case 16: return rgb16(red,green,blue);
    case 24: return rgb24(red,green,blue);
    case 32: return rgb32(red,green,blue);
  }
  return (red*7+green*3+blue)%256;
}

DWORD indexcolor(int index)
{
  if (d==8) return index;
  switch(index) {
    case 0: return color(0,0,0);
    case 1: return color(255,255,255);
    case 2: return color(255,0,0);
    case 3: return color(0,255,0);
    case 4: return color(0,0,255);
  }
  return color(255,255,255);
}

#undef BLACK
#undef WHITE
#undef RED
#undef GREEN
#undef BLUE
#define BLACK indexcolor(0)
#define WHITE indexcolor(1)
#define RED   indexcolor(2)
#define GREEN indexcolor(3)
#define BLUE  indexcolor(4)
#define NUMINDEXCOLORS 5

void realize_palette(void)
{
  int i;
  
  if (d!=8) return;
  
  grx_setcolor(0,0,0,0);
  grx_setcolor(1,255,255,255);
  grx_setcolor(2,255,0,0);
  grx_setcolor(3,0,255,0);
  grx_setcolor(4,0,0,255);
  srand(47);
  for (i=5;i<256;i++)
    grx_setcolor(i,rand()%256,rand()%256,rand()%256);
}

#define DYCHAR 8
#define DXCHAR 8

void continue_graph(void)
{
  grx_text("Press a key to continue...",(w-DXCHAR*26)/2,50,RED,BLACK);
  press_a_key();
}

/*
 *
 * Those are the Demos
 *
 */

void lines_run(void)
{
  int x,y;
  int xo,yo;
  int i;
  int col;
  
  grx_clear(BLACK);  
  for (y=0;y<h;y+=16)
    grx_line(0,0,w-1,y,WHITE);
  continue_graph();

  grx_clear(BLACK);  
  for (y=0;y<h;y+=16)
    grx_line(w-1,0,0,y,WHITE);
  continue_graph();

  grx_clear(BLACK);  
  for (y=0;y<h;y+=16)
    grx_line(w-1,h-1,0,y,WHITE);
  continue_graph();

  grx_clear(BLACK);  
  for (y=0;y<h;y+=16)
    grx_line(0,h-1,w-1,y,WHITE);
  continue_graph();
  
  srand(19);
  grx_clear(BLACK);
  xo=rand()%w;
  yo=rand()%h;
  for (i=0,col=0;i<50;i++) {
    x=rand()%w;
    y=rand()%h;
    grx_line(xo,yo,x,y,indexcolor(col+2));
    xo=x;
    yo=y;
    col=(col+1)%3;
  }
  continue_graph();

  return;
}

void griglia_run(void)
{
  double scale_x,scale_y;
  int x,y;

  #define NUMXBLOCK 64
  #define NUMYBLOCK (NUMXBLOCK*3/4)
  
  scale_x=w/(double)(NUMXBLOCK-1);
  scale_y=h/(double)(NUMYBLOCK-1);
  grx_clear(BLACK);  
  for (y=0;y<NUMYBLOCK;y+=1)
    grx_line(0,y*scale_y,w-1,y*scale_y,WHITE);
  for (x=0;x<NUMXBLOCK;x+=1)
    grx_line(x*scale_x,0,x*scale_x,h-1,WHITE);
  continue_graph();

  scale_x=w/(double)(NUMXBLOCK/2-1);
  scale_y=h/(double)(NUMYBLOCK/2-1);  
  grx_clear(BLUE);  
  for (y=0;y<NUMYBLOCK/2;y+=1)
    grx_line(0,y*scale_y,w-1,y*scale_y,WHITE);
  for (x=0;x<NUMXBLOCK/2;x+=1)
    grx_line(x*scale_x,0,x*scale_x,h-1,WHITE);
  continue_graph();

  return;
}

void circles_run(void)
{
  int x0,y0;
  int r;
  
  grx_clear(BLACK);
  for (r=5;r<w/2&&r<h/2;r+=15)
    grx_circle(w/2,h/2,r,RED);
  continue_graph();

  grx_clear(BLACK);
  for (x0=w/4;x0<w;x0+=w/4)
    for (y0=h/4;y0<h;y0+=h/4)
      for (r=5;r<w/4&&r<h/4;r+=15)
	grx_circle(x0,y0,r,GREEN);
  continue_graph();

  return;
}

void discs_run(void)
{
  int x0,y0;
  int r;
  int index=1;
  
  grx_clear(BLACK);
  for (x0=w/4;x0<w;x0+=w/4)
    for (y0=h/4;y0<h;y0+=h/4)
      for (r=150;r>15;r-=15) {
	grx_disc(x0,y0,r,indexcolor(index));
	index=(index+1)%NUMINDEXCOLORS;
	if (index==0) index=1;
      }
  continue_graph();

  return;
}

void colors_run(void)
{
  int c;
  int dx;

  dx=w/256;
  for (c=0;c<256;c++)
    grx_box(c*dx,0,(c+1)*dx-1,h-1,color(c,0,0));
  continue_graph();

  dx=w/256;
  for (c=0;c<256;c++)
    grx_box(c*dx,0,(c+1)*dx-1,h-1,color(0,c,0));
  continue_graph();

  dx=w/256;
  for (c=0;c<256;c++)
    grx_box(c*dx,0,(c+1)*dx-1,h-1,color(0,0,c));
  continue_graph();

  dx=w/256;
  for (c=0;c<256;c++)
    grx_box(c*dx,0,(c+1)*dx-1,h-1,color(c,c,c));
  continue_graph();

  return;
}

void box_run(void)
{
  int c;
  int dx,dy;
  DWORD col;
  
  dx=w/128;
  dy=h/128;
  for (c=0;c<256;c+=4) {
    switch((c/4)%3) {
      case 0: col=color(c,0,0); break;
      case 1: col=color(0,c,0); break;
      case 2: col=color(0,0,c); break;
    }
    grx_box(c/4*dx,c/4*dy,w-c/4*dx-1,h-c/4*dy-1,col);
  }
  continue_graph();
    
}

void rect_run(void)
{
  int c;
  int dx,dy;
  DWORD col;
  
  dx=w/128;
  dy=h/128;
  for (c=0;c<256;c+=4) {
    switch((c/4)%3) {
      case 0: col=color(c,0,0); break;
      case 1: col=color(0,c,0); break;
      case 2: col=color(0,0,c); break;
    }
    grx_rect(c/4*dx,c/4*dy,w-c/4*dx-1,h-c/4*dy-1,col);
  }
  continue_graph();

  return;
}

void text_run(void)
{
  int dx,dy;
  char s[2]="\0";
  int c;
  
  dx=10;
  dy=10;
  for (c=0;c<128;c++) {
    *s=c;
    grx_text(s,100+(c/8)*(dx+4)+dx,100+(c%8)*(dy+4)+dy,WHITE,BLACK);
  }
  continue_graph();

  return;
}

void pixel_run(void)
{
  int i;
  int x,y;
  
  srand(137);
  for (i=0;i<40000;i++)
    grx_plot(rand()%w,rand()%h,color(rand()%256,rand()%256,rand()%256));
  continue_graph();

  for (x=0;x<w/2;x++)
    for (y=0;y<h;y++)
      grx_plot(w-x-1,y,grx_getpixel(x,y));
  continue_graph();

  return;
}

/**/

struct {
  char *name;
  char *desc;
  void (*run)(void);
} demo[]={
  { "line", "grx_line() demo", lines_run},
  { "circ", "grx_circle() demo", circles_run},
  { "disc", "grx_disc() demo", discs_run},
  { "box ", "grx_box() demo", box_run},
  { "rect", "grx_rect() demo", rect_run},
  { "text", "grx_text() demo", text_run},
  { "pixe", "grx_plot() & grx_getpixel() demo", pixel_run},
  { "grig", "show grid", griglia_run},
  { "colo", "show bar of colors", colors_run},
  { "", "", NULL}
};

/*
 *
 *
 *
 */

void my_ctrlC(KEY_EVT *evt)
{
  grx_close();
  sys_end();
  exit(0);
}

int main(int argc,char *argv[])
{
  SYS_PARMS   parms=BASE_SYS;
  MODEL       model=BASE_MODEL;
  KEYB_PARMS  keyb =BASE_KEYB;
  MOUSE_PARMS mouse=BASE_MOUSE;
  int         result;
  int         ch;
  int         i;
  int         cont;
  
  /* sys, keyb and mouse init */
  sys_def_nocheck(parms);
  sys_init(&parms);

  /* keyboard initialization */
  keyb_def_ctrlC(keyb,my_ctrlC);
  keyb_init(&keyb);
  
  mouse_def_ps2(mouse);
  mouse_init(&mouse);
  
  if (argc!=2) exit_error(CMDLINE);
  result=sscanf(argv[1],"%ix%ix%i",&w,&h,&d);
  if (result!=3) exit_error(CMDLINE);

  result=grx_init();
  if (result==-1) exit_error(GRXINITERR);
  
  mode=grx_getmode(w,h,d);
  if (mode==-1) exit_error(MODEERR);

  grx_cardinfo();
  press_a_key();

  for (;;) {
    clear();
    cprintf("\nResolution: %ix%i\n",w,h);
    cprintf("Depth:      %i bits/pixel\n\n",d);

    cprintf("Choose a test:\n\n");
    i=0;
    for (;;) {
      if (demo[i].run==NULL) break;
      cprintf("%c: %s\n",(char)(i+'a'),demo[i].desc);
      i++;
    }
    cprintf("x: Exit\n\n");

    cprintf("> ");
    cont=TRUE;
    while (cont) {
      ch=keyb_getch(BLOCK);
      if (ch=='x') {
	sys_end();
	grx_close();
	return 0;
      }
      if (ch>='a'&&ch<'a'+i) {
	enter_graph_mode();
	demo[ch-'a'].run();
	leave_graph_mode();
	cont=FALSE;
      }
    }
  }

  /* safety */
  sys_end();
  return 0;
}

