#
# Customizing the makefile:
# 1) Select the name of the final application
#   APP = <chosen name>
#
# 2) If the application is composed by a single .obj file, that's all.
#    Otherwise add to the APP_OBJS the other .obj files
#
# 3) Use M <chosen name> to build up the application
#    Use M <chosen name> clean to clean up all files produced (.obj & .err)
#

#
# Application name
#

ifndef APP
	APP = aster
endif

APP_OBJS = $(APP).o

#
# Standard path
#

HARTIK       = ..
APP_PATH     = .
INCLUDE_PATH = $(HARTIK)/H
LIB_PATH     = $(HARTIK)/LIB

CC           = gcc
C_OPT        =  -o $*.o -O -finline-functions -fno-builtin \
		-I$(INCLUDE_PATH) -I$(INCLUDE_PATH)/x86 -I$(INCLUDE_PATH)/sys \
		-D__BASE_CONST__ -D__C_HEADER__ -D__GNU__ -c
ASM_OPT      = -I$(INCLUDE_PATH) -I$(INCLUDE_PATH)/x86 -I$(INCLUDE_PATH)/sys -D__GNU__
ifdef REDIR
LINK_OPT     = -oformat coff-go32 -s -nostartfiles -nostdlib -L$(LIB_PATH)
LINK_STARTUP = $(LIB_PATH)/x0.o $(LIB_PATH)/k_standa.o
LIB_MATH     = -lhm
LD           = ld
else
LINK_OPT     = -s -nostartfiles -nostdlib -L$(LIB_PATH)
LINK_STARTUP = $(LIB_PATH)/x0.o
LIB_MATH     = -lm
LD           = gcc
endif

LIB_VM       = -lvm
LIB_OS       = -lhart
LIB_DEV      = -lhdev
LIB_STD      = -lhlib
LIB_PCI      = -lhpci
LIB_COMP     = -lcomp
LIB_NET      = -lhnet
LIB_SND      = -lhsnd
#LIB_GL       = -lvgagl
LIB_GRAPH    = -lhgd

LINK_LIB = $(LIB_OS)    \
	   $(LIB_DEV)   \
	   $(LIB_GL)    \
	   $(LIB_GRAPH) \
	   $(LIB_NET)   \
	   $(LIB_COMP)  \
	   $(LIB_PCI)   \
	   $(LIB_SND)   \
	   $(LIB_VM)    \
	   $(LIB_STD)   \
	   $(LIB_MATH)
#
# Ok! Finally the dependency rules!
# We do not mess with automatic depencencies here!!
#

$(APP) : $(APP).o
	$(LD) $(LINK_OPT) $(LINK_STARTUP) $(APP).o $(LINK_LIB) -o $(APP)

%.o : %.c
	$(REDIR) $(CC) $(C_OPT) $<

#
# An example dependency for a .S file!
# NOTE: To use #include & #define in .S file You need to declare
#       it as .S and not .s EVEN if the DOS do not understand the difference
#
# OtherFile.o : OtherFile.S
#   as $(ASM_OPT) -c $< -o $@

clean :
	-del *.o
	-del *.err
	-del $(APP)
	
