/*--------------------------------------------------------------*/
/*		SIMULATION OF JUMPING BALLS			*/
/*--------------------------------------------------------------*/

/* CVS $Id: palles.c,v 1.3 1999/07/16 13:20:07 massy Exp $ */

void	my_fine();

#include <sys/kern.h>
#include <glib.h>
#include <keyb.h>
#include <math.h>

#define	R	2		/* dimension of a ball		*/
#define	G	9.8		/* acceleration of gravity	*/
#define	YMENU	50		/* menu level			*/
#define	BASE	50		/* position of the floor	*/
#define	TOP	300		/* initial height of the ball	*/
#define	XMIN	50		/* min position X of the ball	*/
#define	XMAX	500		/* max position X of the ball	*/
#define	VELX	15.		/* horizontal ball velocity	*/
#define	VMIN	11.		/* velocit minima per suono	*/
#define	ESC	27		/* ASCII code of ESCAPE key	*/
#define MAX_P	10		/* max number of balls		*/

SEM	mxa, mxv;		/* mutual esclusion semaphores	*/
double	v0[MAX_P];		/* impact velocity with floor	*/
double	tick = 1.0;		/* system tick = 1 ms		*/
double	period = 12.0;		/* period of ball task=10 tick	*/
PID	pid, ps;

/*--------------------------------------------------------------*/

void my_fine()
{
	grx_close();
	sys_end();
}

/*--------------------------------------------------------------*/
/*	Aperiodic task for sound generation			*/
/*--------------------------------------------------------------*/

TASK	suono()
{
	while (1) {
		//sound(2000,5);
		task_endcycle();
	}
}

/*--------------------------------------------------------------*/
/*	Periodic task for ball simulation			*/
/*--------------------------------------------------------------*/

TASK	palla(WORD i)
{
int	x, y;		/* coordinate grafiche pallina	*/
int	ox, oy;		/* vecchia posizione pallina	*/
int	x0;		/* posizione iniziale X pallina	*/
float	vx, vy;		/* velocit della pallina	*/
float	t, tx;		/* variabile temporale		*/
float	z, z0;		/* posizione della pallina	*/
float	dt;		/* incremento temporale		*/
double	arg;		/* variabile di appoggio	*/

    y = oy = TOP;
    x = ox = x0 = XMIN;

    arg = 2. * G * (float)TOP;
    vy = sqrt(arg);
    vx = VELX;
    tx = 0.0;
    t = vy / G;
    dt = period * tick / 100.;

    while (1) {
	x = x0 + vx * tx;
	y = BASE + vy*t - .5*G*t*t;

	if (y < BASE) {
		if (v0[i] > VMIN) task_activate(ps);
		t = 0.0;
		sem_wait(mxv,BLOCK);
		v0[i] = .9 * v0[i];
		vy = v0[i];
		sem_signal(mxv);
		y = BASE + vy*t - .5*G*t*t;
	}

	if (x > XMAX) {
		tx = 0.0;
		x0 = XMAX;
		vx = -vx;
		x = x0 + vx * tx;
	}

	if (x < XMIN) {
		tx = 0.0;
		x0 = XMIN;
		vx = -vx;
		x = x0 + vx * tx;
	}
	y = 480-y;

	sem_wait(mxa,BLOCK);
	grx_disc(ox, oy, R, 0);
	grx_disc(x, y, R, 2+i);
	sem_signal(mxa);

	oy = y; ox = x;
	t += dt;
	tx += dt;
	task_endcycle();
    }

    task_kill(task_exec());
}

/*--------------------------------------------------------------*/
/*			MAIN process				*/
/*--------------------------------------------------------------*/

#define PALLAGROUP 511

TASK	main(void)
{
char	c;		/* character from keyboard	*/
int	i;		/* pressed number		*/
int	npc = 0;	/* number of tasks created	*/
double	arg;		/* temporary variable		*/

SYS_PARMS	parms = BASE_SYS;
KEYB_PARMS	keyb = BASE_KEYB;
MODEL		ma = BASE_MODEL;
MODEL		mp = BASE_MODEL;

    arg = 2. * G * (float)TOP;
    for (i=0; i<MAX_P; i++) v0[i] = sqrt(arg);

    grx_open(640,480,8);
    grx_rect(XMIN-R-1, 480-TOP-BASE-R-1, XMAX+R+1, 480-BASE+R+1, 14);
    grx_text("SPACE create a ball", XMIN, YMENU+10, 12, 0);
    grx_text("0-9   jump ball 0-9", XMIN, YMENU+20, 12, 0);
    grx_text("ESC   exit to DOS",   XMIN, YMENU+30, 12, 0);

    sys_def_tick(parms,tick*1000,uSec);
    sys_def_nocheck(parms);
    sys_init(&parms);
    keyb_init(&keyb);

    mxa = sem_create(1);
    mxv = sem_create(1);

    ps = task_create("suono", suono, NRT, APERIODIC, 2, &ma);

    do {
	c = keyb_getch(BLOCK);
	i = c - '0';
	if ((c == ' ') && (npc < MAX_P)) {
	    task_def_arg(mp, (WORD)npc);
	    task_def_group(mp, PALLAGROUP);
	    task_def_wcet(mp, 50);
	    pid = task_create("palla", palla, HARD, PERIODIC, period, &mp);
	    if (pid == NIL) {
		group_kill(PALLAGROUP);
		grx_close();
		perror();
		sys_end();
		return -1;
	    }
	    task_activate(pid);
	    npc++;
	}

	if ((i >= 0) && (i <= 9)) {
	    arg = 2. * G * (float)TOP;
	    sem_wait(mxv, BLOCK);
	    v0[i] = sqrt(arg);
	    sem_signal(mxv);
	}

    } while (c != ESC);

    sys_end();
    grx_close();
}

/*--------------------------------------------------------------*/
