/*--------------------------------------------------------------*/
/*                    TEST ON SRP SCHEDULING                    */
/*--------------------------------------------------------------*/

#include "hartik.h"

#include "cons.h"
#include "keyb.h"
#include "glib.h"

#define LM      	40              /* line of main         */
#define OFFSET  	20              /* initial phase        */
#define CHAR_DIM	8		/* Height of chars in pixels */
#define RES		0		/* SRP resource		*/

int     col[3] = {2, 4, 14};            /* colors of timelines  */
int     lev[3] = {80, 120, 160};        /* level of timelines   */
int     ptime[3] = {5, 5, 5};           /* number of cycles     */
int     cstime[3] = {20, 20, 20};       /* crit. section cycles */
int     period[3] = {80, 100, 130};    	/* tasks' periods       */
int     tick = 50;                     /* system tick          */
int     tscale = 50;                   /* time scale           */
TIME 	starttime = 0;			/* Simulation start time */

SEM     mutex;				/* Semaphore for graphix*/

/*--------------------------------------------------------------*/
/*      Prints a grid to show task periods during execution     */
/*--------------------------------------------------------------*/

void    print_grid()
{
int     i;
int     a1, a2, a3;
int     temp;
char    str[50];

	a1 = 0;
	a2 = 0;
	a3 = 0;
	temp = 0;

        grx_text("EDF+SRP SCHEDULE", 272, 240 ,10, 0);

        grx_line(OFFSET, lev[0], 639, lev[0], 5);
        grx_line(OFFSET, lev[1], 639, lev[1], 5);
        grx_line(OFFSET, lev[2], 639, lev[2], 5);
        grx_text("T1", 0, lev[0]-8, 9, 0);
        grx_text("T2", 0, lev[1]-8, 9, 0);
        grx_text("T3", 0, lev[2]-8, 9, 0);
        grx_text("MAIN", 0,     LM, 8, 0);

	for (i = OFFSET; i < 640; i++) {
		if (temp >= a1) {
                        grx_line(i, lev[0] - 1, i, lev[0] - 20, 3);
			a1 += period[0];
		}
		if (temp >= a2) {
                        grx_line(i, lev[1] - 1, i, lev[1] - 20, 3);
			a2 += period[1];
		}
		if (temp >= a3) {
                        grx_line(i, lev[2] - 1, i, lev[2] - 20, 3);
			a3 += period[2];
		}
		temp += tick/tscale;
	}
}

/*--------------------------------------------------------------*/
/*      This function is called at system termination           */
/*--------------------------------------------------------------*/
void    my_end()
{
        grx_close();
	sys_status(0xFFFF);
}

/*--------------------------------------------------------------*/
/*      GENERIC PERIODIC PROCESS                                */
/*--------------------------------------------------------------*/

TASK    color(int k)
{
int     i;
DWORD   x = OFFSET;
TIME 	t;
	while (x < 600L) {
		for (i = 0; i < ptime[k]; i++) {
			t = sys_ticks();
			x = t - starttime + OFFSET;
			sem_wait(mutex, BLOCK);
                        grx_plot(x, lev[k] - 4, col[k]);
                        grx_plot(x, lev[k] - 5, col[k]);
                        grx_plot(x, lev[k] - 6, col[k]);
                        grx_plot(x, lev[k] - 7, col[k]);
			sem_signal(mutex);
			while (sys_ticks() == t);
		}

		srp_wait(RES);
		for (i = 0; i < cstime[k]; i++) {
			t = sys_ticks();
			x = t - starttime + OFFSET;
			sem_wait(mutex, BLOCK);
                        grx_plot(x, lev[k] - 4, 15);
                        grx_plot(x, lev[k] - 5, 15);
                        grx_plot(x, lev[k] - 6, 15);
                        grx_plot(x, lev[k] - 7, 15);
			sem_signal(mutex);
			while (sys_ticks() == t);
		}
		srp_signal(RES);

		for (i = 0; i < ptime[k]; i++) {
			t = sys_ticks();
			x = t - starttime + OFFSET;
			sem_wait(mutex, BLOCK);
                        grx_plot(x, lev[k] - 4, col[k]);
                        grx_plot(x, lev[k] - 5, col[k]);
                        grx_plot(x, lev[k] - 6, col[k]);
                        grx_plot(x, lev[k] - 7, col[k]);
			sem_signal(mutex);
			while (sys_ticks() == t);
		}
		task_endcycle();
	}
}

/*--------------------------------------------------------------*/
/*      MAIN PROCESS                                            */
/*--------------------------------------------------------------*/

void  main(void)
{
int     i;
int     x = OFFSET;
MODEL   m = BASE_MODEL;
SYS_PARMS si = BASE_SYS;
char 	c;
KEY_EVT emerg;
int modenum;

	sys_def_tick(si, tick, mSec);
	sys_def_nocheck(si);
	sys_init(&si);
	
        if (grx_init() == -1) {
		cprintf("Error initing GraphLib!!!\n");
		sys_end();
	}
        modenum = grx_getmode(640, 480, 8);
	cprintf("Modenum :%d\n", modenum);

        if (grx_setmode(modenum) == -1) {
		cprintf("No SetMode!!!\n");
		sys_end();
    	}
	print_grid();

	sys_atexit(my_end, AFTER_EXIT);
	keyb_init(NULL);
  	emerg.ascii = 'x';
    	emerg.scan = KEY_X;
    	emerg.flag = ALTL_BIT;
    	keyb_hook(emerg,my_end);
	keyb_getchar();

	mutex = sem_create(1);

	task_def_group(m, 1);

	task_def_arg(m, 0);
	task_use_res(m, RES, 1);
	task_def_wcet(m, ptime[0] * sys_tick);
	if (task_create("verde", color, HARD, PERIODIC, period[0], &m) == -1) {
	    perror();
	    cprintf("Edf.C(main) Could not create <green>\n");
	    sys_end();
	    exit(-1);
	}

	task_def_arg(m, 1);
	task_use_res(m, RES, 1);
	task_def_wcet(m, ptime[1] * sys_tick);
	if (task_create("red", color, HARD, PERIODIC, period[1], &m) == -1) {
	    perror();
	    cprintf("Edf.C(main) Could not create <red>\n");
	    sys_end();
	    exit(-1);
	}

	task_def_arg(m, 2);
	task_use_res(m, RES, 1);
	task_def_wcet(m, ptime[2] * sys_tick);
	if (task_create("yellow", color, HARD, PERIODIC, period[2], &m) == -1) {
	    perror();
	    cprintf("Edf.C(main) Could not create <yellow>\n");
	    sys_end();
	    exit(-1);
	}
	starttime = sys_ticks();
	group_activate(1);

	while (x < 640L) {
		x = sys_ticks() - starttime + OFFSET;
		sem_wait(mutex, BLOCK);
                grx_plot(x, LM, 7);
		sem_signal(mutex);
	}

	c = keyb_getchar();
	sys_end();
}
