/* Project:     HARTIK 3.0 Network Library                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Giuseppe Lipari & Luca Abeni                    */
/* Date:        1/12/97						*/

/* File:        Talk.C                                          */
/* Revision:    1.00                                            */

/*
   Simple Netlib demo: nothing of seriously real-time, only another Unix
   Talk clone.
   Read it to see how the UDP/IP layers of the networ library work.
*/                   
#include <hartik.h>
#include <mem.h>
#include <stdlib.h>
#include <string.h>
#include <cons.h>
#include <crtwin.h>
#include <keyb.h>

#include <udpip.h>

WIN dbg;
BYTE esc = FALSE;

char talk_myipaddr[20];
char talk_toipaddr[20];

/*
   This non real-time task reads UDP packets from the network and writes
   them in a window
*/
TASK scrittore(void)
{
    char str[2000];
    UDP_ADDR from, local;
    WIN displ;
    int s,n;

    /* Connect on the local port #100 */
    local.s_port = 100;
    s = udp_bind(&local, NULL);

    /* Open the window */
    win_init(&displ,0,0,79,6);
    win_frame(&displ,BLACK,WHITE,"Remote",2);

    while (1) {
        /* Clear the buffer for receiving the packet...*/
	memset(str, 0, 1999);
        /*...and receive the packet (block until a packet arrives */
	n = udp_recvfrom(s, str, &from);
	win_puts(&displ, str);
    }
}

/*
   This non real-time task reads strings from the keyoard and sends them
   to the remote host
*/
TASK write(void)
{
    WIN wr;
    UDP_ADDR to,local;
    char str[80];
    int s;
    IP_ADDR bindlist[5];

    /* Create a socket for transitting */
    ip_str2addr(talk_myipaddr,&(local.s_addr));
    local.s_port = 101;

    /*
       If we want the address of the remote host in the ARP table before
       begginning the transmission (to eliminate a possible source of
       unpredictability), we can use the bindlist, otherwise we set the
       second parameter of udp_bind to NULL
    */
    ip_str2addr(talk_toipaddr,&(bindlist[0]));
    memset(&(bindlist[1]), 0, sizeof(IP_ADDR));
    s = udp_bind(&local, /*bindlist*/NULL);

    win_init(&wr,0,7,79,6);
    win_frame(&wr,BLACK,WHITE,"Local",2);
    ip_str2addr(talk_toipaddr,&(to.s_addr));
    to.s_port = 100;
    sprintf(str,"Local IP address %d.%d.%d.%d\n", local.s_addr.ad[0],
    				local.s_addr.ad[1], local.s_addr.ad[2],
				local.s_addr.ad[3]);
    win_puts(&dbg,str);
    sprintf(str,"Talk to %d.%d.%d.%d   ",to.s_addr.ad[0],to.s_addr.ad[1],
    				to.s_addr.ad[2],to.s_addr.ad[3]);
    win_puts(&dbg,str);
    while (1) {
        /* Get the string...*/
	win_gets(&wr,str,78);
	strcat(str,"\n");
        /*...and send it!!! */
	udp_sendto(s,str,strlen(str)+2,&to);
    }
}

/* This function is called when the user presses CTRL-C (stops the systems) */
void esci(KEY_EVT *k)
{
    esc = TRUE;
    cprintf("Ctrl-Brk pressed!\n");
    sys_status(NORM_STATUS);
    sys_end();
    exit(1);
}

void main(void)
{
    KEY_EVT k;
    SYS_PARMS sp = BASE_SYS;
    struct net_model m = net_base;

   sys_def_nocheck(sp);
/*    sys_set_tick(sp,1,mSec);
    sys_set_Us(sp,0.3);*/
    sys_init(&sp);
    keyb_init(NULL);
    k.flag = CNTL_BIT;
    k.scan = KEY_C;   
    k.ascii = 'c';
    keyb_hook(k,esci);
    k.flag = CNTR_BIT;
    k.scan = KEY_C;   
    k.ascii = 'c';
    keyb_hook(k,esci);

    clear();
    cprintf(" Hartik Talk! 			Ver. 1.00\n");
    strcpy(talk_myipaddr, "193.205.82.40");
    strcpy(talk_toipaddr, "193.205.82.46");

    /* We want a task for TX mutual exclusion */
    net_setmode(m, TXTASK);
    /* We use UDP/IP stack */
    net_setudpip(m, talk_myipaddr);
    /* OK: let's start the NetLib! */
    if (net_init(&m) == 1) {
		cprintf("Net Init OK...\n");
	} else {
		cprintf("Net Init Failed...\n");
		sys_abort(300);
	}


//dump_irq();
	
    cprintf("\n\n\n\tPress ENTER\n");
    while (!esc) {
	keyb_getcode(&k,BLOCK);
	if (k.ascii == 13) esc = TRUE;
    }

    esc = FALSE; 
    clear();
    win_init(&dbg,0,20,60,3);
    win_frame(&dbg,BLACK,WHITE,"Debug",2);
    
    /* Start the sender and receiver tasks...*/
    task_activate(task_create("aaa",scrittore,NRT,APERIODIC,11,NULL));
    task_activate(task_create("bbb",write,NRT,APERIODIC,11,NULL));

    /*...and wait!!! */
#if 0
#endif
    while (!esc) {
    }
    sys_end();
    clear();
}
