/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Sebastiano Piga					*/
/* Date:	9/5/96 						*/

/* File: 	GRAPH.H						*/
/* Revision:	1.0 						*/

#ifndef __GRAPH_H__
#define __GRAPH_H__

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************************************/
/*                       Global Constant		                   */
/***************************************************************************/

#define G_NOCOL         0x80    /* nessun colore scelto         */
#define G_POLYCROM	0x79	/* poligono policromo		*/
#define G_NOPP          0x00    /* modalita' di scrittura video */
#define G_ANDP          0x01    /* modalita' di scrittura video */
#define G_ORP           0x02    /* modalita' di scrittura video */
#define G_XORP          0x03    /* modalita' di scrittura video	*/

#define G_TEXT		0x03	/* modalita' testo		*/
#define G_VGA           0x12    /* modalita' grafica VGA        */
#define G_SVGAL         0x6a    /* modalita' grafica SVGAL      */
#define G_EGA		0x10    /* modalita' grafica EGA        */
#define G_MCGA		0x13	/* modalita' grafica MCGA       */

#define G_DIREZ_0	0x00	/* direzione destra -> sinistra */
#define G_DIREZ_1	0x01    /* direzione sinistra -> destra */
#define G_DIREZ_2	0x02    /* direzione verticale          */
#define G_DIREZ_3	0x03    /* varianti delle precedenti    */
#define G_DIREZ_4	0x04
#define G_DIREZ_5	0x05
#define G_DIREZ_6	0x06    /* direzione sinistra -> destra	*/
				/* con interlinea doppia	*/

#define G_PRECLEAR	0x01	/* Flag x la gPrintf()		*/
#define G_INCRX		0x02
#define G_INCRY		0x04

#define G_BLACK		0x00	/* costanti indicanti 16 colori	*/
#define G_BLUE		0x01
#define G_GREEN		0x02
#define G_CYAN          0x03
#define G_RED           0x04
#define G_MAGENTA       0x05
#define G_BROWN         0x06
#define G_DARKGRAY      0x07
#define G_LIGHTGRAY     0x08
#define G_LIGHTBLUE     0x09
#define G_LIGHTGREEN    0x0a
#define G_LIGHTCYAN     0x0b
#define G_LIGHTRED      0x0c
#define G_LIGHTMAGENTA  0x0d
#define G_YELLOW        0x0e
#define G_WHITE         0x0f

enum ROT_MASK {ROT0,ROT1,ROT2,ROT3,ROT4,ROT5,ROT6,ROT7};

/***************************************************************************/
/*                              Datat structures	                   */
/***************************************************************************/

/* 2D Point */
typedef struct {
	int x;
	int y;
	BYTE color;
} point;

/* Line */
typedef struct {
	int x1,y1,x2,y2;
	BYTE color;
} lpoint;

/* RGB color */
typedef struct {
	BYTE red, green ,blue;
} rgb_color;

/* Standard VGA color palette */
typedef rgb_color palette[256];

/* Bitmap data structure */
typedef struct {
	int dx,dy;
	BYTE *buf;
} bitmap;

/***************************************************************************/
/*                        Function prototypes		                   */
/***************************************************************************/

int  gOpen(BYTE mode);
void gClose(void);
void __cdecl gSync(void);
void gClear(BYTE col, BYTE pag);
void gVideoOn(void);
void gVideoOff(void);

BYTE gGetPixel(int x, int y);
void gPlot(int x, int y, BYTE col);
void gLine(int x1,int y1,int x2,int y2,BYTE col);
void gRect(int x1, int y1, int x2, int y2, BYTE col);
void gRectFl(int x1,int y1,int x2,int y2,BYTE col,BYTE brk);
void gCircle(int x1, int y1, int r, BYTE col);
void gEllipse(int x1, int y1, int ra, int rb, BYTE col);
void gEllipseFl(int x1,int y1,int ra,int rb,BYTE col,BYTE brk);
void gCircleFl(int x1, int y1, int r, BYTE col, BYTE brk);
void gPoly(point V[],int lun,int col);
int gPolyFl(point pol[], int lung, BYTE col, BYTE bord);

void gGetPos(point *p);
void gSetPos(int x, int y);
void gLineTo(int x, int y, BYTE colore);
void gLineRel(int dx, int dy, BYTE colore);

int  gSetActivePage(BYTE pag);
void gSetWritePage(BYTE pag);
void gSetMode(BYTE func);
void gSetStyle(BYTE style);
void gSetFirstPoint(BYTE endp);

int gGetActivePage(void);
int gGetWritePage(void);
int gGetMode(void);
int gGetStyle(void);
int gGetFirstPoint(void);

int gXMax(void);
int gYMax(void);
int gMaxPage(void);
DWORD gMaxCol(void);

void gSetRGB(BYTE ind, BYTE R, BYTE V, BYTE B);
void gSetOverscan(BYTE ind);
void gSetColReg(BYTE* R);
void gSetPalette(palette Pal);

int gGetOverscan(void);
int gGetBkg(BYTE pagina);
void gGetColReg(BYTE *Reg);
void gGetRGB(BYTE ind,rgb_color *r);
void gGetPalette(palette p);

void gPutStyle(BYTE msk[], BYTE direz);
void gPutc(int x,int y,BYTE col,BYTE brk,BYTE car);
void gPuts(int x,int y,int col, int brk, BYTE *stri);
int gPrintf(int *x, int *y, BYTE col, BYTE bkg, unsigned fl, char *fmt, ...);
unsigned long gImageSize(int left,int top,int right,int bottom);
int gGetImage(int left,int top,int right,int bottom,bitmap *bmp_p);
int gPutImage(int left,int top,BYTE op,bitmap *bmp_p);

#ifdef __cplusplus
};
#endif

#endif
