/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/*              Giuseppe Lipari                                 */
/* Date:        1/7/96                                          */

/* File: 	Exc.H 						*/
/* Revision:    3.0 	                                        */
/* Date:	14/3/97						*/

/* The list of the kernel error codes				*/
/* $HEADER+							*/

/* The exception table is allocated as following 			*/
/* Entries in the [0..63] range are reserved for VM exceptions		*/
/* Entries in the [64..127] range are reserved for Kernel system	*/
/* Entries in the [128..191] range are used for the HOOKs		*/
/* Entries in the [192..255] range are the user level exceptions	*/

/* Anyway this can be subject to change, if necessary!			*/

#ifndef __EXC_H__
#define __EXC_H__

#define DIV_BY_0		0 /* These are the VM... exceptions */
#define MATH_EXC		1
#define NMI_EXC			2
#define DEBUG_EXC		3
#define BREAKPOINT_EXC		4
#define VM_FAULT		5
#define NO_MORE_HW_DESC		6

#define CLOCK_OVERRUN		64 /* Warning this is used in vm1.asm   */

#define HARD_MISS		65 /* These are the kernel errors       */
#define NO_AVAIL_TASK		66 /* Check for others kernel exception */
#define NO_AVAIL_STACK_MEM	67 /* index into CABS.C & PORT.C	*/
#define UNVALID_SET		68 /* Port errors: 90..100		*/
#define WRONG_QUEUE		69 /* Cabs errors: 110..120		*/
#define UNVALID_SEM		70 /* Log errors:  121..125		*/
#define NO_MORE_SEM		71
#define WRONG_INT_NO		72
#define UNVALID_KILL		73
#define UNVALID_CREATE		74
#define UNVALID_GROUP		75
#define OUT_OF_MEM			76
#define CANNOT_GUARANTEE	77
#define SRP_STACK_FULL		78
#define SRP_STACK_EMPTY		79
#define SRP_BAD_EXTRACT		80
#define TOOMUCH_EXITFUNC	81
#define WCET_VIOLATION		82
#define UNVALID_TASK_ID		83
#define USED_INT_NO			84
#define UNUSED_INT_NO		85
#define GENERIC_ERROR		86

#define LAST_KERN_ERROR		86

#define DOUBLE_EXCEPTION	126
#define UNKNOWN_ERROR		127

#define CREATE_HOOK		128 /* The hooks are executed each time */
#define ACTIVATE_HOOK		129 /* a primitive is invoked		*/
#define KILL_HOOK		130 /* But only if we have compiled the */
#define ABORT_HOOK		131 /* kernel with __HOOK__ set!	*/
#define ENDCYCLE_HOOK		132
#define SLEEP_HOOK		133
#define DELAY_HOOK		134
#define GROUP_ACT_HOOK		135
#define GROUP_KIL_HOOK		136
#define SEM_CREATE_HOOK		137
#define SEM_DELETE_HOOK		138
#define SEM_WAIT_HOOK		139
#define SEM_SIGNAL_HOOK		140

#endif /* __EXC_H__ */

