/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/*              Giuseppe Lipari                                 */
/* Date:        1/7/96                                          */

/* CVS $Id: model.h,v 1.3 1999/07/16 10:08:15 massy Exp $ */

/* File:        Model.H                                         */
/* Revision:    1.0                                             */
/* Date:	14/3/97						*/

/* $HEADER+							*/

/* User appeal for the task model				*/

/* This structure is only used to group together a set of optional  	*/
/* parameters describing the task model passing them to the		*/
/* task_create. 							*/
/* The control field is used to set special task processing		*/
/* functions. Currently it only supports the USE_FPU bit to 		*/
/* indicate that the task uses also FPU registers. If the hw		*/
/* architecture is not smart enough to allow automatic FPU context  	*/
/* switch, this information is used at the VM level to perform 		*/
/* transparently the preemption of a FPU-task   			*/

#ifndef __MODEL_H__
#define __MODEL_H__

typedef struct {
    WORD drel;
    WORD stacksize;
    WORD group;
    WORD arg;
    WORD control;
    DWORD wcet;
    DWORD res[MAX_RESOURCE];
    DWORD *exect;
} MODEL;

/* Added support for task excution time monitoring; if the bit is set 	   */
/* task execution will be monitored to obtain the wcet (to be implemented) */

#define USE_FPU         0x0001
#define MONITOR_WCET    0x0002
#define TRACE_TASK	0x0004

/* Some macros to set various task-model parameters */
#define BASE_MODEL              {0,0,0,0,0,0,{0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},NULL}
#define task_def_arg(m,a)       (m.arg = a)
#define task_def_stack(m,s)     (m.stacksize = s)
#define task_def_group(m,g)     (m.group = g)
#define task_def_wcet(m,w)      (m.wcet = w)
#define task_def_drel(m,d)      (m.drel = d)
#define task_use_res(m,r,b)     (m.res[r] = b)
#define task_use_math(m)        (m.control |= USE_FPU)
#define task_use_monitor(m)     (m.control |= MONITOR_WCET)
#define task_use_trace(m)       (m.control |= TRACE_TASK)

#endif /* __MODEL_H__ */

