/* Project:     HARTIK 3.0                                      */

/* Description: Hard Real TIme Kernel for 8086 compatible       */

/* Author:      Luca Abeni					*/

/* Date:        25/9/96						*/



/* File:        UDPIP.h                                         */

/* Revision:    4.0						*/



/* First, include the net general definitions */

#include <net.h>



#ifndef __UDPIP_H__

#define __UDPIP_H__



#ifdef __cplusplus

extern "C" {

#endif



#define net_setudpip(m,localaddr) {net_setprotocol(&m, ip_init, localaddr); net_setprotocol(&m, udp_init, NULL);}



typedef struct ip_addr{

    BYTE ad[4];

} IP_ADDR;



#define IP_ERROR_BASE	(ETH_ERROR_BASE + 20)

#define IP_INIT_ERROR 	(0 + IP_ERROR_BASE)



#define IP_UDP_TYPE     17

#define IP_TCP_TYPE     6



#define setIpAddr(q,w) memcpy(q.ad,w.ad,sizeof(IP_ADDR))

#define getVers(b)      ((b) >> 4)

#define getHlen(b)      (b & 0x0f)

#define getFlags(w)     ((w) >> 13)

#define getFrOffset(w)  (w & 0x1fff)



typedef struct ip_header {

    BYTE vers_hlen;

    BYTE servType;

    WORD lenght;

    WORD ident;

    WORD flags_frOffset;

    BYTE ttl;

    BYTE protocol;

    WORD headChecksum;

    IP_ADDR source;

    IP_ADDR dest;

} IP_HEADER;



#define IP_MAX_ENTRIES 2

typedef struct ip_table{

    BYTE protocol;

    void (*rfun)(void *m);

} IP_TABLE;



void ip_print_header(IP_HEADER *p);

int ip_compAddr(IP_ADDR ip1, IP_ADDR ip2);

int ip_str2addr(char *a, IP_ADDR *ip);

void *ip_getFDB(void *pkt);

int ip_setProtocol(BYTE p, void (*recv)(void *m));

void ip_send(IP_ADDR dest, void *pkt, WORD len);

void ip_init(void *la);

IP_ADDR *ip_getAddr();





/***************************************************************************/

/*									   */

/*				UDP					   */

/*									   */

/***************************************************************************/



#define UDP_ERROR_BASE	165

#define UDP_BADCHK_EXC	(0 + UDP_ERROR_BASE)



typedef struct udp_addr{

    IP_ADDR s_addr;

    WORD s_port;

} UDP_ADDR;





typedef struct udp_header{

    WORD s_port;

    WORD d_port;

    WORD mlen;

    WORD checksum;

} UDP_HEADER;



typedef struct pseudo_hd {

    IP_ADDR source;

    IP_ADDR dest;

    BYTE zero;

    BYTE protocoll;

    WORD len;

} PSEUDO_HD;



typedef struct udp_msg{

    WORD mlen;

    UDP_ADDR addr;

    void *buff;

} UDP_MSG;



typedef struct udp_table{

    WORD port;

    BYTE valid;

    PORT hport;

    PORT pport;

    BYTE notify;

    int (*notify_fun)(int l, BYTE *buff, void *p);

    void *notify_par;

} UDP_TABLE;



void udp_init(void *dummy);

int udp_recvfrom(int s, char *buff, UDP_ADDR *from);

int udp_sendto(int s, char *buff, int nbytes, UDP_ADDR *to);

int udp_bind(UDP_ADDR *a, IP_ADDR *bindlist);

int udp_notify(int s, int (*f)(int len, BYTE *buff, void *p), void *p);



#ifdef __cplusplus

};

#endif



#endif

