/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/

/* File:	Hw386.H						*/
/* Revision:	1.0						*/

/* Specifications for all 386 hardware stuff			*/

#ifndef __HW386_H__

/* Include X Selector Index */

#include "sel.h"

#define __HW386_H__

#ifdef __cplusplus
extern "C" {
#endif

/* Useful basic types */

#ifndef __BASIC_DATA__
    #define __BASIC_DATA__
    typedef unsigned long  LIN_ADDR;
    typedef unsigned long  DWORD;
    typedef unsigned short WORD;
    typedef unsigned char  BYTE;
    typedef unsigned long  TIME;
    typedef unsigned long  SYS_FLAGS;
    #define TRUE	1
    #define FALSE	0
    #define MAX_DWORD	0xFFFFFFFF
    #define MAX_WORD	0xFFFF
    #define MAX_BYTE	0xFF
#endif

/* Irq services specifications */

#define TIMER_IRQ	0
#define	KEYB_IRQ	1
#define COM2_IRQ	3
#define COM1_IRQ	4
#define COM4_IRQ	3
#define COM3_IRQ	4
#define SB_IRQ		5
#define FDC_IRQ		6
#define SB2_IRQ		7
#define RTC_IRQ		8
#define PS2MOUSE_IRQ	12
#define COPROC_IRQ	13
#define IDE0_IRQ	14
#define IDE1_IRQ	15

/* Hardware based types (Self explanatory) */

#pragma pack(1);

typedef struct gate {
    WORD offset_lo;
    WORD sel;
    BYTE dword_cnt;
    BYTE access;
    WORD offset_hi;
} GATE;

typedef struct descriptor {
    WORD lim_lo;
    WORD base_lo;
    BYTE base_med;
    BYTE access;
    BYTE gran;
    BYTE base_hi;
} DESCRIPTOR;

/* A LDT/GDT entry could be a gate or a selector */
/* An IDT entry could be a gate only	     	 */

union gdt_entry {
    DESCRIPTOR d;
    GATE g;
};

typedef union x_regs16 {
    struct {
	WORD ax;
	WORD bx;
	WORD cx;
	WORD dx;
	WORD si;
	WORD di;
	WORD cflag;
    } x; 
    struct {
	BYTE al,ah;
	BYTE bl,bh;
	BYTE cl,ch;
	BYTE dl,dh;
    } h;
} X_REGS16;

typedef struct x_sregs16 {
	WORD es, cs, ss, ds;
} X_SREGS16;

#define STACK_ACCESS	0x92	/* Basic Access bytes */
#define DATA_ACCESS	0x92
#define CODE_ACCESS	0x9A

/* At this level we just need to set up 2 gates to enter/exit PM */
/* The entry gate is a 386 32 bit call gate			 */
/* The exit (Back To Real Mode) gate is a 286 16 bit gate	 */

#define CALL_GATE286	0x84	/* Call & Int Gate Access bytes */
#define CALL_GATE386	0x8C	
#define TASK_GATE	0x85
#define INT_GATE286	0x86
#define INT_GATE386	0x8E
#define TRAP_GATE286	0x87
#define TRAP_GATE386	0x8F

/* TSS selectors */

#define FREE_TSS386	0x89
#define BUSY_TSS386	0x8B
#define FREE_TSS286	0x81
#define BUSY_TSS286	0x83

#define GRAN_32B	0xC0	/* Granularity settings */
#define GRAN_32		0x40
#define GRAN_16		0x00

#define MAX_ARGS	10
struct info {   
    DWORD dos_base;		/* These are the basic memory info 	*/
    DWORD dos_size;
    DWORD extended_base;
    DWORD extended_size;
    DWORD exec_base;
    DWORD exec_offset;
    DWORD GDT_base;
    DWORD IDT_base;
    X_REGS16 ir;		/* These are used for BIOS calling 	*/
    X_REGS16 or;
    X_SREGS16 sr;
    BYTE irqno;
    WORD ver;			/* This is the X version 		*/
    WORD argc;			/* These are used to pass parms		*/
    char argv[MAX_ARGS][80];
};

/* This is the TSS image for a 386 hardware task 			*/
/* I added two other fields to the basic structure:			*/
/* 1) The CONTROL field which is used by system software to detect	*/
/*    particular conditions; in this the first phase it is mainly used  */
/*    to mark the unused TSS & TSS which use math, altough thanks to 	*/
/*    the automatic FPU preemption supported in 386 this would be not   */
/*    necessary.							*/
/* 2) The ctx_FPU field used to store the FPU context if necessaary	*/

#define TSS_USED		0x8000
#define FPU_CONTEXT_SIZE	108

typedef short CONTEXT;

typedef struct tss {
    WORD back_link;
    WORD _fill0;
    DWORD esp0;
    WORD ss0;
    WORD _fill1;
    DWORD esp1;
    WORD ss1;
    WORD _fill2;
    DWORD esp2;
    WORD ss2;
    WORD _fill3;
    DWORD cr3;
    DWORD eip;
    DWORD eflags;
    DWORD eax;
    DWORD ecx;
    DWORD edx;
    DWORD ebx;
    DWORD esp;
    DWORD ebp;
    DWORD esi;
    DWORD edi;
    WORD es;
    WORD _fill5;
    WORD cs;
    WORD _fill6;
    WORD ss;
    WORD _fill7;
    WORD ds;
    WORD _fill8;
    WORD fs;
    WORD _fill9;
    WORD gs;
    WORD _fill10;
    WORD ldt;
    WORD _fill11;
    WORD trap;
    WORD io_base;
    DWORD control;
    BYTE ctx_FPU[FPU_CONTEXT_SIZE];
} TSS;

/* Here are some useful macros 		*/
/* These are all Watcom dependent 	*/

WORD get_CS(void);
#pragma aux get_CS = \
	"mov ax,cs" \
	value [ax];

WORD get_DS(void);
#pragma aux get_DS = \
	"mov ax,ds" \
	value [ax];

WORD get_TR(void);
#pragma aux get_TR = \
	".386P"  \
	"str ax" \
	value [ax];

/* Clear TaskSwitched Flag! Used for FPU preemption */
void clts(void);
#pragma aux clts = "clts";

/* Hlt Instruction */
void hlt(void);
#pragma aux hlt = "hlt";

/*
    The 32 bit port access function used in Hartik
    cannot be accessible when compiling X! As X uses hw386.h
    to access the 32 Bit PM structures, We isolated them with
    #ifdef ... #endif
*/
#ifndef __XTN__
    DWORD get_SP(void);
    #pragma aux get_SP = \
	    ".386P" \
	    "mov eax,esp" \
	    value [eax];
	    
    unsigned char inp(unsigned port);
    #pragma aux inp = \
	    "in al,dx" parm [edx] value [al];
	    
    void outp(unsigned port,unsigned char value);
    #pragma aux outp = \
	    "out dx,al" parm [edx] [al];
	    
    unsigned short inpw(unsigned short port);
    #pragma aux inpw = \
	    "in ax,dx" parm [edx] value [ax];
	    
    void outpw(unsigned short port,unsigned short value);
    #pragma aux outpw = \
	    "out dx,ax" parm [edx] [ax];
    
    unsigned long inpd(unsigned port);
    #pragma aux inpd = \
	    "in eax,dx" parm [edx] value [eax];
	    
    void outpd(unsigned port,unsigned long value);
    #pragma aux outpd = \
	    "out dx,eax" parm [edx] [eax];

    #ifdef __USE_SELECTOR__
    void lmempokeb(unsigned long a, unsigned char v);
    #pragma aux lmempokeb = \
    	"mov eax,038H" \
	"mov fs,ax" \
	"mov byte ptr fs:[edi],dl"\
	parm [edi] [edx];
    void lmempokew(unsigned long a, unsigned short v);
    #pragma aux lmempokew = \    	
    	"mov eax,038H" \
	"mov fs,ax" \
	"mov word ptr fs:[edi],dx"\
	parm [edi] [edx];
    void lmempoked(unsigned long a, unsigned long v);
    #pragma aux lmempoked = \    	
    	"mov eax,038H" \
	"mov fs,ax" \
	"mov dword ptr fs:[edi],edx"\
	parm [edi] [edx];
    unsigned char lmempeekb(unsigned long a);
    #pragma aux lmempeekb = \
    	"mov eax,038H" \
	"mov fs,ax" \
	"mov al,byte ptr fs:[esi]" \
	parm [esi] value [al];
    unsigned short lmempeekw(unsigned long a);
    #pragma aux lmempeekw = \    	
    	"mov eax,038H" \
	"mov fs,ax" \
	"mov ax,word ptr fs:[esi]" \	
	parm [esi] value [ax]; 
    unsigned long lmempeekd(unsigned long a);
    #pragma aux lmempeekd = \    	
    	"mov eax,038H" \
	"mov fs,ax" \
	"mov eax,dword ptr fs:[esi]" \
	parm [esi] value [eax];
    #else /* __USE_SELCTOR__ */
    void lmempokeb(unsigned long a, unsigned char v);
    #pragma aux lmempokeb = \
	"mov byte ptr fs:[edi],dl"\
	parm [edi] [edx];
    void lmempokew(unsigned long a, unsigned short v);
    #pragma aux lmempokew = \    	
	"mov word ptr fs:[edi],dx"\
	parm [edi] [edx];
    void lmempoked(unsigned long a, unsigned long v);
    #pragma aux lmempoked = \    	
	"mov dword ptr fs:[edi],edx"\
	parm [edi] [edx];
    unsigned char lmempeekb(unsigned long a);
    #pragma aux lmempeekb = \
	"mov al,byte ptr fs:[esi]" \
	parm [esi] value [al];
    unsigned short lmempeekw(unsigned long a);
    #pragma aux lmempeekw = \    	
	"mov ax,word ptr fs:[esi]" \	
	parm [esi] value [ax]; 
    unsigned long lmempeekd(unsigned long a);
    #pragma aux lmempeekd = \    	
	"mov eax,dword ptr fs:[esi]" \
	parm [esi] value [eax];
    #endif /* __USE_SELECTOR__ */

#define VM_in(p)	inp(p)
#define VM_out(p,v)	outp(p,v)
#define VM_inw(p)	inpw(p)
#define VM_outw(p,v)	outpw(p,v)
#define VM_ind(p)	inpd(p)
#define VM_outd(p,v)	outpd(p,v)

/* These functions are used to mask/unmask interrupts 		*/
/* Warning #PRAGMA using is quite peculiar for Watcom C  	*/
/* If you use another compiler you probaly have to modify this! */
void VM_enable(void);
#pragma aux VM_enable = "sti";

void VM_disable(void);
#pragma aux VM_disable = "cli";

SYS_FLAGS VM_save(void);
#pragma aux VM_save = "pushfd" \
		     "cli" \
		     "pop eax" \
		     value [eax];

void VM_restore(SYS_FLAGS f);
#pragma aux VM_restore = "push eax" \
			"popfd" \
			parm [eax];

/* This functions exports FPU management to upper level, i.e. the KERN  */
/* layer; this is necessary as we need to make some real number         */
/* calculation into the kernel; to avoid unnecessary FPU context saves  */
/* the VM_save/restore_FPU is used every time a primitive need to make  */
/* a math operation.                                                    */

extern BYTE VM_FPU_savearea[]; /* Global FPU scratch SaveArea */

void VM_FPU_reset(void);
#pragma aux VM_FPU_reset = "fninit";

void VM_FPU_save(void);
#pragma aux VM_FPU_save = "fsave VM_FPU_savearea";

void VM_FPU_restore(void);
#pragma aux VM_FPU_restore = "frstor VM_FPU_savearea";

/* This function returns 1 if the FPU is on the chip, 0 if the FPU   */
/* externally mounted. This is accomplished trying to change the ET  */
/* bit into the MSW register; if the bit can be changed then the FPU */
/* is external to the chip.                                          */

int VM_FPU_check(void);
#pragma aux VM_FPU_check = \
	"push	ebx" \
	"mov	eax,cr0" \
	"mov	ebx,eax" \
	"and	eax,0FFFFFFEFH" \
	"mov	cr0,eax" \
	"mov	eax,cr0" \
	"xchg	eax,ebx" \
	"mov	cr0,eax" \
	"xor	eax,eax" \
	"mov	al,bl" \
	"shr	al,4" \
	"pop	ebx" \
	value [eax];

/* Set up FPU control activating NE bit for 486 machines and activating */
/* MP bit to perform automatic FPU preemption                           */

void VM_FPU_init(void);
#pragma aux VM_FPU_init = \
	"mov	eax,cr0" \
	"or	eax,34" \
	"mov	cr0,eax" \
	"fninit";

#endif /* !__XTN__ */

/* Standard pointer to irq function prototype */
typedef void (__cdecl * INTERRUPT)(void);

/* Any Kernel primitive is declared with the CDECL modifier 		*/
/* This is useful to add special purposes meaning to the function 	*/
/* defclaration								*/

#define CDECL

#pragma pack();

#ifdef __cplusplus
};
#endif

#endif
