/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/

/* File:	Hw386GNU.H					*/
/* Revision:	1.0						*/

/* GNU-C COFF support */

/* Specifications for all 386 hardware stuff under GNU/DJGPP	*/

#ifndef __HW386GNU_H__
#define __HW386GNU_H__

#ifdef __cplusplus
extern "C" {
#endif    

/*
    Far pointer simulation; because of difference between V1.xx & V2.0
    we need to include the correct stub here!! I Hope it works...
*/

/* __DJGPP__ should be 2 for DJGPP 2.xx and 1 for DJGPP 1.xx */

#ifdef __DJGPP__
    #if __DJGPP__ >= 2
    #include "farptr.h"
    #else
    #include "_farptr.h"
    #endif
#else
#ifdef __LINUX__
    /* Need to #undef these, otherwise we loose farns*() functions */
    #undef __dj_ENFORCE_ANSI_FREESTANDING
    #undef __STRICT_ANSI__
    #undef _POSIX_SOURCE
    #include "farptr.h"
#else
    #include "_farptr.h"
#endif
#endif

#include "sel.h"

/* Useful basic types */

#ifndef __BASIC_DATA__
    #define __BASIC_DATA__
    typedef unsigned long  LIN_ADDR;
    typedef unsigned long  DWORD;
    typedef unsigned short WORD;
    typedef unsigned char  BYTE;
    typedef unsigned long  TIME;
    typedef unsigned long  SYS_FLAGS;
    #define TRUE	1
    #define FALSE	0
    #define MAX_DWORD	0xFFFFFFFF
    #define MAX_WORD	0xFFFF
    #define MAX_BYTE	0xFF
#endif

typedef short CONTEXT;

/* Hardware based types (Self explanatory) */

typedef struct gate {
    WORD offset_lo __attribute__ ((packed));
    WORD sel __attribute__ ((packed));
    BYTE dword_cnt __attribute__ ((packed));
    BYTE access __attribute__ ((packed));
    WORD offset_hi __attribute__ ((packed));
} GATE;

typedef struct descriptor {
    WORD lim_lo __attribute__ ((packed));
    WORD base_lo __attribute__ ((packed));
    BYTE base_med __attribute__ ((packed));
    BYTE access __attribute__ ((packed));
    BYTE gran __attribute__ ((packed));
    BYTE base_hi __attribute__ ((packed));
} DESCRIPTOR;

/* A LDT/GDT entry could be a gate or a selector */
/* An IDT entry could be a gate only	     	 */

union gdt_entry {
    DESCRIPTOR d __attribute__ ((packed));
    GATE g __attribute__ ((packed));
};

typedef union x_regs16 {
    struct {
	WORD ax __attribute__ ((packed));
	WORD bx __attribute__ ((packed));
	WORD cx __attribute__ ((packed));
	WORD dx __attribute__ ((packed));
	WORD si __attribute__ ((packed));
	WORD di __attribute__ ((packed));
	WORD cflag __attribute__ ((packed));
    } x __attribute__ ((packed));
    struct {
	BYTE al,ah __attribute__ ((packed));
	BYTE bl,bh __attribute__ ((packed));
	BYTE cl,ch __attribute__ ((packed));
	BYTE dl,dh __attribute__ ((packed));
    } h __attribute__ ((packed));
} X_REGS16;

typedef struct x_sregs16 {
	WORD es __attribute__ ((packed));
	WORD cs __attribute__ ((packed));
	WORD ss __attribute__ ((packed));
	WORD ds __attribute__ ((packed));
} X_SREGS16;

#define STACK_ACCESS	0x92	/* Basic Access bytes */
#define DATA_ACCESS	0x92
#define CODE_ACCESS	0x9A

/* At this level we just need to set up 2 gates to enter/exit PM */
/* The entry gate is a 386 32 bit call gate			 */
/* The exit (Back To Real Mode) gate is a 286 16 bit gate	 */

#define CALL_GATE286	0x84	/* Call & Int Gate Access bytes */
#define CALL_GATE386	0x8C	
#define TASK_GATE	0x85
#define INT_GATE286	0x86
#define INT_GATE386	0x8E
#define TRAP_GATE286	0x87
#define TRAP_GATE386	0x8F

/* TSS selectors */

#define FREE_TSS386	0x89
#define BUSY_TSS386	0x8B
#define FREE_TSS286	0x81
#define BUSY_TSS286	0x83

#define GRAN_32B	0xC0	/* Granularity settings */
#define GRAN_32		0x40
#define GRAN_16		0x00

/* Here are some useful macros */

inline static WORD get_CS(void)  {WORD r; __asm__ __volatile__ ("movw %%cs,%0" : "=q" (r)); return(r);}
inline static WORD get_DS(void)  {WORD r; __asm__ __volatile__ ("movw %%ds,%0" : "=q" (r)); return(r);}
inline static DWORD get_SP(void) {DWORD r; __asm__ __volatile__ ("movw %%esp,%0" : "=q" (r)); return(r);}
inline static DWORD get_TR(void) {WORD r; __asm__ __volatile__ ("strw %0" : "=q" (r)); return(r); }
inline static void clts(void) {__asm__ __volatile__ ("clts"); }
inline static void hlt(void) {__asm__ __volatile__ ("hlt"); }

#define MAX_ARGS	10
struct info {   
    /* These are the basic memory info */
    DWORD dos_base  __attribute__ ((packed));
    DWORD dos_size __attribute__ ((packed));
    DWORD extended_base __attribute__ ((packed));
    DWORD extended_size __attribute__ ((packed));
    DWORD exec_base __attribute__ ((packed));
    DWORD exec_offset __attribute__ ((packed));
    DWORD GDT_base __attribute__ ((packed));
    DWORD IDT_base __attribute__ ((packed));
    /* These are used for BIOS calling */
    X_REGS16 ir __attribute__ ((packed));
    X_REGS16 or __attribute__ ((packed));
    X_SREGS16 sr __attribute__ ((packed));
    BYTE irqno __attribute__ ((packed));
    /* This is the X version 	*/
    WORD ver __attribute__ ((packed));
    /* These are used to pass parms	*/
    WORD argc __attribute__ ((packed));
    char argv[MAX_ARGS][80] __attribute__ ((packed));
};

/* This is the TSS image for a 386 hardware task 			*/
/* I added two other fields to the basic structure:			*/
/* 1) The CONTROL field which is used by system software to detect	*/
/*    particular conditions; in this the first phase it is mainly used  */
/*    to mark the unused TSS & TSS which use math, altough thanks to 	*/
/*    the automatic FPU preemption supported in 386 this would be not   */
/*    necessary.							*/
/* 2) The ctx_FPU field used to store the FPU context if necessaary	*/

#define TSS_USED		0x8000
#define FPU_CONTEXT_SIZE	108

typedef struct tss {
    WORD back_link __attribute__ ((packed));
    WORD _fill0 __attribute__ ((packed));
    DWORD esp0 __attribute__ ((packed));
    WORD ss0 __attribute__ ((packed));
    WORD _fill1 __attribute__ ((packed));
    DWORD esp1 __attribute__ ((packed));
    WORD ss1 __attribute__ ((packed));
    WORD _fill2 __attribute__ ((packed));
    DWORD esp2 __attribute__ ((packed));
    WORD ss2 __attribute__ ((packed));
    WORD _fill3 __attribute__ ((packed));
    DWORD cr3 __attribute__ ((packed));
    DWORD eip __attribute__ ((packed));
    DWORD eflags __attribute__ ((packed));
    DWORD eax __attribute__ ((packed));
    DWORD ecx __attribute__ ((packed));
    DWORD edx __attribute__ ((packed));
    DWORD ebx __attribute__ ((packed));
    DWORD esp __attribute__ ((packed));
    DWORD ebp __attribute__ ((packed));
    DWORD esi __attribute__ ((packed));
    DWORD edi __attribute__ ((packed));
    WORD es __attribute__ ((packed));
    WORD _fill5 __attribute__ ((packed));
    WORD cs __attribute__ ((packed));
    WORD _fill6 __attribute__ ((packed));
    WORD ss __attribute__ ((packed));
    WORD _fill7 __attribute__ ((packed));
    WORD ds __attribute__ ((packed));
    WORD _fill8 __attribute__ ((packed));
    WORD fs __attribute__ ((packed));
    WORD _fill9 __attribute__ ((packed));
    WORD gs __attribute__ ((packed));
    WORD _fill10 __attribute__ ((packed));
    WORD ldt __attribute__ ((packed));
    WORD _fill11 __attribute__ ((packed));
    WORD trap __attribute__ ((packed));
    WORD io_base __attribute__ ((packed));
    DWORD control __attribute__ ((packed));
    BYTE ctx_FPU[FPU_CONTEXT_SIZE] __attribute__ ((packed));
} TSS;

/*
    The code for inp, outp, inpw, outpw, inpd & outpd is from
    standard GNU C distribution!
    The full source code for the GNU-C distribution is available
    from www.delorie.com

    The following code is ... 
    copyright (C) 1995 DJ Delorie, see COPYING.DJ for details
*/

extern __inline__ unsigned char inp(unsigned short _port)
{
    unsigned char rv;
    __asm__ __volatile__ ("inb %1, %0"
	  : "=a" (rv)
	  : "d" (_port));
    return(rv);
}

extern __inline__ unsigned short inpw (unsigned short _port)
{
    unsigned short rv;
    __asm__ __volatile__ ("inw %1, %0"
	  : "=a" (rv)
	  : "d" (_port));
    return(rv);
}

extern __inline__ unsigned long inpd(unsigned short _port)
{
    unsigned long rv;
    __asm__ __volatile__ ("inl %1, %0"
	  : "=a" (rv)
	  : "d" (_port));
    return(rv);
}

extern __inline__ void outp(unsigned short _port, unsigned char _data)
{
    __asm__ __volatile__ ("outb %1, %0"
	  :
	  : "d" (_port),
	    "a" (_data));
}

extern __inline__ void outpw(unsigned short _port, unsigned short _data)
{
    __asm__ __volatile__ ("outw %1, %0"
	  :
	  : "d" (_port),
	    "a" (_data));
}

extern __inline__ void outpd(unsigned short _port, unsigned long _data)
{
    __asm__ __volatile__ ("outl %1, %0"
	  :
	  : "d" (_port),
	    "a" (_data));
}

/*
    These routines are based on the farptr.h or _farptr.h
    (depending on DJGPP version!); they use the FS register
    to access to linear memory space. So I assume that FS
    is loaded with X_FLATDATA_SEL, as it happens at startup;
    I use the _farns*() functions which are faster!
    If you need to use FS, then #define __USE_SELECTOR__!!
*/

#ifdef __USE_SELECTOR__
    #define lmempokeb(a,v) 	_farpokeb(X_FLATDATA_SEL,a,v)
    #define lmempokew(a,v) 	_farpokew(X_FLATDATA_SEL,a,v)
    #define lmempoked(a,v) 	_farpokel(X_FLATDATA_SEL,a,v)
    #define lmempeekb(a) 	_farpeekb(X_FLATDATA_SEL,a)
    #define lmempeekw(a) 	_farpeekw(X_FLATDATA_SEL,a)
    #define lmempeekd(a) 	_farpeekl(X_FLATDATA_SEL,a)
#else
    #define lmempokeb(a,v) 	_farnspokeb(a,v)
    #define lmempokew(a,v) 	_farnspokew(a,v)
    #define lmempoked(a,v) 	_farnspokel(a,v)
    #define lmempeekb(a) 	_farnspeekb(a)
    #define lmempeekw(a) 	_farnspeekw(a)
    #define lmempeekd(a) 	_farnspeekl(a)
#endif

/*
    Well, these are simple macros... to map the HARTIK names
    onto the standard names!
 */
#define VM_in(port)	inp(port)
#define VM_out(port,v)	outp(port,v)
#define VM_inw(port)	inpw(port)
#define VM_outw(port,v)	outpw(port,v)
#define VM_ind(port)	inpd(port)
#define VM_outd(port,v)	outpd(port,v)

/* These functions are used to mask/unmask interrupts 		*/
/* Warning #PRAGMA using is quite peculiar for Watcom C  	*/
/* If you use another compiler you probaly have to modify this! */
inline static void enable(void) {__asm__ __volatile__ ("sti"); }
inline static void disable(void) {__asm__ __volatile__ ("cli"); }

#define VM_enable() 	enable()
#define VM_disable()	disable()

extern __inline__ SYS_FLAGS VM_save(void) {
    SYS_FLAGS result;
    __asm__ __volatile__ ("pushfl");
    __asm__ __volatile__ ("cli");
    __asm__ __volatile__ ("popl %eax");
    __asm__ __volatile__ ("movl %%eax,%0"
    	: "=r" (result)
	:
	: "eax" );
    return(result);
}

extern __inline__ void VM_restore(SYS_FLAGS f) {
    __asm__ __volatile__ ("mov %0,%%eax"
    	: 
    	: "r" (f)
	: "eax");
    __asm__ __volatile__ ("pushl %eax");
    __asm__ __volatile__ ("popfl");
}

/*
    FPU context switch management functions!
    FPU management exported at kernel layer to allow the use
    of floating point in kernel primitives; this turns to be
    useful for bandwidth reservation or guarantee!
*/

extern BYTE VM_FPU_savearea[];

extern __inline__ void VM_FPU_reset(void)
{
    __asm__ __volatile__ ("fninit");
}

extern __inline__ void VM_FPU_save(void)
{
    #ifdef __LINUX__
	__asm__ __volatile__ ("fsave VM_FPU_savearea");
    #else
	__asm__ __volatile__ ("fsave _VM_FPU_savearea");
    #endif
}

extern __inline__ void VM_FPU_restore(void)
{
    #ifdef __LINUX__
	__asm__ __volatile__ ("frstor VM_FPU_savearea");
    #else
	__asm__ __volatile__ ("frstor _VM_FPU_savearea");
    #endif
}

extern __inline__ int VM_FPU_check(void)
{
    int result;
    
    __asm__ __volatile__ ("movl	%cr0,%eax");
    __asm__ __volatile__ ("movl	%eax,%edi");
    __asm__ __volatile__ ("andl	$0x0FFFFFFEF,%eax");
    __asm__ __volatile__ ("movl	%eax,%cr0");
    __asm__ __volatile__ ("movl	%cr0,%eax");
    __asm__ __volatile__ ("xchgl %edi,%eax");
    __asm__ __volatile__ ("movl	%eax,%cr0");
    __asm__ __volatile__ ("xorl	%eax,%eax");
    __asm__ __volatile__ ("movb	%bl,%al");
    __asm__ __volatile__ ("shrb	$4,%al");
    __asm__ __volatile__ ("movl %%eax,%0"
    	: "=r" (result)
	:
	: "eax" );
    return(result);
}

extern __inline__ void VM_FPU_init(void)
{
    __asm__ __volatile__ ("movl %cr0,%eax");
    __asm__ __volatile__ ("orl  $34,%eax");
    __asm__ __volatile__ ("movl %eax,%cr0");
    __asm__ __volatile__ ("fninit");
}

/* Irq services specifications */

#define TIMER_IRQ	0
#define	KEYB_IRQ	1
#define COM2_IRQ	3
#define COM1_IRQ	4
#define COM4_IRQ	3
#define COM3_IRQ	4
#define SB_IRQ		5
#define FDC_IRQ		6
#define SB2_IRQ		7
#define RTC_IRQ		8
#define PS2MOUSE_IRQ	12
#define COPROC_IRQ	13
#define IDE0_IRQ	14
#define IDE1_IRQ	15

/* Standard pointer to irq function prototype */
typedef void (__cdecl *INTERRUPT)(void);

/* Any Kernel primitive is declared with the CDECL modifier 		*/
/* This is useful to add special purposes meaning to the function 	*/
/* defclaration								*/

#define CDECL

/*
    For Linux-ELF support... Cannot hack with __cdecl...
*/
#ifdef __LINUX__
void _VM_abort(int code);
CONTEXT _VM_context_save(void);
void _VM_context_change(CONTEXT c);
void _VM_context_load(CONTEXT c);

///* Luca's try... */ double fmod(double x, double y) __attribute__((alias("_fmod")));


void VM_abort(int code) __attribute__((alias("_VM_abort")));
CONTEXT VM_context_save(void)  __attribute__((alias("_VM_context_save")));
void VM_context_change(CONTEXT c) __attribute__((alias("_VM_context_change")));
void VM_context_load(CONTEXT c) __attribute__((alias("_VM_context_load")));

#endif

#ifdef __cplusplus
};
#endif

#endif
