/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 8086 compatible	*/
/* Author:	Gerardo Lamastra				*/
/*		Giuseppe Lipari					*/
/* Date:	9/5/96 						*/

/* File: 	Hw86.H			*/
/* Revision:	1.0			*/

/* Hardware dependent declarations	*/

/* Some useful basic definitions */

#ifndef __HW86_H__
#define __HW86_H__

#include <i86.h>

#ifdef __cplusplus
extern "C" {
#endif    

#ifndef __BASIC_DATA__
    #define __BASIC_DATA__
    typedef unsigned long  LIN_ADDR;
    typedef unsigned long  DWORD;
    typedef unsigned short WORD;
    typedef unsigned char  BYTE;
    typedef unsigned long  TIME;
    typedef unsigned short SYS_FLAGS;
    #define TRUE	1
    #define FALSE	0
    #define MAX_DWORD	0xFFFFFFFF
    #define MAX_WORD	0xFFFF
    #define MAX_BYTE	0xFF
#endif

/* This is a pointer to the block of memory where we can find the  */
/* context of a task. We don't know how the context is implemented */
/* inside the virtual machine, because the context is handled	   */
/* by the VM							   */

typedef int CONTEXT;

/* Irq services specifications */

#define TIMER_IRQ	0
#define	KEYB_IRQ	1
#define COM2_IRQ	3
#define COM1_IRQ	4
#define COM4_IRQ	3
#define COM3_IRQ	4
#define SB_IRQ		5
#define FDC_IRQ		6
#define SB2_IRQ		7
#define RTC_IRQ		8
#define PS2MOUSE_IRQ	12
#define COPROC_IRQ	13
#define IDE0_IRQ	14
#define IDE1_IRQ	15

/* Standard pointer to irq function prototype */
typedef void (__cdecl __interrupt * INTERRUPT)(void);

/* Here are some useful macros 		*/
/* These are all Watcom dependent 	*/

WORD get_CS(void);
#pragma aux get_CS = \
	"mov ax,cs" \
	value [ax];

WORD get_DS(void);
#pragma aux get_DS = \
	"mov ax,ds" \
	value [ax];

DWORD get_SP(void);
#pragma aux get_SP = \
	"mov ax,ss" \
	"mov dx,ax" \
	"mov ax,sp" \
	value [ax dx];

/* Hlt Instruction */
void hlt(void);
#pragma aux hlt = "hlt";

/* Port Access! */
unsigned inp(unsigned port);
#pragma aux inp = \
	"in al,dx" parm [dx] value [ax];
	
void outp(unsigned port,unsigned value);
#pragma aux outp = \
	"out dx,al" parm [dx] [ax];

unsigned inpw(unsigned port);
#pragma aux inpw = \
	"in ax,dx" parm [dx] value [ax];
	
void outpw(unsigned port,unsigned value);
#pragma aux outpw = \
	"out dx,ax" parm [dx] [ax];

#define lmempokeb(a,v) *((unsigned char __far *)(MK_FP((a & 0xFFFF0) >> 4,a & 0x0000F))) = (unsigned char)(v)
#define lmempokew(a,v) *((unsigned short __far *)(MK_FP((a & 0xFFFF0) >> 4,a & 0x0000F))) = (unsigned short)(v)
#define lmempeekb(a) *((unsigned char __far *)(MK_FP((a & 0xFFFF0) >> 4,a & 0x0000F)))
#define lmempeekw(a) *((unsigned short __far *)(MK_FP((a & 0xFFFF0) >> 4,a & 0x0000F)))

/*
    This is a dirty joke... Just to compile PCI device support
    under 16 bit systems. They do not support PCI, however, so
    this is just to do testing...
    They are defined externally in the DEV\NET section...
*/

unsigned long int inpd(unsigned short int port);
void outpd(unsigned short int port, unsigned long intvalue);

#define VM_in(p)		inp(p)
#define VM_out(p,v)		outp(p,v)
#define VM_inw(p)		inpw(p)
#define VM_outw(p,v)	outpw(p,v)
#define VM_ind(p)		inpd(p)
#define VM_outd(p,v)	outpd(p,v)

/* These functions are used to mask/unmask interrupts 		*/
/* Warning #PRAGMA using is Watcom C Peculiar 			*/
/* If you use another compiler you probaly have to modify this! */
/* You may use the enable/disable if you are writing a global 	*/
/* primitive, else if the primitive can be nested, you better	*/
/* use save/restore!						*/

void VM_enable(void);
#pragma aux VM_enable = "sti";

void VM_disable(void);
#pragma aux VM_disable = "cli";

SYS_FLAGS VM_save(void);
#pragma aux VM_save = "pushf" \
		     "cli" \
		     "pop ax" \
		     value [ax];

void VM_restore(SYS_FLAGS f);
#pragma aux VM_restore = "push ax" \
			"popf" \
			parm [ax];

/*
   FPU Management macro functions! They are used to handle FPU
   context switch in 16 bit mode! We decided to use a VM standard
   function set to export explicit FPU management to kernel layer.
   This turns out to be useful if one need to perform some floating
   point calculation in kernel primitive (for example for guarantee
   or bandwidth reservation!).
   The correct way of using this is the following:
   primitive()
   {
       VM_disable() or VM_save or VM_context_save();
       VM_save_FPU();
        ... here we can use math ....
       VM_restore_FPU();
       VM_enable() or VM_restore or VM_context_change();
   }
*/

/* The FPU register context dimension */
#define FPU_CTX_SIZE    108
/* The VM_control flag used to mark FPU use */
#define CTX_BUSY        0x8000

/* This is declared in Kern\VM\WC16\VmFPU.C !! */
extern BYTE VM_FPU_savearea[FPU_CTX_SIZE];

void VM_FPU_save(void);
#pragma aux VM_FPU_save = \
	"mov	ax,seg VM_FPU_savearea" \
	"mov	ds,ax" \   
	"fsave	byte ptr ds:VM_FPU_savearea";

void VM_FPU_restore(void); 
#pragma aux VM_FPU_restore = \
	"mov	ax,seg VM_FPU_savearea" \
	"mov	ds,ax" \   
	"frstor	byte ptr ds:VM_FPU_savearea";
	
void VM_FPU_reset(void);
#pragma aux VM_FPU_reset = "fninit";

#ifdef __cplusplus
};
#endif

#endif
