/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	2.0						*/

/* File: Sel.H
   The Basic Selector defined in X are here!
*/

/* These are the selector used by the extender 			    */
/* Do not remove them from GDT or you won't be able to return to RM */

#define NULL_SEL		0x00
#define X_DATA_SEL		0x08
#define X_CODE16_SEL		0x10
#define X_CODE32_SEL		0x18
#define X_RM_BACK_GATE		0x20
#define X_PM_ENTRY_GATE		0x28
#define X_VIDEO_SEL		0x30
#define X_FLATDATA_SEL		0x38
#define X_FLATCODE_SEL		0x40

/* These selectors are set by the loader to make PM application run  */
/* Into its own address space					     */
/* Problem: Using Flat model requires that stack resides into the    */
/* data segment; then the stack selector & application data selector */
/* MUST be the same! If we do not make so, this will cause problem   */
/* when referencing pointers which are local variables and are then  */
/* allocated onto stack & global pointers which are into data 	     */
/* segment; if we assign local_ptr = global_ptr the assignment will  */
/* not take into account that SS != DS; this result in a big mess!   */

#define PM_APPLCODE_SEL		0x48
#define PM_APPLDATA_SEL		0x50

/* This structure is used to send info to the PM application     */
/* and receive back other info, it this is necessary.	         */
/* X will set up the memory it has found and which is available  */
/* for the PM application, and the list of the program arguments */
/* passed to the pm application!				 */
/* The effective structure is located into the X data segment;   */
/* In order to access it from PM easily we shall use a specific  */
/* selector.							 */
/* In the PM application we shall build a pointer to access it:  */
/* struct info *X_info = MK_FP(X_INFO_SEL,0);			 */
/* STRONGLY REMEMBER!						 */
/* Pay attention when declaring types into info structure; these */
/* can have different byte dimension under 16 bit and 32 bit 	 */
/* compiler; for example if you declare a field of type int it   */
/* will be 2 byte long in 16 bit & 4 byte long in 32 bit; this   */
/* will cause problems when dealing with the same object in the  */
/* different environment.					 */

#define X_INFO_SEL	0x58
#define X_CALLBIOS_GATE	0x60


