/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/

/* File: 	StdLib.H					*/
/* Revision:	1.0						*/

/* Standard library for X applications */

#ifndef __STDLIB_H__
#define __STDLIB_H__

#ifndef __XSYS_H__
#include <xsys.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif    

#ifndef NULL
    #define NULL 0L
#endif

/* Number limits */

#define INT_MAX    	2147483647
#define UINT_MAX  	4294967295U

#ifndef FLT_MIN
#define FLT_MIN 	1.175494351e-38f
#endif

#ifndef DBL_MIN
#define DBL_MIN 	2.22507385850720200e-308
#endif

#ifndef FLT_MAX
#define FLT_MAX 	3.402823466e+38f
#endif

#ifndef DBL_MAX
#define DBL_MAX 	1.79769313486231500e+308
#endif

#ifndef FLT_EPSILON
#define FLT_EPSILON	1.192092896e-7f
#endif

#ifndef DBL_EPSILON
#define DBL_EPSILON	2.22044604925031300e-016
#endif

/* Generic utility functions 	*/
/* File StdLib.C		*/

void srand(long int seed);
long int rand(void);
unsigned abs(int x);

/* String conversion functions */
char _toupper(char c);
char _tolower(char c);
int _tonumber(char c);
char _todigit(int c);
int _isnumber(char c,int base);
int _isspace(char c);
int _isspecial(double d,char *bufp);

/* The modf function, used to split a real into integer & mantissa */
/* Strange to say, it is not contained into the libm.a or math.lib */

#ifndef __LINUX__
double modf (double x, double *iptr);
#else
double _modf (double x, double *iptr);
double modf (double x, double *iptr) __attribute__ ((weak,alias("_modf")));
#endif

/* Stdlib Macro */
#ifndef __WC16__
#define labs(x)		abs(x)
#endif

#if !defined(__max)
#define __max(a,b)  (((a) > (b)) ? (a) : (b))
#endif
#if !defined(max) && !defined(__cplusplus)
#define max(a,b)  (((a) > (b)) ? (a) : (b))
#endif
#if !defined(__min)
#define __min(a,b)  (((a) < (b)) ? (a) : (b))
#endif
#if !defined(min) && !defined(__cplusplus)
#define min(a,b)  (((a) < (b)) ? (a) : (b))
#endif

#ifdef __cplusplus
};
#endif

#endif
