/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/* Date:        1/10/96                                         */

/* File:        KBuild.C                                        */
/* Revision:    1.0                                             */

/* This is an automatic builder for the kernel libraries	*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h>
#include <ctype.h>
#include <graph.h>
#include <direct.h>
#include <sys\types.h>

char *yesno[] = {"No","Yes"};
unsigned version = 0x0100;
char header_path[80],distr_path[80],actual_path[80],hartik_version[20];

/* Support */

unsigned port_support = 0;
unsigned cab_support = 0;
unsigned srp_support = 0;
unsigned trace_support = 0;
unsigned keyb_support = 0;
unsigned crtwin_support = 0;
unsigned com_support = 0;
unsigned smouse_support = 0;

/* Input/Output makefiles */

FILE *file_orig_make,*file_make;
char data[1024];

/* Architecture dependent definitions */

#define ARCH_MAX	3

unsigned arch;

char *arch_makefile[] = {
    "makefile.w16",
    "makefile.w32",
    "makefile.gnu"
};

char *arch_opts[] = {
    "Watcom C (16 bit)",
    "Watcom C (32 bit)",
    "GNU    C (32 Bit)"
};

char *arch_setting[] = {"16","32","32G"};

char *arch_make[] = {"Wmake distrib","Wmake distrib","Make distrib"};

unsigned yes(void)
{
    char c = 0;
    while (c != 'Y' && c != 'N') c = toupper(getch());
    if (c == 'Y') printf("Yes\n");
    else printf("No\n");
    return (c == 'Y' ? 1 : 0);
}

void wanna_support(char *s,unsigned *flag)
{
    printf("%s support (Y/N) ? ",s);
    flushall();
    if (yes()) *flag = 1;
    else *flag = 0;
}

unsigned select(char *s,char *opts[],unsigned m)
{
    register unsigned i;
    char c = 0xFF;

    printf("%s\n\n",s);
    for (i = 0; i < m; i++) printf("[%u] %s\n",i,opts[i]);
    putchar('\n');
    while (c >= m) c = (getch() - '0');
    return((unsigned)(c));
}

void copy(char *s,char *d)
{
    char command[80];

    sprintf(command,"copy %s %s",s,d);
    system(command);
}

void add_header(FILE *dst,char *header)
{
    FILE *src;
    unsigned status,command,index;
    char *result;

    if ((src = fopen(header,"rt")) == NULL) {
	printf("Error! File %s not found\n",header);
	exit(-1);
    }
    status = command = 0;
    
    while ((result = fgets(data,1024,src)) != NULL) {
	command = 0;
	/* Discard space */
	for (index = 0; index < 80 && (data[index] == ' ' || data[index] == '\t'); index++);
	/* If we have found a comment start-up */
	if (data[index] == '/' && data[index+1] == '*') {
	    index += 2;
	    /* Discard space */
	    for (;  index < 80 && (data[index] == ' ' || data[index] == '\t'); index++);
	    /* If we find the $HEADER string 		*/
	    /* Detect if it is a begin or an end! 	*/
	    if (strncmp(&data[index],"$HEADER+",8) == 0) {
		/* Begin */
		status = 1;
		command = 1;
	    }
	    else if (strncmp(&data[index],"$HEADER-",8) == 0) {
		/* End */
		status = 0;
		command = 1;
	    }
	}
	if (status && !command && result != NULL) fputs(data,dst);
    }
    fclose(src);
}

void build_hartik_h(void)
{
    FILE *dst;
    unsigned status,command;
    time_t time_of_day;

    dst = fopen("..\\H\\HARTIK.H","wt");
    time_of_day = time(NULL);
    
    fprintf(dst,"/* Project:     HARTIK 3.0                                      */\n");
    fprintf(dst,"/* Description: Hard Real TIme Kernel for 8086 compatible       */\n");
    fprintf(dst,"/* Authors:     Gerardo Lamastra (lamastra@sssup2.sssup.it)     */\n");
    fprintf(dst,"/*              Giuseppe Lipari  (lipari@sssup2.sssup.it)       */\n");
    fprintf(dst,"/* Date:        1/10/96                                         */\n\n");
    
    fprintf(dst,"/* DEC Alpha support by Antonino Casile                         */\n");
    fprintf(dst,"/* Project coordinated by Giorgio Buttazzo                      */\n");
    fprintf(dst,"\n");
    fprintf(dst,"/* Please DO NOT USE MicroSoft compilers with HARTIK+           */\n");
    fprintf(dst,"/* BOYCOTT B. G.!                                            */\n");
    
    fprintf(dst,"\n\n");

    fprintf(dst,"/*\n    Tech Report\n");
    fprintf(dst,"    ---- ------\n");
    fprintf(dst,"    Your kernel has been compiled including\n");
    if (port_support) fprintf(dst,"    Port  Support\n");
    if (cab_support) fprintf(dst,"    CAB   Support\n");
    if (srp_support) fprintf(dst,"    SRP   Support\n");
    if (trace_support) fprintf(dst,"    Trace Support\n");
    fprintf(dst,"\n    KBuild release is %x.%x\n",((version & 0xFF00) >> 8),(version & 0x0FF));
    fprintf(dst,"    Actual date is %25s*/\n",ctime(&time_of_day));
    fprintf(dst,"    Actual kernel version: %s\n\n",kernel_version);

    fprintf(dst,"#ifndef __HARTIK_H__\n");
    fprintf(dst,"#define __HARTIK_H__\n");
    fprintf(dst,"#ifdef  __cplusplus\n");
    fprintf(dst,"extern \"C\" {\n");

    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    fprintf(dst,"/*                   The Hardware Dependecies                   */\n");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    add_header(dst,"..\\H\\SYS\\HW_DEP.H");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    fprintf(dst,"/*                   The Basic system constant                  */\n");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    add_header(dst,"..\\H\\SYS\\CONST.H");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    fprintf(dst,"/*                       Exception index                        */\n");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    add_header(dst,"..\\H\\SYS\\EXC.H");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    fprintf(dst,"/*                        The Task model                        */\n");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    add_header(dst,"..\\H\\SYS\\MODEL.H");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    fprintf(dst,"/*                   General system primitives                  */\n");
    fprintf(dst,"/* ------------------------------------------------------------ */\n");
    add_header(dst,"..\\H\\SYS\\SYS.H");
    
    if (port_support) {
	fprintf(dst,"/* ------------------------------------------------------------ */\n");
	fprintf(dst,"/*                         Port Support                         */\n");
	fprintf(dst,"/* ------------------------------------------------------------ */\n");
	add_header(dst,"..\\H\\SYS\\PORT.H");
    }
    if (cab_support) {
	fprintf(dst,"/* ------------------------------------------------------------ */\n");
	fprintf(dst,"/*                         CAB Support                         */\n");
	fprintf(dst,"/* ------------------------------------------------------------ */\n");
	add_header(dst,"..\\H\\SYS\\CABS.H");
    }
    if (srp_support) {
	fprintf(dst,"/* ------------------------------------------------------------ */\n");
	fprintf(dst,"/*                         SRP Support                         */\n");
	fprintf(dst,"/* ------------------------------------------------------------ */\n");	
	add_header(dst,"..\\H\\SYS\\SRP.H");
    }
    if (trace_support) {
	fprintf(dst,"/* ------------------------------------------------------------ */\n");
	fprintf(dst,"/*                         Tracing Support                      */\n");
	fprintf(dst,"/* ------------------------------------------------------------ */\n");	
	add_header(dst,"..\\H\\SYS\\LOG.H");
    }
    fprintf(dst,"\n#ifdef __cplusplus\n");
    fprintf(dst,"};\n");
    fprintf(dst,"#endif\n");
    fprintf(dst,"\n#endif\n\n");
    fclose(dst);
}

void main(void)
{
    unsigned error;

    printf("\nKBuild -- The automatic HARTIK builder!\n");
    printf("Version %x.%x\n\n",((version & 0xFF00) >> 8),(version & 0x0FF));
    printf("Set the kernel version number [Base.Revision.BugFix]\n");
    scanf("%s",hartik_version);
    do {
	printf("Distribution path : ");
	flushall();
	scanf("%s",distr_path);
	printf("\nIs %s correct (Y/N) ? ",distr_path);
	flushall();
    } while (!yes());

    error = mkdir(distr_path);
    if (error) {
	printf("Error detected! Path %s already exists\n",distr_path);
	printf("Do you want to install anyway ? ");
	flushall();
	if (!yes()) {
	    printf("Installation aborted\n");
	    exit(-1);
	}
	else printf("Ok\n");
    }
    arch = select("Select architecture",arch_opts,ARCH_MAX);
    if ((file_orig_make = fopen(arch_makefile[arch],"rt")) == NULL) {
	printf("Error\nCannot find %s\n",arch_makefile[arch]);
	exit(-1);
    } else fclose(file_orig_make);

    printf("\nKernel auxiliary files to include\n");
    printf("---------------------------------\n");
    wanna_support("Port",&port_support);
    wanna_support("CAB",&cab_support);
    wanna_support("SRP",&srp_support);
    wanna_support("External tracing",&trace_support);

    printf("\nDevice driver to include\n");
    printf("------------------------\n");
    wanna_support("Keyboard",&keyb_support);
    wanna_support("Text Windows",&crtwin_support);
    wanna_support("Serial",&com_support);
    wanna_support("Serial mouse",&smouse_support);
    
    _clearscreen(0);
    printf("Summary\n");
    printf("-------\n\n");
    printf("Selected Architecture : %s\n\n",arch_opts[arch]);
    printf("Port  support : %s\n",yesno[port_support]);
    printf("Cab   support : %s\n",yesno[cab_support]);
    printf("SRP   support : %s\n",yesno[srp_support]);
    printf("Trace support : %s\n\n",yesno[trace_support]);
    printf("Device drivers\n\n");
    if (keyb_support) printf("Keyboard\n");
    if (crtwin_support) printf("Text windows\n");
    if (com_support) printf("Serial communications\n");
    if (smouse_support) printf("Serial mouse\n");

    printf("\nIs everything set up correctly ? ");
    flushall();
    if (!yes()) {
	printf("Then re-run KBUILD!\n");
	exit(-1);
    }
    printf("\nOk\nDo you want to edit the number of available kernel descriptors\n");
    printf("for the various resources ? ");
    flushall();
    if (yes()) system("edit ..\\H\\SYS\\CONST.H");
    build_hartik_h();
    
    _clearscreen(0);
    printf("Setup complete\nBuild in process...\n");
    getcwd(actual_path,80);
    printf("\nCreating distribution tree\n");
    chdir(distr_path);
    mkdir("bin");
     mkdir("lib");
    mkdir("h");	    
    mkdir("examples");
    chdir(actual_path);
    
    printf("Producing makefile\n");
    file_make = fopen("config.mk","wt");
    fprintf(file_make,"#\n");
    fprintf(file_make,"# Kernel Config Makefile\n");
    fprintf(file_make,"#\n\n");
    fprintf(file_make,"#\n");
    fprintf(file_make,"# Logged information\n");
    fprintf(file_make,"# KBuild version : %x.%x\n",((version & 0xFF00) >> 8),(version & 0x0FF));
    fprintf(file_make,"#\n\n");
    fprintf(file_make,"#\n");
    fprintf(file_make,"# Kernel support\n");
    fprintf(file_make,"#\n\n");
    if (port_support)  fprintf(file_make,"PORT   = -D__PORT__\n");
    if (cab_support)   fprintf(file_make,"CAB    = -D__CAB__\n");
    if (srp_support)   fprintf(file_make,"SRP    = -D__SRP__\n");
    if (trace_support) fprintf(file_make,"TRACE  = -D__TRACE__\n");
    fprintf(file_make,"\n#\n");
    fprintf(file_make,"# Device support\n");
    fprintf(file_make,"#\n\n");
    if (keyb_support) fprintf(file_make,"KEYB     = Active\n");
    if (crtwin_support) fprintf(file_make,"CRTWIN   = Active\n");
    if (com_support) fprintf(file_make,"SCOM     = Active\n");
    if (smouse_support) fprintf(file_make,"SMOUSE   = Active\n");
    fprintf(file_make,"\n#\n");
    fprintf(file_make,"# The HARTIK main path\n");
    fprintf(file_make,"#\n\n");
    chdir("..");
    fprintf(file_make,"MAIN_PATH = %s\n\n",getcwd(NULL,0));
    chdir("KERN");
    fclose(file_make);
    copy(arch_makefile[arch],"makefile");
    
    setenv("HARTIK_SRC",arch_setting[arch],1);
    setenv("HARTIK_DISTR",distr_path,1);
    
    printf("Making the kernel & device library!\n");
    system(arch_make[arch]);

    printf("Setting up header files\n");
    strcpy(header_path,distr_path);
    strcat(header_path,"\\H");
    if (keyb_support) copy("..\\H\\KEYB.H",header_path);
    if (crtwin_support) copy("..\\H\\CRTWIN.H",header_path);
    if (com_support) copy("..\\H\\SCOM.H",header_path);
    if (smouse_support) copy("..\\H\\SMOUSE.H",header_path);
    copy("..\\H\\HARTIK.H",header_path);
    
    printf("Operation completed\n");
    printf("You can now edit the local makefile in %s\\EXAMPLES for the final setup\n",distr_path);
    printf("Then you can try some examples with M <example>\n");
    printf("The environment variable SRC controls what kind of output you will produce.\n");
    printf("The environment variable HARTIK should contain the distribution path.\n\n");
    
    
}
