# Standard library for X/COFF kernel
# Makefile for GNU MAKE & GCC 2.8.0

CC  = gcc
AS  = gcc
HARTIK  = ../..
INCL    = -I$(HARTIK)/h/x86 -I$(HARTIK)/h -I.
LIBS    = $(HARTIK)/lib

C_OPT =  $(INCL) -O -finline-functions -fno-builtin -D__GNU__
A_OPT =  $(INCL) -D__GNU__

#
# Main settings!
# 

KERN_PATH    = .
DEV_PATH     = ../dev
INCLUDE_PATH = ../h
LIB_PATH     = ../lib

CC           = gcc
C_OUTPUT     = -o $*.o
C_OPT        =  -I$(INCLUDE_PATH) -I$(INCLUDE_PATH)/sys -I$(INCLUDE_PATH)/x86 \
		-D__BASE_CONST__ -D__C_HEADER__ -D__GNU__ -O \
		-finline-functions -fno-builtin
ASM_OPT      = -x assembler-with-cpp -I. -I$(INCLUDE_PATH) -I$(INCLUDE_PATH)/sys -I$(INCLUDE_PATH)/x86

LIB_OS       = hart

#
# Kernel files
#       

KERN_OBJ  = $(KERN_PATH)/kern.o \
	$(KERN_PATH)/srp.o    \
	$(KERN_PATH)/queue.o  \
	$(KERN_PATH)/sem.o    \
	$(KERN_PATH)/exc.o    \
	$(KERN_PATH)/load.o   \
	$(KERN_PATH)/log.o    \
	$(KERN_PATH)/port.o   \
	$(KERN_PATH)/cabs.o

.PHONY : clean allclean depend info 

#
# Ok! Finally the dependency rules!
#

info : 
	@type ..\info.txt

all : lib$(LIB_OS).a
	@copy lib$(LIB_OS).a ..\\lib

install : lib$(LIB_OS).a
	@copy lib$(LIB_OS).a ..\\lib

clean :
	-del *.o
	-del *.a
	-del *.err
	-del makefile

allclean : clean
	echo # Kernel Dependency file > deps
	gcd $(LIB_PATH)
	-del lib$(LIB_OS).a
	cd ..\kern

depend deps:
	$(CC) $(C_OPT) -M $(KERN_OBJ:.o=.c) > deps

#
# The kernel library!!
#

lib$(LIB_OS).a : $(KERN_OBJ)
	ar rs lib$(LIB_OS).a $(KERN_OBJ)

include deps

%.o : %.c
	$(REDIR) $(CC) $(C_OPT) $(C_OUTPUT) -c $<
	
