/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/*              Giuseppe Lipari                                 */
/* Date:        9/5/96                                          */

/* File:        Queue.C                         */
/* Revision:    1.3                             */
/* (see kern.his for history of version)        */

/* System queues function management    */
/* The functions use the process descriptor table entries, particularly */
/* the next & prev to mantain the queues order; the insertion is based  */
/* on deadline priority, so to respect the EDF strategy                 */

#include "vm.h"
#include "kern.h"

void q_insert(PID i, QUEUE *que)
{
    DWORD dl;
    PID p,q;

    p = NIL;
    q = *que;
    dl = proc_table[i].priority;

    while ((q != NIL) && (dl >= proc_table[q].priority)) {
	p = q;
	q = proc_table[q].next;
    }

    if (p != NIL) proc_table[p].next = i;
    else *que = i;
    if (q != NIL) proc_table[q].prev = i;
    proc_table[i].next = q;
    proc_table[i].prev = p;
}

QUEUE q_extract(PID i, QUEUE *que)
{
    PID p,q;

    p = proc_table[i].prev;
    q = proc_table[i].next;

    if (p == NIL) *que = q;
    else proc_table[p].next = proc_table[i].next;

    if (q != NIL) proc_table[q].prev = proc_table[i].prev;
    return(i);
}

PID q_getfirst(QUEUE *que)
{
    QUEUE q = *que;

    if (q == NIL) return(NIL);
    *que = proc_table[q].next;
    if (*que != NIL) proc_table[*que].prev = NIL;
    return(q);
}

