# Standard library for X/COFF kernel
# Makefile for GNU MAKE & GCC 2.8.0

#
# Standard path
#

INCLUDE_PATH = ../../h
LIB_PATH     = ../../lib

AS           = gcc
CC           = gcc
C_OUTPUT     = -o $*.o 
C_OPT        =  -O -finline-functions -fno-builtin \
		-I$(INCLUDE_PATH) -I$(INCLUDE_PATH)/x86 -I$(INCLUDE_PATH)/sys \
		-D__BASE_CONST__ -D__C_HEADER__ -D__GNU__ -c \
#               -D__VM_DEBUG__ -D__MEM_DEBUG__ -D__FPU_DEBUG__
 
A_OPT        = -I$(INCLUDE_PATH) -I$(INCLUDE_PATH)/x86 -I$(INCLUDE_PATH)/sys -D__GNU__

ifdef REDIR
LINK_OPT     = -oformat coff-go32 -s -nostartfiles -nostdlib -L$(LIB_PATH)
LINK_STARTUP = $(LIB_PATH)/x0.o $(LIB_PATH)/k_standa.o
LD = ld
LINK_LIB     = -lhlib -lhm
else 
LINK_OPT     = -s -nostartfiles -nostdlib -L$(LIB_PATH)
LINK_STARTUP = $(LIB_PATH)/x0.o
LD = gcc
LINK_LIB     = -lhlib -lm
endif

VM_C_OBJ = vmfpu32.o \
	vmstuff.o \
	vmstuf32.o \
	vmmem.o \
	vmcx32b.o \
	vminit32.o

VM_OBJ = $(VM_C_OBJ) vmcx32a.o vmcpu32.o
	
#
# Ok! Finally the dependency rules!
# We do not mess with automatic depencencies here!!
#

.PHONY : all clean depend

all : vmtest libvm.a
	@copy libvm.a ..\..\lib

install : libvm.a
	@copy libvm.a ..\..\lib
	
clean :
	-del makefile
	-del *.o
	-del *.err
	-del vmtest
	-del libvm.a
	
allclean : clean
	echo # Kernel Dependency file > deps
	-del ..\..\lib\libvm.a

depend deps:
	$(CC) $(C_OPT) -M $(VM_C_OBJ:.o=.c) vmdemo.c > deps

vmtest : $(VM_OBJ) vmdemo.o
	$(LD) $(LINK_OPT) $(LINK_STARTUP) $(VM_OBJ) vmdemo.o $(LINK_LIB) -o vmtest

libvm.a : $(VM_OBJ)
	ar rs libvm.a $(VM_OBJ)

%.o : %.c
	$(REDIR) $(CC) $(C_OPT) $(C_OUTPUT) $<

include deps

vmcx32a.o : vmcx32a.S
	$(REDIR) $(CC) $(A_OPT) -o vmcx32a.o -c vmcx32a.S

vmcpu32.o : vmcpu32.S
	$(REDIR) $(CC) $(A_OPT) -o vmcpu32.o -c vmcpu32.S
