; Project:     HARTIK 3.0                                       
; Description: Hard Real TIme Kernel for 8086 compatible        
; Author:      Gerardo Lamastra                                 
; Date:        9/5/96                                           

; File:         VmCx16a.Asm
; Revision:     2.0G                                    


.model large

;
; FPU context size
;
FPU_CTX_SIZE    EQU     108     

;
; Warning!
; These constants are also present in the Kern.H file; so please remember
; to update them at once, or the kernel will suffer malfunctions!
;

DIV_BY_0        EQU     0  ; These are the VM... exceptions 
MATH_EXC        EQU     1  
NMI_EXC         EQU     2
DEBUG_EXC       EQU     3
BREAKPOINT_EXC  EQU     4

VM_FAULT        EQU     5
CLOCK_OVERRUN   EQU     64 ; Clock overrun error code
MAX_PROC	EQU	66
MAIN_INDEX	EQU	(MAX_PROC-2)	

;
; External kernel interface
;
EXTRN	_wake_up	: FAR
EXTRN	_act_int	: FAR
EXTRN	_act_exc	: FAR
EXTRN	_sys_abort_tail : FAR
EXTRN	_VM_FPU_hook 	: FAR

; 
; This is the safety stack! If something goes wrong we cannot assume
; anything about the actual context; we must have some safe memory
; to use as stack area which must be statically allocated!
;
safestack	SEGMENT STACK
		ASSUME CS:safestack
PUBLIC _SafeStack
		
		db      4096    dup(0)
safestacktop	label near
_SafeStack	label near
safestack	ENDS            

;
; Data segment; only few variables we use during context switch!
; I tried to take these into VM2.C but the Watcom assembler does not
; manage the external reference correctly (Well, i did not succeded to
; make it work). This caused the duplication of MAX_PROC & MAIN_INDEX into
; this module!
;
vmcx16a_DATA	SEGMENT PARA PUBLIC 'FAR_DATA'
		ASSUME CS:vmcx16a_DATA
		
PUBLIC _VM_CPU
PUBLIC _VM_has_CPU
PUBLIC _VM_clock

_ctx_tmp        dw      0
_VM_CPU         label   far
		dd      MAX_PROC DUP (0)
_VM_has_CPU     dw      MAIN_INDEX
_VM_clock	label	far
VM_clocklo	dw	0
VM_clockhi	dw	0
vmcx16a_DATA	ENDS

;
; Code segment; any context stuff is here; also int handlers are here.
; All functions follow the C standard passing parameter sequence
;
vmcx16a_TEXT	SEGMENT USE16
		ASSUME CS:vmcx16a_TEXT,DS:NOTHING,ES:NOTHING

PUBLIC _VM_context_change
PUBLIC _VM_context_load
PUBLIC _VM_context_save
PUBLIC _CPU_init
PUBLIC _VM_abort        
PUBLIC _VM_timer
PUBLIC _exc0
PUBLIC _exc1
PUBLIC _exc2
PUBLIC _exc3
PUBLIC _exc4
PUBLIC _h01
PUBLIC _h02
PUBLIC _h03
PUBLIC _h04
PUBLIC _h05
PUBLIC _h06
PUBLIC _h07
PUBLIC _h08
PUBLIC _h09
PUBLIC _h10
PUBLIC _h11
PUBLIC _h12
PUBLIC _h13
PUBLIC _h14
PUBLIC _h15

;
; void __cdecl VM_abort(int code)
;
; The VM_abort freeze the kernel disabling interrupts, switch to a safe
; stack and transfer control to the kernel _sys_abort_tail which completes 
; the kernel shutdown.

_VM_abort               PROC FAR
			cli
			;
			; Get argument
			;
			push    bp
			mov     bp,sp                   
			mov     bx,6[bp]
			;
			; Switch to safe stack
			;
			cld
			mov     ax,SEG safestack
			mov     ss,ax
			mov     sp,OFFSET safestacktop
			mov     ax,bx
			push	ax
			call    _sys_abort_tail
_VM_abort               ENDP

;
; BYTE * __cdecl CPU_init(void (*task)(),BYTE *stack,
;                         WORD parm,void (*killer)(void));
; This function must push onto the stack all the regs for the first process
; execution. It returns the current stack top

_CPU_init               PROC FAR
			push    bp
			mov     bp,sp
			push    es
			push    di
			;
			; Load into es:di the pointer to the 
			; process stack
			;
			mov     es,[bp].12      ; SS is here
			mov     di,[bp].10      ; SP is here
			sub     di,2            ; Get first usable location
			
			; Build task stack!
			; The first object is the task argument (a WORD)
			mov     ax,[bp].14
			mov     word ptr es:[di],ax
			sub     di,2
			
			; Now place the return address for the task
			; This will be the pointer to a task killer 
			; primitive
			mov     ax,[bp].18              ; This is CS
			mov     word ptr es:[di],ax     
			sub     di,2
			mov     ax,[bp].16              ; This is IP
			mov     word ptr es:[di],ax
			sub     di,2
			
			; Store flags register
			; All flags cleared except IF
			mov     word ptr es:[di],0200H
			sub     di,2
			
			mov     ax,[bp].8       ; CS is here
			mov     es:[di],ax      
			sub     di,2
			mov     ax,[bp].6       ; IP is here 
			mov     es:[di],ax
			sub     di,2
			
			mov     cx,7            ; General regs now
			mov     ax,0            ; They are all zero
			std                     ; (AX,BX,CX,DX,SI,DI,BP)
			rep stosw               
			
			; This should be not necessary if we compile
			; with far data threshold set to 0; in this case
			; the compiler will reload the segment register
			; each time it accesses to a global variable
			; Anyway we can't know what if ....
			
			mov     ax,ds           ; Now the DATA segments
			mov     es:[di],ax      ; DS
			sub     di,2
			mov     es:[di],ax      ; ES
			mov     ax,di
			mov     dx,es
			;
			; Back to the caller
			;
			pop     di
			pop     es
			pop     bp
			cld
			ret
_CPU_init               ENDP

;
; void __interrupt __cdecl VM_timer(void);
; This function is called each time a timer (INT 08h usually) is
; detected; the register are saved for an eventual context switch;
; Anyway even if we do not switch context ,we must save all regs because
; the timer int is completely asynchronous!
; any kernel operation is performed into _wake_up function (written in C)
; which should be quite machine indipendent;
; CONTEXT wake_up(CONTEXT); 
; If a preemption occurs the actual context is stored into process descriptor
; table and the new context is loaded from the return value of function.

_VM_timer               PROC FAR
			push    ax ; Save all regs!
			push    bx ;
			push    cx
			push    dx
			push    si
			push    di
			push    bp
			push    ds
			push    es
			
			mov     dx,020H ; Send EOI to master PIC
			mov     al,020H ; to perform later the overrun test
			out     dx,al
			
			; Get current context into ax & on stack top
			; to pass it later to wake_up                   
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,ds:_VM_has_CPU
			push    ax                      
			; Get context_table entry corresponding to
			; the used context & save there the stack pointer
			mov     bx,ax
			shl     bx,1
			shl     bx,1
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,ss
			mov     word ptr ds:_VM_CPU[bx].2,ax
			mov     ax,sp
			add     ax,2
			mov     word ptr ds:_VM_CPU[bx],ax			
			; Remember we have pushed the current context value
			; onto stack; then add 2 to the current stack top
			
			mov     ax,SEG vmcx16a_DATA
			mov	ds,ax
			mov	ax,VM_clocklo
			inc 	ax
			mov	VM_clocklo,ax
			jnc	VM_clockdone
			mov	ax,VM_clockhi
			inc 	ax
			mov	VM_clockhi,ax
VM_clockdone:		cld
			call    _wake_up
			add     sp,2    ; remove parms from stack
			;
			; ax is the return value of _wake_up
			; Put into _VM_has_CPU & make context switch
			;
			push    ax
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			pop     ax
			mov     ds:_VM_has_CPU,ax
			push    ax
			
			; Test if FPU context switch is necessary
			; And perform it
			
ifdef __FPU_ENABLED__			
			cld
			call    _VM_FPU_hook
endif
			;
			; This is the overrun test
			; Do it after sending EOI to master PIC
			;                       
			mov     dx,20H
			mov     al,0AH
			out     dx,al
			in      al,dx
			test    al,1
			jz      Timer_OK
			mov     ax,CLOCK_OVERRUN
			push    ax
			; It would be a nonsense to pass through the 
			; kernel exception handling, because this would
			; mean to continue to trigger the interrupt 
			; In any case, it is impossible for the kernel to
			; work correctly
			call    _VM_abort
			
Timer_OK:               ;
			; Get the SS:SP pointer from the
			; context_table structure
			;
			pop     ax
			mov     bx,ax
			shl     bx,1
			shl     bx,1                            
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,word ptr ds:_VM_CPU[bx].2
			mov     ss,ax
			mov     ax,word ptr ds:_VM_CPU[bx]
			mov     sp,ax
			
			pop     es
			pop     ds
			pop     bp
			pop     di
			pop     si
			pop     dx
			pop     cx
			pop     bx
			pop     ax
			iret
_VM_timer               ENDP

;
; CONTEXT VM_context_save(void);
; This function return the stack top of the actual task; it should be called
; only from a kernel primitive and it must be the first operation a primitive
; executes; It also clear INT for atomic primitive!

_VM_context_save        PROC FAR 
			cli
			push    bx
			push    cx
			; Get the current active context
			; and store it into ax and onto stack top
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,ds:_VM_has_CPU
			push    ax
			; Access to the VM_CPU array
			; each entry is 4 bytes long; shifting left
			; 2 times is the same!
			mov     bx,ax
			shl     bx,1
			shl     bx,1
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			;
			; Put into the first field of the structure
			; the SS:(SP-18H+4H-4H) pointer, representing the
			; saved CPU context status!
			; The context is 18H bytes long; we subtract
			; 4 bytes as we have pushed the return value;
			; subtract other 2 bytes since we have pushed ax
			; which is to be returned later; finally add 2 bytes
			; as the VM_context_change has a 2 bytes argument!
			; Other 4 bytes to sub as we have pushed bx & cx
			mov     ax,ss
			mov     word ptr ds:_VM_CPU[bx].2,ax
			mov     ax,sp 
			sub     ax,10h  ; 18H is the CONTEXT size!
			mov     word ptr ds:_VM_CPU[bx],ax
			; Return _VM_has_CPU, the current context                  
			pop     ax
			pop     cx
			pop     bx
			ret
_VM_context_save        ENDP

;
; void VM_context_change(CONTEXT c);
; This function perform the real context switch!

_VM_context_change      PROC FAR
			; First we have to reorder the stack!
			; So we can IRET!
			; From  ->   To
			;  CS        F  
			;  IP        CS
			;            IP
			pushf
			push    bp
			mov     bp,sp
			push    ax
			push    ds
			mov     ax,[bp].2
			or      ax,0200h        ; Unmask INT flag!
			xchg    ax,[bp].6
			xchg    ax,[bp].4
			mov     [bp].2,ax
			;
			; Store into temporary vars the parameter
			; Remember that we are in large model, then we
			; must reload DS to access global vars!
			;
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,[bp].8
			mov     ds:_ctx_tmp,ax
			pop     ds
			pop     ax
			pop     bp
			;
			; Save actual context
			;
			push    ax
			push    bx
			push    cx
			push    dx
			push    si
			push    di
			push    bp
			push    ds
			push    es                      
			;
			; Set the new active context into _VM_has_CPU
			;
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,ds:_ctx_tmp
			push    ax                      
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			pop     ax
			mov     ds:_VM_has_CPU,ax
			push    ax

			; Test if FPU context switch is necessary
			; And perform it
ifdef __FPU_ENABLED__			
			cld
			call    _VM_FPU_hook
endif
			; Get the SS:SP pointer into the
			; context_table structure
			pop     ax
			mov     bx,ax
			shl     bx,1
			shl     bx,1
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,word ptr ds:_VM_CPU[bx].2                       
			mov     ss,ax
			mov     ax,word ptr ds:_VM_CPU[bx]
			mov     sp,ax
			;
			; Load the new context
			;
			pop     es
			pop     ds
			pop     bp
			pop     di
			pop     si
			pop     dx
			pop     cx
			pop     bx
			pop     ax
			iret                            
_VM_context_change      ENDP

;
; void VM_context_load(CONTEXT c);
; If the old task has been killed we only need to load the context
; of the new executing task; we do not need to save anything.

_VM_context_load        PROC FAR
			push    bp
			mov     bp,sp                   
			;
			; Set the new active context into _VM_has_CPU
			;
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,[bp].6
			mov     ds:_VM_has_CPU,ax
			pop     bp

			; Test if FPU context switch is necessary
			; And perform it
			push    ax
ifdef __FPU_ENABLED__			
			cld
			call    _VM_FPU_hook
endif
			pop     ax
			
			; Get the SS:SP pointer into the
			; context_table structure
			mov     bx,ax
			shl     bx,1
			shl     bx,1
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,word ptr ds:_VM_CPU[bx].2
			mov     ss,ax
			mov     ax,word ptr ds:_VM_CPU[bx]
			mov     sp,ax
			;
			; Load the new context
			;
			pop     es
			pop     ds
			pop     bp
			pop     di
			pop     si
			pop     dx
			pop     cx
			pop     bx
			pop     ax
			iret                            
_VM_context_load        ENDP

;
; These functions are the entry point to the common handlers
; They only need to set in AX the interrupt code number
; which will be processed by the act_int/act_exc functions

_exc0                   PROC FAR
			push    ax
			mov     ax,DIV_BY_0
			jmp     VM_exception
_exc0                   ENDP

_exc1                   PROC FAR
			push    ax
			mov     ax,DEBUG_EXC
			jmp     VM_exception
_exc1                   ENDP
 
_exc2                   PROC FAR
			push    ax
			mov     ax,NMI_EXC
			jmp     VM_exception
_exc2                   ENDP

_exc3                   PROC FAR
			push    ax
			mov     ax,BREAKPOINT_EXC
			jmp     VM_exception
_exc3                   ENDP

_exc4                   PROC FAR
			push    ax
						
			;                       
			; Send EOI for 8086 coprocessor trick
			; Send 0H on Coprocessor port 0F0H
			;
			xor     ax,ax
			out     0F0H,al                 
			mov     al,020H                 
			out     0A0H,al
			out     020H,al
			
			mov     ax,MATH_EXC
			jmp     VM_exception
_exc4                   ENDP

VM_exception            LABEL NEAR                      
			push    bx
			push    cx
			push    dx
			push    bp                      
			push    si
			push    di
			push    ds
			push    es
			
			cld
			push    ax
			call    _act_exc
			add     sp,2
			
			pop     es
			pop     ds
			pop     di
			pop     si
			pop     bp
			pop     dx
			pop     cx
			pop     bx
			pop     ax
			iret

_h01                    PROC FAR
			push    ax

			; This stuff code  is only used to avoid
			; keyboard stuck on exit! 
			; It is used ONLY for VM debugging purposes!
			;push    dx
			;mov     dx,060H
			;in      al,dx
			;pop     dx
		
			mov     ax,01H
			jmp     VM_handler
_h01                    ENDP

_h02                    PROC FAR 
			push    ax
			mov     ax,02h
			jmp     VM_handler
_h02                    ENDP

_h03                    PROC FAR 
			push    ax
			mov     ax,03h
			jmp     VM_handler
_h03                    ENDP

_h04                    PROC FAR 
			push    ax
			mov     ax,04h
			jmp     VM_handler
_h04                    ENDP

_h05                    PROC FAR 
			push    ax
			mov     ax,05h
			jmp     VM_handler
_h05                    ENDP

_h06                    PROC FAR 
			push    ax
			mov     ax,06h
			jmp     VM_handler
_h06                    ENDP

_h07                    PROC FAR 
			push    ax
			mov     ax,07h
			jmp     VM_handler
_h07                    ENDP

_h08                    PROC FAR
			push    ax
			mov     ax,08h
			jmp     VM_handler
_h08                    ENDP

_h09                    PROC FAR 
			push    ax
			mov     ax,09h
			jmp     VM_handler
_h09                    ENDP

_h10                    PROC FAR 
			push    ax
			mov     ax,0ah
			jmp     VM_handler
_h10                    ENDP

_h11                    PROC FAR 
			push    ax
			mov     ax,0bh
			jmp     VM_handler
_h11                    ENDP

_h12                    PROC FAR 
			push    ax
			mov     ax,0ch
			jmp     VM_handler
_h12                    ENDP

_h13                    PROC FAR 
			push    ax
			mov     ax,0dh
			jmp     VM_handler
_h13                    ENDP

_h14                    PROC FAR 
			push    ax
			mov     ax,0eh
			jmp     VM_handler
_h14                    ENDP

_h15                    PROC FAR 
			push    ax
			mov     ax,0fh
			jmp     VM_handler
_h15                    ENDP

VM_handler              LABEL NEAR
			push    bx      ; Terminate the context save
			push    cx      
			push    dx
			push    si
			push    di
			push    bp
			push    ds
			push    es
			
			push    ax      ; Push int number onto stack

			; Get current context into ax & on stack top
			; to pass it later to act_int   
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,ds:_VM_has_CPU
			push    ax
			
			; Now we have on stack the irq number and
			; the current context!
			cld
			call    _act_int
			add     sp,2    ; remove parms from stack
			pop     bx      ; OK now stack is clear
					; IRQ num is in BX
			
			push    bx      ; Save IRQ number
			push    ax      ; Save return value
						
			; Get context_table entry corresponding to
			; the used context & save there the stack pointer
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     bx,ds:_VM_has_CPU
			shl     bx,1
			shl     bx,1
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,ss
			mov     word ptr ds:_VM_CPU[bx].2,ax
			mov     ax,sp   
			; Remember we have pushed the interrupt number 
			; and the current context value onto stack;
			; then add 4 to the current stack top
			add     ax,4
			mov     word ptr ds:_VM_CPU[bx],ax
			;
			; ax is the return value of _act_int
			; Put into _VM_has_CPU & make context switch
			; bx is the irqno
			;
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			pop     ax
			mov     ds:_VM_has_CPU,ax
			
			; Test if FPU context switch is necessary
			; And perform it (Save BX throughout)
ifdef __FPU_ENABLED__			
			cld
			call    _VM_FPU_hook
endif			
			;
			; Send EOI to master & slave (if necessary) PIC
			;
			pop     bx
			mov     al,20h
			cmp     bx,08h
			jb      eoi_master
eoi_slave:              mov     dx,0a0h
			out     dx,al
eoi_master:             mov     dx,020h
			out     dx,al
			;
			; Get the SS:SP pointer into the
			; context_table structure
			;
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax                   
			mov     bx,ds:_VM_has_CPU
			shl     bx,1
			shl     bx,1    
			mov     ax,SEG vmcx16a_DATA
			mov     ds,ax
			mov     ax,word ptr ds:_VM_CPU[bx].2
			mov     ss,ax
			mov     sp,word ptr ds:_VM_CPU[bx]
			;
			; Load the new context
			;
			pop     es
			pop     ds
			pop     bp
			pop     di
			pop     si
			pop     dx
			pop     cx
			pop     bx
			pop     ax
			iret
			
vmcx16a_TEXT		ENDS

END
