/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/* Date:        9/5/96                                          */

/* File: 	VmCx16b.C					*/
/* Revision:    2.0G                                            */

/* C Code to implement 8086 virtual machine                     */

#include <mem.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>

#include <cons.h>
#include "vm.h"
#include "kern.h"

/* Used to control if a context descriptor has been allocated */
WORD VM_control[MAX_PROC];

/* They are declared in VmCx16a.Asm */
extern BYTE *VM_CPU[MAX_PROC]; /* Global CPU context management */
extern CONTEXT VM_has_CPU;

/* FPU context management! */
/* They are declared in VMFPU.C */
extern BYTE VM_FPU_savearea[FPU_CTX_SIZE]; /* Global FPU scratch SaveArea */
extern CONTEXT VM_has_FPU;
extern BYTE VM_FPU_buffer[MAX_PROC][FPU_CTX_SIZE];
extern BYTE VM_FPU_std[FPU_CTX_SIZE];

/* Enhanced context switch routines; They are used to manage FPU */
/* preepmtion transparently also into 16 bit VM                  */

CONTEXT __cdecl VM_context_create(void __cdecl (*task)(),BYTE *stack,
				  WORD arg,void (*killer)(void),WORD ctrl)
{
    register i = 0,found = FALSE;
    /* Stack setup routine */
    extern BYTE * __cdecl CPU_init(void __cdecl (*task)(),BYTE *stack,WORD arg,void (*killer)());

    /* Find a free descriptor */
    while (!found && i < MAX_PROC) {
	if (!(VM_control[i] & CTX_BUSY)) found = TRUE;
	else {
	    i++;
	}
    }
    if (i == MAX_PROC) return(-1);    
    else VM_control[i] = CTX_BUSY | ctrl;

    /* Init FPU status if necessary */
    #ifdef __FPU_ENABLED__
    if (VM_control[i] & USE_FPU) {
	memcpy(&VM_FPU_buffer[i][0],VM_FPU_std,FPU_CTX_SIZE);
    }
    #endif
    VM_CPU[i] = CPU_init(task,stack,arg,killer);
    return(i);
}

void __cdecl VM_context_delete(CONTEXT c)
{
    if (VM_control[c] & USE_FPU) {
	if (VM_has_FPU == c) VM_has_FPU = NIL;
    }
    VM_CPU[c] = NULL;
    VM_control[c] = 0;
}

char *VM_context_sprintf(char *str,CONTEXT c)
{
    sprintf(str,"Id : %2d CPU : %lp",c,VM_CPU[c]);
    return(str);
}
