; Project:      HARTIK 3.0                                      
; Description:  Hard Real TIme Kernel for 386 & higher machines 
; Author:       Gerardo Lamastra                        
; Date:         9/5/96                                          
; Revision:     2.0

; Module: VMCtx32a.Asm
; Virtual Machine level for 386 & upper compatible processor
; Context Switch Assembler Functions

include sel.inc

OCW2_M          equ     020H
OCW2_S          equ     0A0H
EOI             equ     020H ; End Of Interrupt code for PIC!

CLOCK_OVERRUN   equ     64   ; This is also present in Exc.H
MATH_EXC        equ     1    ; This is also present in Exc.H   

EXTERN  _wake_up        : NEAR
EXTERN  _act_int        : NEAR
EXTERN  _act_exc        : NEAR
EXTERN  _sys_abort_tail : NEAR
EXTERN  _VM_exc_hook	: NEAR
EXTERN  _VM_FPU_hook	: NEAR
EXTERN  __GETDS         : NEAR

.386p
		NAME    vmcx32a.asm

_TEXT           SEGMENT BYTE PUBLIC USE32 'CODE'
		ASSUME  CS:_TEXT,DS:DGROUP,SS:DGROUP

		PUBLIC _VM_timer
		PUBLIC _VM_context_save
		PUBLIC _VM_context_change
		PUBLIC _VM_context_load		
		PUBLIC _VM_abort
		
		PUBLIC _h01
		PUBLIC _h02
		PUBLIC _h03
		PUBLIC _h04
		PUBLIC _h05
		PUBLIC _h06
		PUBLIC _h07
		PUBLIC _h08
		PUBLIC _h09
		PUBLIC _h10
		PUBLIC _h11
		PUBLIC _h12
		PUBLIC _h13
		PUBLIC _h13_bis
		PUBLIC _h14
		PUBLIC _h15

		PUBLIC _exc0
		PUBLIC _exc1
		PUBLIC _exc2
		PUBLIC _exc3
		PUBLIC _exc4
		PUBLIC _exc5
		PUBLIC _exc6
		PUBLIC _exc7
		PUBLIC _exc8
		PUBLIC _exc9
		PUBLIC _exc10
		PUBLIC _exc11
		PUBLIC _exc12
		PUBLIC _exc13
		PUBLIC _exc14
		PUBLIC _exc15
		PUBLIC _exc16

		PUBLIC init_TR_

; This function assign a value to the TASK register of 386 architecture
; We MUST do this BEFORE we use the HW multitask
;
init_TR_                PROC
			ltr     ax
			mov     JmpSel,ax
			ret
init_TR_                ENDP

		
; CONTEXT __cdecl VM_context_save(void);
; The context is returned into AX; Interrupts are also cleared as we are
; entering into kernel primitives
;
_VM_context_save        PROC
			cli
			xor     eax,eax
			str     ax
			ret
_VM_context_save        ENDP

; void __cdecl VM_context_change(CONTEXT c)
; Use 386 task switch ability. This is the last call of any preemption
; generating primitive; when the original task is re-activated the 
; interrupt flag is restored with STI
; In 32 bit systems, VM_context_load is an alias for VM_context_change!

_VM_context_load:
_VM_context_change      PROC
			push    ebp
			mov     ebp,esp
			call    __GETDS                 
			mov     eax,+8[ebp]
			cmp     JmpSel,ax
			je      NoPreempt
			mov     JmpSel,ax
			jmp     PWORD PTR JmpZone
NoPreempt:              pop     ebp
			sti
			ret
_VM_context_change      ENDP

_VM_abort               PROC
			; As we are terminating we cannnot handle
			; any other interrupt!
			cli
			; Get argument
			mov     eax,4[esp]
			; Switch to safe stack
			mov     esp,OFFSET SafeStack
			; Call sys_abort(code)
			push	eax
			call    _sys_abort_tail
_VM_abort               ENDP

; This is the timer handler; it reloads for safety the DS register; this
; prevents from mess when timer interrupts linear access to memory (not in
; ELF address space); then the EOI is sent in order to detect timer overrun
; The high level kernel procedure wake_up() is called to perform the 
; preeption at higher level (process descriptos); the resulting context
; if different from the old one is used to trigger the task activation.
;
_VM_timer               PROC    FAR
			pushad
			push    ds
			push    ss
			push    es
			push    fs
			push    gs
			mov     ax,PM_APPLDATA_SEL
			mov     ds,ax
			mov     es,ax
			
			; Send EOI to master PIC
			; to perform later the overrun test
			mov     al,EOI
			out     OCW2_M,al
			
			;
			; Call wake_up(actual_context)
			;
			cld
			mov	eax,_VM_clock
			inc 	eax
			mov	_VM_clock,eax
			xor     eax,eax
			str     ax
			push    eax
			call    _wake_up
			add     esp,4
			push    eax     ; Save result on stack
			;
			; This is the overrun test
			; Do it after sending EOI to master PIC
			;
			
			mov     al,0AH
			out     020H,al
			in      al,020H
			test    al,1
			jz      Timer_OK
			mov     eax,CLOCK_OVERRUN
			push    eax
			call    _VM_abort

Timer_OK:               pop     eax             ;Recover wake_up result
			cmp     ax,JmpSel       
			je      NoPreempt2
			mov     JmpSel,ax
			jmp     PWORD PTR JmpZone ; DISPATCH!
NoPreempt2:             pop     gs
			pop     fs
			pop     es
			pop     ss
			pop     ds
			popad
			iretd
_VM_timer               ENDP

;
; These are the hardware handlers; they all jump to VM_handler setting
; the interrupt number into EAX & save the registers on stack
;
_h01                    PROC FAR
			pushad
			mov     eax,1
			jmp     VM_handler
_h01                    ENDP                            

_h02                    PROC FAR
			pushad
			mov     eax,2
			jmp     VM_handler
_h02                    ENDP                            

_h03                    PROC FAR
			pushad
			mov     eax,3
			jmp     VM_handler
_h03                    ENDP                            

_h04                    PROC FAR
			pushad
			mov     eax,4
			jmp     VM_handler
_h04                    ENDP                            

_h05                    PROC FAR
			pushad
			mov     eax,5
			jmp     VM_handler
_h05                    ENDP                            

_h06                    PROC FAR
			pushad
			mov     eax,6
			jmp     VM_handler
_h06                    ENDP

_h07                    PROC FAR
			pushad
			mov     eax,7
			jmp     VM_handler
_h07                    ENDP                            

_h08                    PROC FAR
			pushad
			mov     eax,8
			jmp     VM_handler
_h08                    ENDP                            

_h09                    PROC FAR
			pushad
			mov     eax,9
			jmp     VM_handler
_h09                    ENDP                            

_h10                    PROC FAR
			pushad
			mov     eax,10
			jmp     VM_handler
_h10                    ENDP                            

_h11                    PROC FAR
			pushad
			mov     eax,11
			jmp     VM_handler
_h11                    ENDP                            

_h12                    PROC FAR
			pushad
			mov     eax,12
			jmp     VM_handler
_h12                    ENDP                            

_h13                    PROC FAR
			pushad
			mov     eax,13
			jmp     VM_handler
_h13                    ENDP                            

_h13_bis                PROC FAR
			pushad
			;
			; Send EOI for 8086 coprocessor trick
			; Send 0H on Coprocessor port 0F0H
			;
			xor     al,al
			out     0F0H,al
			mov     al,020H
			out     0A0H,al         
			out     020H,al
			push    ds
			push    ss
			push    es
			push    fs
			push    gs
			mov	ax,PM_APPLDATA_SEL
			mov	es,ax
			mov	ds,ax
			mov     eax,MATH_EXC
			push    eax
			call    _act_exc
			add     esp,4                   
			pop     gs
			pop     fs
			pop     es
			pop     ss
			pop     ds
			popad
			ret
_h13_bis                ENDP                            

_h14                    PROC FAR
			pushad
			mov     eax,14
			jmp     VM_handler
_h14                    ENDP                            

_h15                    PROC FAR
			pushad
			mov     eax,15
			jmp     VM_handler
_h15                    ENDP                            

;
; The VM_handler process the request using the kernel function act_int()
; Then sends EOI & schedules any eventual new task!
;
VM_handler              LABEL NEAR
			; 
			; We do not know what is the DS value
			; Then we save it & set it correctly
			;
			push    ds
			push    ss
			push    es
			push    fs
			push    gs              
			; But we first transfer to the _act_int
			; the interrupt number which is required
			; as second argument
			push    eax
			mov     ax,PM_APPLDATA_SEL
			mov     es,ax
			mov     ds,ax
			;
			; Now save the actual context on stack
			; to pass it to _act_int (C caling convention)
			; CLD is necessary when calling a C function
			;
			str     ax
			push    eax
			cld
			call    _act_int
			;
			; AX now contains the new context value!
			;
			add     esp,4   ; Drop old context value
			pop     ebx     ; Store in EBX the Int number
			;
			; Send EOI to master & slave (if necessary) PIC
			;
			push    eax
			mov     al,EOI
			cmp     ebx,08h
			jb      eoi_master
eoi_slave:              out     OCW2_S,al
eoi_master:             out     OCW2_M,al
			;
			; Resume the return value of _act_int
			; & do the context switch if necessary!
			;
			pop     eax
			cmp     ax,JmpSel
			je      NoPreempt3
			mov     JmpSel,ax
			jmp     PWORD PTR JmpZone ; DISPATCH!
NoPreempt3:             pop     gs
			pop     fs
			pop     es
			pop     ss
			pop     ds
			popad
			iretd

_exc0			PROC FAR
			pushad
			mov eax,0
			jmp VM_handler2
_exc0			ENDP
				
_exc1			PROC FAR
			pushad
			mov eax,1
			jmp VM_handler2
_exc1			ENDP

_exc2			PROC FAR
			pushad
			mov eax,2
			jmp VM_handler2
_exc2			ENDP
				
_exc3			PROC FAR
			pushad
			mov eax,3
			jmp VM_handler2
_exc3			ENDP
				
_exc4			PROC FAR
			pushad
			mov eax,4
			jmp VM_handler2
_exc4			ENDP
				
_exc5			PROC FAR
			pushad
			mov eax,5
			jmp VM_handler2
_exc5			ENDP
				
_exc6			PROC FAR
			pushad
			mov eax,6
			jmp VM_handler2
_exc6			ENDP
				
_exc7			PROC FAR
			pushad
			push    ds
			push    ss
			push    es
			push    fs
			push    gs
			mov     ax,PM_APPLDATA_SEL
			mov     es,ax
			mov     ds,ax
			cld
			call	_VM_FPU_hook
                        pop     gs
			pop     fs
			pop     es
			pop     ss
			pop     ds
			popad
			iretd				
_exc7			ENDP
				
_exc8			PROC FAR
			pushad
			mov eax,8
			jmp VM_handler2
_exc8			ENDP
				
_exc9			PROC FAR
			pushad
			mov eax,9
			jmp VM_handler2
_exc9			ENDP
				
_exc10			PROC FAR
			pushad
			mov eax,10
			jmp VM_handler2
_exc10			ENDP
				
_exc11			PROC FAR
			pushad
			mov eax,11
			jmp VM_handler2
_exc11			ENDP

_exc12			PROC FAR
			pushad
			mov eax,12
			jmp VM_handler2
_exc12			ENDP
				
_exc13			PROC FAR
			pushad
			mov eax,13
			jmp VM_handler2
_exc13			ENDP
				
_exc14			PROC FAR
			pushad
			mov eax,14
			jmp VM_handler2
_exc14			ENDP
				
_exc15			PROC FAR
			pushad
			mov eax,15
			jmp VM_handler2
_exc15			ENDP
				
_exc16			PROC FAR
			pushad
			mov eax,16
			jmp VM_handler2
_exc16			ENDP

VM_handler2		LABEL NEAR
			push    ds
			push    ss
			push    es
			push    fs
			push    gs
			;
			; After saving Segment Register
			; We push the exception id onto the stack
			; and we call the exc_handler
			;
			push    eax
			mov     ax,PM_APPLDATA_SEL
			mov     es,ax
			mov     ds,ax
			cld
			call	_VM_exc_hook
			add	esp,4
                        pop     gs
			pop     fs
			pop     es
			pop     ss
			pop     ds
			popad
			iretd				
_TEXT           ENDS

_DATA           SEGMENT DWORD PUBLIC USE32 'DATA'
		PUBLIC _VM_clock
		PUBLIC _SafeStack
		;
		; VM clock tick is stored here
		;
_VM_clock	DD	(0)
		;
		; Used as JMP entry point to check if a real
		; context switch is necessary
		;
JmpZone         LABEL   PWORD
JmpOffset       DD      (0)
JmpSel          DW      (0)
		; 
		; Safe stack area for aborts
		;
		DD      2048 dup (0)
SafeStack       LABEL   NEAR
_SafeStack      LABEL   NEAR
_DATA           ENDS

		END
