/* Project:      HARTIK 3.0                                      */
/* Description:  Hard Real TIme Kernel for 386 & higher machines */
/* Author:       Gerardo Lamastra                        	 */
/* Date:         9/5/96                                          */
/* Revision:     2.0						 */

/* Module: VMCx32a.S						 */
/* Virtual Machine level for 386 & upper compatible processor 	 */

.title	"VmCx32a.S"

#include "sel.h"

/* This is also present in Exc.H */
#define CLOCK_OVERRUN 	64
#define MATH_EXC        1    

.data          
		.globl _VM_clock
		.globl _SafeStack
		
		/* Used as JMP entry point to check if a real	*/
		/* context switch is necessary			*/
		
_VM_clock:	.int	0

JmpZone:
JmpOffset:	
.word	0
.word	0
JmpSel:	
.word	0		 
		/* Safe stack area for aborts	*/		
.space		4096,0
SafeStack:
_SafeStack:


#ifdef __LINUX__
#define _wake_up wake_up
#define _act_int act_int
#define _act_exc act_exc
#define _sys_abort_tail sys_abort_tail
#define _VM_exc_hook VM_exc_hook
#define _VM_FPU_hook VM_FPU_hook
#endif

.extern  _wake_up        
.extern  _act_int        
.extern  _act_exc        
.extern  _sys_abort_tail
.extern  _VM_exc_hook
.extern  _VM_FPU_hook

.text
		.globl _VM_timer
		.globl _VM_context_save
		.globl _VM_context_change
		.globl _VM_context_load
		.globl _VM_abort

		.globl _h01
		.globl _h02
		.globl _h03
		.globl _h04
		.globl _h05
		.globl _h06
		.globl _h07
		.globl _h08
		.globl _h09
		.globl _h10
		.globl _h11
		.globl _h12
		.globl _h13
		.globl _h13_bis
		.globl _h14
		.globl _h15
		.globl _exc0
		.globl _exc1
		.globl _exc2
		.globl _exc3
		.globl _exc4
		.globl _exc5
		.globl _exc6
		.globl _exc7
		.globl _exc8
		.globl _exc9
		.globl _exc10
		.globl _exc11
		.globl _exc12
		.globl _exc13
		.globl _exc14
		.globl _exc15
		.globl _exc16

		.globl _init_TR

/* This function assign a value to the TASK register of 386 architecture */
/* We MUST do this BEFORE we use the HW multitask			 */

_init_TR:	pushl	%ebp
		movl	%esp,%ebp
		movl	8(%ebp),%eax
		ltrw	%ax
		movw	%ax,JmpSel
		popl	%ebp
		ret
		
/* CONTEXT __cdecl VM_context_save(void);				  */
/* The context is returned into AX; Interrupts are also cleared as we are */
/* entering into kernel primitives					  */

_VM_context_save:	cli
			xorl     %eax,%eax
			strw     %ax
			ret

/* void __cdecl VM_context_change(CONTEXT c)				*/
/* Use 386 task switch ability. This is the last call of any preemption */
/* generating primitive; when the original task is re-activated the 	*/
/* interrupt flag is restored with STI					*/
/* In 32 bit systems, VM_context_load is an alias for VM_context_change!*/

_VM_context_load:
_VM_context_change:	pushl   %ebp
			movl    %esp,%ebp
			movl	$(PM_APPLDATA_SEL),%eax
			movw	%ax,%ds
			movw	%ax,%es
			movl    8(%ebp),%eax
			cmpw    JmpSel,%ax
			je      NoPreempt
			movw    %ax,JmpSel
			ljmp    JmpZone
NoPreempt:              popl	%ebp
			sti
			ret

_VM_abort:		/* As we are terminating we cannnot handle */
			/* any other interrupt!			   */
			cli
			/* Get argument */
			movl    4(%esp),%eax
			/* Switch to safe stack */
			movl    $(SafeStack),%esp
			/* Push argument	*/
			pushl	%eax
			/* Call sys_abort(code) */
			call    _sys_abort_tail

/* This is the timer handler; it reloads for safety the DS register; this   */
/* prevents from mess when timer interrupts linear access to memory (not in */
/* ELF address space); then EOI is sent in order to detect timer overrun    */
/* The high level kernel procedure wake_up() is called to perform the 	    */
/* preeption at higher level (process descriptos); the resulting context    */
/* if different from the old one is used to trigger the task activation.    */

_VM_timer:		pushal
			pushw	%ds
			pushw	%es
			pushw	%fs
			pushw	%gs
			
			movl    $(PM_APPLDATA_SEL),%eax
			movw    %ax,%ds
			movw    %ax,%es

			/* Send EOI to master PIC		*/
			/* to perform later the overrun test	*/
			movb    $0x20,%al
			movl	$0x20,%edx
			outb    %al,%dx
			
			/* Call wake_up(actual_context) */
			
			cld
			movl	_VM_clock,%eax
			incl 	%eax
			movl	%eax,_VM_clock
			
			xorl    %eax,%eax
			strw    %ax
			pushl   %eax
			call    _wake_up
			addl    $4,%esp
			pushl   %eax     /* Save result on stack */
			
			/* This is the overrun test		 */
			/* Do it after sending EOI to master PIC */
					
			movb    $0x0A,%al
			movl	$0x020,%edx
			outb    %al,%dx
			inb     %dx,%al
			testb   $1,%al
			jz      Timer_OK
			movl    $(CLOCK_OVERRUN),%eax
			pushl	%eax
			call    _VM_abort

Timer_OK:               popl    %eax             /* Recover wake_up result */
			cmpw    %ax,JmpSel
			je      NoPreempt2
			movw    %ax,JmpSel
			ljmp    JmpZone  /* DISPATCH! */
NoPreempt2:             popw	%gs
			popw	%fs
			popw	%es
			popw	%ds
			popal
			iret


/* These are the hardware handlers; they all jump to VM_handler setting */
/* the interrupt number into EAX & save the registers on stack		*/

_h01:			pushal
			movl    $1,%eax
			jmp     VM_handler

_h02:                   pushal
			movl    $2,%eax
			jmp     VM_handler

_h03:                   pushal
			movl    $3,%eax
			jmp     VM_handler

_h04:                   pushal
			movl    $4,%eax
			jmp     VM_handler

_h05:			pushal
			movl    $5,%eax
			jmp     VM_handler

_h06:                   pushal
			movl    $6,%eax
			jmp     VM_handler

_h07:                   pushal
			movl    $7,%eax
			jmp     VM_handler

_h08:                   pushal
			movl    $8,%eax
			jmp     VM_handler

_h09:                   pushal
			movl    $9,%eax
			jmp     VM_handler

_h10:                   pushal
			movl    $10,%eax
			jmp     VM_handler

_h11:                   pushal
			movl    $11,%eax
			jmp     VM_handler

_h12:                   pushal
			movl    $12,%eax
			jmp     VM_handler

_h13:                   pushal
			movl    $13,%eax
			jmp     VM_handler

_h13_bis:               pushal
			
			/* Send EOI for 8086 coprocessor trick	*/
			/* Send 0H on Coprocessor port 0F0H	*/
			
			xorb    %al,%al		
			outb    %al,$0x0F0
			movb    $0x020,%al
			outb    %al,$0x0A0
			outb    %al,$0x020
			pushl   %ds
			pushl   %ss
			pushl   %es
			pushl   %fs
			pushl   %gs
			movl	$(PM_APPLDATA_SEL),%eax
			movw	%ax,%es
			movw	%ax,%ds
			movl    $(MATH_EXC),%eax
			pushl	%eax
			call    _act_exc
			addl    $4,%esp
			popl    %gs
			popl    %fs
			popl    %es
			popl    %ss
			popl    %ds
			popal
			ret

_h14:                   pushal
			movl    $14,%eax
			jmp     VM_handler

_h15:                   pushal
			movl    $15,%eax
			jmp     VM_handler


/* The VM_handler process the request using the kernel function act_int() */
/* Then sends EOI & schedules any eventual new task!			  */

VM_handler:             
			/* We do not know what is the DS value	*/
			/* Then we save it & set it correctly  	*/
			
			pushl	%ds
			pushl	%ss
			pushl	%es
			pushl	%fs
			pushl	%gs              
			/* But we first transfer to the _act_int  */
			/* the interrupt number which is required */
			/* as second argument			  */
			pushl   %eax
			movl    $(PM_APPLDATA_SEL),%eax
			movw    %ax,%es
			mov     %ax,%ds
			
			/* Now save the actual context on stack		*/
			/* to pass it to _act_int (C caling convention)	*/
			/* CLD is necessary when calling a C function	*/
			
			strw	%ax
			pushl	%eax
			cld
			call    _act_int
			
			/* AX now contains the new context value!	*/
			
			addl	$4,%esp /* Drop old context value 	*/
			popl	%ebx    /* Store in EBX the Int number	*/
			
			/* Send EOI to master & slave (if necessary) PIC */
			
			pushl	%eax
			movb	$0x20,%al
			cmpl	$0x08,%ebx
			jb	eoi_master
eoi_slave:              movl	$0xA0,%edx
			outb	%al,%dx
eoi_master:             movl	$0x20,%edx
			outb	%al,%dx
			
			/* Resume the return value of _act_int 		*/
			/* & do the context switch if necessary!	*/
			
			popl	%eax
			cmpw    JmpSel,%ax
			je      NoPreempt3
			movw    %ax,JmpSel
			ljmp    JmpZone 
			
NoPreempt3:             popl	%gs
			popl	%fs
			popl	%es
			popl	%ss
			popl	%ds
			popal
			iret
				
_exc0:			pushal
			mov	$0,%eax
			jmp	VM_handler2
			
_exc1:			pushal
			mov	$1,%eax
			jmp	VM_handler2
			
_exc2:			pushal
			mov	$2,%eax
			jmp	VM_handler2
			
_exc3:			pushal
			mov	$3,%eax
			jmp	VM_handler2
			
_exc4:			pushal
			mov	$4,%eax
			jmp	VM_handler2
			
_exc5:			pushal
			mov	$5,%eax
			jmp	VM_handler2
			
_exc6:			pushal
			mov	$6,%eax
			jmp	VM_handler2
			
_exc7:			pushal
            		pushl	%ds
			pushl	%ss
			pushl	%es
			pushl	%fs
			pushl	%gs
			movl	$(PM_APPLDATA_SEL),%eax
			movl	%ax,%es
			movl	%ax,%ds
			cld
			call	_VM_FPU_hook
			popl	%gs
			popl	%fs
			popl	%es
			popl	%ss
			popl	%ds
			popal
			iret
			
_exc8:			pushal
			mov	$8,%eax
			jmp	VM_handler2
			
_exc9:			pushal
			mov	$9,%eax
			jmp	VM_handler2
			
_exc10:			pushal
			mov	$10,%eax
			jmp	VM_handler2
			
_exc11:			pushal
			mov	$11,%eax
			jmp	VM_handler2
			
_exc12:			pushal
			mov	$12,%eax
			jmp	VM_handler2
			
_exc13:			pushal
			mov	$13,%eax
			jmp	VM_handler2
			
_exc14:			pushal
			mov	$14,%eax
			jmp	VM_handler2
			
_exc15:			pushal
			mov	$15,%eax
			jmp	VM_handler2
			
_exc16:			pushal
			mov	$16,%eax
			jmp	VM_handler2

VM_handler2:		pushl	%ds
			pushl	%ss
			pushl	%es
			pushl	%fs
			pushl	%gs
			pushl	%eax
			movl	$(PM_APPLDATA_SEL),%eax
			movl	%ax,%ds
			movl	%ax,%es
			call	_VM_exc_hook
			addl	$4,%esp
			popl	%gs
			popl	%fs
			popl	%es
			popl	%ss
			popl	%ds
			popal
			iret
			

