/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:      Gerardo Lamastra                                */
/* Date:        9/5/96                                          */
/* Revision:    Beta 1.0                                        */

/* File: VMCtx32a.C                              */
/* Virtual machine layer/32 bit 386 architecture */
/* CPU Context Switch management functions	 */

#include <stdlib.h>
#include <string.h>
#include <xsys.h>
#include <mem.h>
#include <cons.h>

#include "tss-ctx.h"
#include "vm.h"
#include "kern.h"

TSS  TSS_table[TSSMax];
WORD TSS_control[TSSMax];
BYTE VM_FPU_stdctx[FPU_CONTEXT_SIZE];

/* Just a debugging function; it dumps the status of the TSS */
void dump_TSS(WORD sel)
{
    BYTE acc,gran;
    DWORD base,lim;
    cprintf("TR %x\n",sel);
    sel = TSSsel2index(sel);
    cprintf("SS:SP %x:%lx\n",TSS_table[sel].ss,TSS_table[sel].esp);
    cprintf("Stack0 : %x:%lx\n",TSS_table[sel].ss0,TSS_table[sel].esp0);
    cprintf("Stack1 : %x:%lx\n",TSS_table[sel].ss1,TSS_table[sel].esp1);
    cprintf("Stack2 : %x:%lx\n",TSS_table[sel].ss2,TSS_table[sel].esp2);
    cprintf("CS : %x DS : %x\n",TSS_table[sel].cs,TSS_table[sel].ds);
    sel = TSSindex2sel(sel);
    cprintf("Descriptor [%x] Info",sel);
    base = GDT_read(sel,&lim,&acc,&gran);
    cprintf("Base : %lx Lim : %lx Acc : %x Gran %x\n",base,lim,(unsigned)(acc),(unsigned)(gran));
}

/* Initialize context -> TSS in 32 bit */
CONTEXT __cdecl VM_context_create(void __cdecl (*task)(),BYTE *stack,
				WORD parm,void (*killer)(void),WORD control)
{
    CONTEXT index = 0;
    DWORD *stack_ptr = (DWORD *)stack;
    /* Push onto stack the input parameter              */
    /* And the entry point to the task killer procedure */
    stack_ptr--;
    *stack_ptr = (DWORD)(parm);
    stack_ptr--;
    *stack_ptr = (DWORD)(killer);
    /* Find a free TSS */
    while ((TSS_control[index] & TSS_USED) && (index < TSSMain)) index++;
    /* This exception would signal an error */
    if (index >= TSSMain) act_exc(NO_MORE_HW_DESC);
    TSS_control[index] |= (TSS_USED | control);
    /* Fill the TSS structure */
    /* No explicit protection; use only one stack */
    TSS_table[index].esp0 = (DWORD)(stack_ptr);
    TSS_table[index].esp1 = 0;
    TSS_table[index].esp2 = 0;
    TSS_table[index].ss0 = get_DS();
    TSS_table[index].ss1 = 0;
    TSS_table[index].ss2 = 0;
    /* No paging activated */
    TSS_table[index].cr3 = 0;
    /* Task entry point */
    TSS_table[index].eip = (DWORD)(task);
    /* Need only to unmask interrupts */
    TSS_table[index].eflags = 0x00000200;
    TSS_table[index].eax = 0;
    TSS_table[index].ebx = 0;
    TSS_table[index].ecx = 0;
    TSS_table[index].edx = 0;
    TSS_table[index].esi = 0;
    TSS_table[index].edi = 0;
    TSS_table[index].esp = (DWORD)(stack_ptr);
    TSS_table[index].ebp = (DWORD)(stack_ptr);
    TSS_table[index].cs = get_CS();
    TSS_table[index].es = get_DS();
    TSS_table[index].ds = get_DS();
    TSS_table[index].ss = get_DS();
    TSS_table[index].gs = X_FLATDATA_SEL;
    TSS_table[index].fs = X_FLATDATA_SEL;
    /* Use only the GDT */
    TSS_table[index].ldt = 0;
    TSS_table[index].trap = 0;
    TSS_table[index].io_base = 0;
    /* Fill in the coprocessor status */
    memcpy(TSS_table[index].ctx_FPU,VM_FPU_stdctx,FPU_CONTEXT_SIZE);
    /* Convert the index into a valid selector  */
    /* Which really represent CONTEXT           */
    return(TSSindex2sel(index));
}

/* Release the used TSS */

void __cdecl VM_context_delete(CONTEXT c)
{
    int index = TSSsel2index(c);
    TSS_control[index] = 0;
}

/* Well this is used for debug purposes mainly; as the context is an       */
/* abstract type, we must provide some function to convert in into a       */
/* printable form; anyway this depend from the architecture as the context */

char *VM_context_sprintf(char *str,CONTEXT c)
{
    sprintf(str,"%x (Hex)",c);
    return(str);
}
