/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:      Gerardo Lamastra                                */
/* Date:        9/5/96                                          */
/* Revision:    Beta 2.0                                        */

/* File: VMDemo.C                                	*/
/* This is a pseudo-kernel stub for easy debugging 	*/

#include <math.h> /* For coprocessor context-switch test */
#include <string.h>
#include <xsys.h>
#include <cons.h>

#include "tss-ctx.h"
#include "vm.h"
#include "kern.h"

#define __DYN_STACK__

/* Sustain VM_FPU_* debugging */
long int t1_called = 0;
long int t2_called = 0;
long int wu_called = 0;
long int ai_called = 0;
#ifdef __FPU_DEBUG__
extern long int ndp_called;
extern long int ndp_switched;
#else
long int ndp_called = -1;
long int ndp_switched = -1;
#endif

/* For stack allocation checking */
void *x_s1;
void *x_s2;
#ifndef __DYN_STACK__
BYTE stack1[STACK_SIZE];
BYTE stack2[STACK_SIZE];
#else
BYTE *stack1;
BYTE *stack2;
#endif
void *smain;
void *sp;
/* Trivial process descriptor */
typedef struct vmtest_proc_des {
    CONTEXT context;
    char name[20];
    WORD valid;
} VMTEST_PROC_DES;

/* For FPU ctx switch check */
double y1_v = 0,y2_v = 0;


/* Here we begin with the PseudoKernel */
VMTEST_PROC_DES VM_proc_table[3];
PID VM_pseudo_exec = 0;

void scheduler(void)
{
    /* Dummy round-robin scheduler */
    do {
	VM_pseudo_exec = (VM_pseudo_exec+1) % 3;
    } while (!VM_proc_table[VM_pseudo_exec].valid);
}

CONTEXT __cdecl wake_up(CONTEXT exit_ctx)
{
    static wake_up_mark = '*';
    lmempokeb(0xB8000 + 20,wake_up_mark);
    wake_up_mark ^= '*'; 
    VM_proc_table[VM_pseudo_exec].context = exit_ctx;
    scheduler();
    wu_called++;
    return(VM_proc_table[VM_pseudo_exec].context);
}

void __cdecl act_exc(int code)
{
    VM_end();
    exit(code);
}

void end_period(void)
{
    VM_proc_table[VM_pseudo_exec].context = VM_context_save();
    scheduler(); 
    VM_context_change(VM_proc_table[VM_pseudo_exec].context);
}

void kill(WORD id)
{
    WORD autokill;
    if ((autokill = (id == VM_pseudo_exec))) VM_disable();
    else VM_proc_table[VM_pseudo_exec].context = VM_context_save();
    VM_proc_table[id].valid = 0;
    scheduler();
    if (autokill) VM_context_load(VM_proc_table[VM_pseudo_exec].context);
    else VM_context_change(VM_proc_table[VM_pseudo_exec].context);
}

void __cdecl task1(WORD x)
{
    static WORD task1_mark = '1';
    for (;;) {
	t1_called++;
	lmempokeb(0xB8000 + 10,task1_mark);
	task1_mark ^= x;
	y1_v += .25;
	end_period();
    }
}

void __cdecl task2(WORD x)
{
    static task2_mark = '2';
    for (;;) {
	t2_called++;
	lmempokeb(0xB8000+8,task2_mark);
	task2_mark ^= x;
	y2_v += 1/3.0;
    }
}

void killer(void)
{
    VM_disable();
    VM_proc_table[VM_pseudo_exec].valid = 0;
    scheduler();   
    VM_context_load(VM_proc_table[VM_pseudo_exec].context);
}

CONTEXT __cdecl act_int(CONTEXT c,BYTE n)
{
    static act_int_mark = '0';
    lmempokeb(0xB8000 + 20,act_int_mark + n);
    ai_called++;
    return(c);
}

int sys_abort(int code)
{
    VM_disable();
    sp = (void *)get_SP();
    VM_abort(code);
    /* Never reach this! Just to shut up compiler warning... 	*/
    /* In the general case, a sys_abort can resume...		*/
    return(1);
}

void __cdecl sys_abort_tail(int code)
{	
	#ifndef __LINUX__
    extern int SafeStack;
	#else
	extern int _SafeStack;
	extern int SafeStack __attribute__((weak,alias("_SafeStack")));
	#endif	
    VM_end();
    VM_enable();
    VM_printf("Code: %d\nException\nStack : Orig->%p Before VM_abort->%p Now->%p\n",code,smain,sp,(void *)get_SP());
    VM_printf("SafeStack: %lp\n",&SafeStack);
    VM_printf("Stack 1 : %lp\n",x_s1);
    VM_printf("Stack 2 : %lp\n",x_s2);
    exit(1);
}

main()
{
    unsigned long i;
    char str[80];
    TIME now;
    void *orig;
    
    VM_disable();
    #if defined(TSS)
    VM_printf("Init TR : %nx\n",get_TR())
    #endif
    VM_printf("Init Stack : %p\n",(void *)get_SP());
    orig = (void *)get_SP();
    VM_init(1000);

    #ifdef __DYN_STACK__
	stack1 = VM_alloc(STACK_SIZE);
	stack2 = VM_alloc(STACK_SIZE);
	VM_printf("Stack status : \n");
	VM_printf("Stack1 : %p\n",stack1);
	VM_printf("Stack2 : %p\n",stack2);
	VM_mem_dump();
    #endif
    smain = (void *)get_SP();
    VM_printf("Smain --> %p\n",smain);
    x_s1 = &stack1[STACK_SIZE];
    x_s2 = &stack2[STACK_SIZE];

    VM_proc_table[0].valid = 1;

    strncpy(VM_proc_table[2].name,"Stronz2",19);
    VM_proc_table[2].valid = 1;
    VM_proc_table[2].context = VM_context_create(task2,&stack2[STACK_SIZE],'2',killer,USE_FPU);
    VM_context_sprintf(str,VM_proc_table[2].context);
    VM_printf("Context #2 : %s! Main Stack is %p\n",str,(void *)get_SP());

    strncpy(VM_proc_table[1].name,"Stronz1",19);
    VM_proc_table[1].valid = 1;
    VM_proc_table[1].context = VM_context_create(task1,&stack1[STACK_SIZE],'1',killer,USE_FPU);
    VM_context_sprintf(str,VM_proc_table[1].context);
    VM_printf("Context #1 : %s! Main Stack is %p\n",str,(void *)get_SP());
    VM_pseudo_exec = 0;
    now = 0;
    VM_enable();
    while (now <= 4000000) {
	VM_disable();
	now = VM_timer_read();
	VM_enable();
    }

    VM_disable();
    VM_end();
    VM_enable();
    sp = (void *)get_SP();
    VM_printf("End reached!\n");
    VM_printf("Actual stack : %p - ",sp);
    VM_printf("Main stack : %p\n",smain);
    VM_printf("Check if same : %s\n",smain == sp ? "Ok :-)" : "No :-(");
    VM_printf("Stack 1 : %p - ",x_s1);
    VM_printf("Stack 2 : %p\n",x_s2);
    VM_printf("DS:%x CS:%x\n",get_DS(),get_CS());
    VM_printf("T1 : %ld\tT2 : %ld\tWU : %ld\tAI %ld\n",t1_called,t2_called,wu_called,ai_called);
    VM_printf("NDP(called/switched) : %ld/%ld\n(A value of -1 means unused!)\n",ndp_called,ndp_switched);

    #if defined(TSS)
    #error OK
	VM_printf("TSS size : %d\n",sizeof(TSS));
	VM_printf("Memory base : %x\n",appl2linear((void *)0L));
	dump_TSS(TSSindex2sel(TSSMain));
    #endif

    VM_printf("y1_v : %g\n",y1_v);
    i = (unsigned long)(y1_v);
    VM_printf("Y1(int) : %ld (T1:%ld)\n",i,t1_called/4);
    VM_printf("y2_v : %g\n",y2_v);
    i = (unsigned long)(y2_v);
    VM_printf("Y2(int) : %ld (T2:%ld)\n",i,t2_called/3);

    exit(1);
}
