/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/* Date:        9/5/96                                          */

/* File: 	VmFPU.C						*/
/* Revision:    Beta 2.0                                        */

/*
   FPU Context switch & management functions!
   WC 16 Module
*/

#include <mem.h>

#include "vm.h"
#include "kern.h"

/* These are declared in VmCx16b.C */
extern BYTE *VM_CPU[MAX_PROC]; /* Global CPU context management */
extern CONTEXT VM_has_CPU;
extern WORD VM_control[MAX_PROC];

BYTE VM_FPU_savearea[FPU_CTX_SIZE]; /* Global FPU scratch SaveArea */

/* FPU context management */
CONTEXT VM_has_FPU = MAIN_INDEX;
BYTE VM_FPU_buffer[MAX_PROC][FPU_CTX_SIZE];
BYTE VM_FPU_std[FPU_CTX_SIZE];

#ifdef __FPU_DEBUG__
    long int ndp_called = 0,ndp_switched = 0;
#endif

/* As the 8086 does not have an hardware mechanism to support task      */
/* switch, also the FPU context switch is implemented via software.     */
/* When a preemption occurs, if the task is marked as a MATH task, the  */
/* preemption routine will save/restore the FPU context.		*/
/* The hook is called whenever a FPU context switch is necessarty	*/

void __cdecl VM_FPU_hook(void)
{
    if (VM_control[VM_has_CPU] & USE_FPU && VM_has_FPU != VM_has_CPU) {
	if (VM_has_FPU != NIL) {
	    #ifdef __FPU_DEBUG__
		ndp_called++;
	    #endif
	    VM_FPU_save();
	    memcpy(&VM_FPU_buffer[VM_has_FPU][0],VM_FPU_savearea,FPU_CTX_SIZE);
	}
	#ifdef __FPU_DEBUG__
	    ndp_switched++;
	#endif
	VM_has_FPU = VM_has_CPU;
	memcpy(VM_FPU_savearea,&VM_FPU_buffer[VM_has_FPU][0],FPU_CTX_SIZE);
	VM_FPU_restore();
    }
}

CONTEXT VM_FPU_get_task(void)
{
    return(VM_has_FPU);
}

