/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/* Date:        9/5/96                                          */

/* File: 	VmFPU32.C					*/
/* Revision:    Beta 2.0                                        */

/*
   FPU Context switch & management functions!
   Generic 32 bit module
*/

#include <mem.h>

#include "tss-ctx.h"
#include "vm.h"
#include "kern.h"

extern TSS TSS_table[];

BYTE VM_FPU_savearea[FPU_CONTEXT_SIZE]; /* Global FPU scratch SaveArea */
#ifdef __FPU_DEBUG__
    long int ndp_called = 0,ndp_switched = 0;
#endif

/* FPU context management */
static CONTEXT VM_has_FPU = TSSMain;

/* As the 8086 does not have an hardware mechanism to support task      */
/* switch, also the FPU context switch is implemented via software.     */
/* When a preemption occurs, if the task is marked as a MATH task, the  */
/* preemption routine will save/restore the FPU context.		*/
/* The hook is called whenever a FPU context switch is necessarty	*/

void __cdecl VM_FPU_hook(void)
{
    clts();
    #ifdef __FPU_DEBUG__
	ndp_called++;
    #endif
    if (VM_has_FPU == TSSsel2index(get_TR())) return;
    #ifdef __FPU_DEBUG__
	ndp_switched++;
    #endif
    VM_FPU_save();
    memcpy(TSS_table[VM_has_FPU].ctx_FPU,VM_FPU_savearea,FPU_CONTEXT_SIZE);
    VM_has_FPU = TSSsel2index(get_TR());
    memcpy(VM_FPU_savearea,TSS_table[VM_has_FPU].ctx_FPU,FPU_CONTEXT_SIZE);
    VM_FPU_restore();
    return;
}

CONTEXT VM_FPU_get_task(void)
{
    return(VM_has_FPU);
}
