/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/* Date:        9/5/96                                          */

/* File: 	VmInit16.C					*/
/* Revision:    2.0G                                            */

/* C Code to implement 8086 virtual machine                     */

#include <mem.h>
#include <string.h>
#include <i86.h>
#include <dos.h>

#include <cons.h>
#include "vm.h"
#include "kern.h"

/* #define __INT_DISABLED__ */

#define IDT_place(where,intnum,code) *(where + intnum) = code
#define bit_on(v,b)     ((v) |= (1 << (b)))
#define bit_off(v,b)    ((v) &= ~(1 << (b)))

static BYTE VM_FPU_original[FPU_CTX_SIZE];

/* These are declared in VmCx16b.C */
extern BYTE *VM_CPU[MAX_PROC]; /* Global CPU context management */
extern CONTEXT VM_has_CPU;
extern WORD VM_control[MAX_PROC];

/* FPU context management! */
/* They are declared in VMFPU.C */
extern BYTE VM_FPU_savearea[FPU_CTX_SIZE]; /* Global FPU scratch SaveArea */
extern CONTEXT VM_has_FPU;
extern BYTE VM_FPU_buffer[MAX_PROC][FPU_CTX_SIZE];
extern BYTE VM_FPU_std[FPU_CTX_SIZE];

/* These are declared in VMCx16a.Asm */
/* The 32-bit clock is handled in the VM_timer assembly subroutine */
extern DWORD VM_clock;

/* The Assembler interface for interrupt calls */
extern void __cdecl VM_timer(void);     /* Timer handler 	 */
extern void __cdecl h01(void);          /* Other hw int handlers */
extern void __cdecl h02(void);          
extern void __cdecl h03(void);
extern void __cdecl h04(void);
extern void __cdecl h05(void);
extern void __cdecl h06(void);
extern void __cdecl h07(void);
extern void __cdecl h08(void);
extern void __cdecl h09(void);
extern void __cdecl h10(void);
extern void __cdecl h11(void);
extern void __cdecl h12(void);
extern void __cdecl h13(void);
extern void __cdecl h14(void);
extern void __cdecl h15(void);

extern void __cdecl exc0(void);
extern void __cdecl exc1(void);
extern void __cdecl exc2(void);
extern void __cdecl exc3(void);
extern void __cdecl exc4(void);

static INTERRUPT dos_IDT[256];
static INTERRUPT *real_IDT;
static BYTE old_master_mask;
static BYTE old_slave_mask;
static WORD memory_base;
static DWORD memory_size;

DWORD VM_tick;
WORD VM_time_const;
VM_ARCH VM_arch;

static BYTE actual_master_mask = 0xFE;
static BYTE actual_slave_mask = 0xFF;

void VM_init(TIME t)
{
    register i;
    union REGS in,out;
    BYTE v;

    /* In VMCPU16.Asm! Used to identify CPU & FPU */
    extern BYTE X86_cpu;
    extern BYTE X86_fpu;
    extern BYTE X86_cpuidFlag;
    extern BYTE X86_vendor;
    extern DWORD X86_signature;
    extern DWORD X86_features1;
    extern DWORD X86_features2;
    extern DWORD X86_features3;
    void __cdecl X86_get_CPU();
    void __cdecl X86_get_FPU();

    /* A consistency check! As MAIN_INDEX is also present in		*/
    /* VMCx16a.Asm, check it is the same in C <-> Asm!!			*/
    /* I know it is a dirty joke, but is saves a lot of debugging!!! 	*/    
    if (VM_has_CPU != MAIN_INDEX) {
	VM_printf("VM Internal Error!\nMAX_PROC different in VmCx16a.Asm & Const.h\nFix it and recompile the VM & Kernel\n");
	exit(-1);
    }
    X86_get_CPU();
    X86_get_FPU();
    VM_arch.x86.arch = __VM_ARCH__;
    VM_arch.x86.cpu = X86_cpu;
    VM_arch.x86.fpu = X86_fpu;
    VM_arch.x86.vendor = &X86_vendor;
    /* TODO! Need to map featuresXXX & Signature onto VM_arch!  */
    /* TODO! Need to check for CPU bugs!!			*/
    //#ifdef __VM_DEBUG__
	VM_printf("VM Architecture: %s\n",__VM_ARCH__);
	VM_printf("CPU : %u\nFPU : %u\n",X86_cpu,X86_fpu);
	VM_printf("Signature : 0x%lx\nVendor: %s\n",X86_signature,&X86_vendor);
	VM_printf("Features #1: 0x%lx\n",X86_features1);
	VM_printf("Features #2: 0x%lx\n",X86_features2);
	VM_printf("Features #3: 0x%lx\n",X86_features3);
    //#endif
    /* Translate the tick value in usec into a suitable time constant   */
    /* for 8254 timer chip; the chip is driven with a 1.19318 MHz       */
    /* frequency; then the effective frequency is given by the base     */
    /* frequency divided for the time constant; the tick is the inverse */
    /* of this effective frequency (in usec!)                           */
    /* Time-Constant = f_base (MHz) * tick (usec)                       */
    /* If T-C == 0 -> T-C = 65536 (Max available)                       */
    t = t * 1197;
    t = t / 1000;
    VM_tick = t * 1000;
    VM_tick = VM_tick / 1197;
    /* Only for security! This should cause timer overrun */
    if (t == 0) t = 3;
    VM_time_const = (WORD)(t & 0xFFFF);
    /* Get all available DOS memory */
    /* Try allocating 1 MB! Dos will fail returning the largest amount   */
    /* Then use this amount as effective memory to allocate              */
    /* Store the segment value from DOS so we can release memory on exit */
    in.x.ax = 0x4800;
    in.x.bx = 0xFFFF;
    intdos(&in,&out);
    memory_size = (DWORD)(out.x.bx) * 16L;
    in.x.bx = out.x.bx;
    intdos(&in,&out);
    memory_base = out.x.ax;

    /* Init Memory allocator */
    VM_mem_init(MK_FP(memory_base,0),memory_size);
    
    /* Save the DOS interrupt table */
    real_IDT = MK_FP(0,0);
    for (i = 0; i < 256; i++)
	dos_IDT[i] = *real_IDT++;
   
    /* Now load HARTIK interrupt table */
    real_IDT = MK_FP(0,0);  
    #ifndef __INT_DISABLED__
	IDT_place(real_IDT,0x00,exc0);
	IDT_place(real_IDT,0x01,exc1);
	IDT_place(real_IDT,0x02,exc2);
	IDT_place(real_IDT,0x03,exc3);
	IDT_place(real_IDT,0x04,exc4);
	IDT_place(real_IDT,0x08,VM_timer);
	IDT_place(real_IDT,0x09,h01);
	IDT_place(real_IDT,0x0A,h02);
	IDT_place(real_IDT,0x0B,h03);
	IDT_place(real_IDT,0x0C,h04);
	IDT_place(real_IDT,0x0D,h05);
	IDT_place(real_IDT,0x0E,h06);
	IDT_place(real_IDT,0x0F,h07);    
	IDT_place(real_IDT,0x70,h08);
	IDT_place(real_IDT,0x71,h09);
	IDT_place(real_IDT,0x72,h10);
	IDT_place(real_IDT,0x73,h11);
	IDT_place(real_IDT,0x74,h12);
	/*
	  The math fault is signaled through IRQ 13
	  At kernel level it is treated as an exception
	  IDT_place(real_IDT,0x75,h13) is the substituted
	  with the following...
	  With real-mode NO exception is usually available!!
	  
	*/
	#ifdef __FPU_ENABLED__
	    IDT_place(real_IDT,0x75,exc4); /* Math error        */
	#else
	    IDT_place(real_IDT,0x75,h13); /* Standard IRQ 	*/
	#endif

	IDT_place(real_IDT,0x76,h14);
	IDT_place(real_IDT,0x77,h15);
    
	IDT_place(real_IDT,0,exc0);    /* Divide by 0       */
	IDT_place(real_IDT,1,exc1);    /* Single step       */
	IDT_place(real_IDT,2,exc2);    /* NMI               */
	IDT_place(real_IDT,3,exc3);    /* Breakpoint        */
    #endif
    /* Save the DOS interrupt masks & set HARTIK masks */
    old_master_mask = VM_in(0x21);
    old_slave_mask = VM_in(0xA1);

    /* Set up coprocessor extension */
    #ifdef __FPU_ENABLED__
	VM_FPU_save();
	memcpy(VM_FPU_original,VM_FPU_savearea,FPU_CTX_SIZE);
	VM_has_FPU = MAIN_INDEX;
	VM_control[MAIN_INDEX] = USE_FPU | CTX_BUSY;
	/* Fill in the standard FPU register setting */
	VM_FPU_reset();
	VM_FPU_save();
	memcpy(VM_FPU_std,VM_FPU_savearea,FPU_CTX_SIZE);
    #endif /* __FPU_ENABLED__ */
    /* Timer interrupt (IRQ #0) & Coprocessor Error (IRQ #13)   */
    /* Are the only unmasked interrupts                         */
    #ifndef __INT_DISABLED__
	VM_out(0x21,actual_master_mask);
	VM_out(0xA1,actual_slave_mask);
	#ifdef __FPU_ENABLED__
	    VM_irq_unmask(13);
	#endif
	/* Set up timer chip */
	/* Mode: Binary/Mode 2/16 bit Time_const/Counter 0 */
	VM_out(0x43,0x34);
	/* Load Time_const with 2 access to CTR */
	v = (BYTE)(VM_time_const);
	VM_out(0x40,v);
	v = (BYTE)(VM_time_const >> 8);
	VM_out(0x40,v);
    #endif
}

void VM_end(void)
{
    #ifdef __FPU_DEBUG__
        extern long int ndp_called,ndp_switched;
    #endif
    register i;
    union REGS in,out;
    struct SREGS sr;
    /* Reset the timer chip according DOS specification */
    /* Mode: Binary/Mode 3/16 bit Time_const/Counter 0 */
    VM_out(0x43,0x36);
    /* Time_const = 65536; write 0 in CTR */
    VM_out(0x40,0);
    VM_out(0x40,0);

    /* Restore DOS interrupts */
    real_IDT = MK_FP(0,0);
    for (i = 0; i < 256; i++)
	*real_IDT++ = dos_IDT[i];

    /* Restore DOS interrupt masks */
    VM_out(0x21,old_master_mask);
    VM_out(0xA1,old_slave_mask);

    /* Release DOS memory */
    in.x.ax = 0x4900;
    sr.es = memory_base;
    intdosx(&in,&out,&sr);
    #ifdef __FPU_DEBUG__
	cprintf("Ndp_called : %lu\n",ndp_called);
	cprintf("Ndp_switched : %lu\n",ndp_switched);
    #endif
    #ifdef __MEM_DEBUG__
    	/* Memory Allocation info! */
	cprintf("Base : %x\n",memory_base);
	cprintf("Size : %lu\n",memory_size);
    #endif
    /* Restore Coprocessor Status */
    #ifdef __FPU_ENABLED__
    	memcpy(VM_FPU_savearea,VM_FPU_original,FPU_CTX_SIZE);
    	VM_FPU_restore();
    #endif
}
