/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 8086 compatible       */
/* Author:      Gerardo Lamastra                                */
/* Date:        12/7/98                                         */
/* Revision:    Beta 2.0                                        */

/* File: 	VmMem.C						*/
/* Basic Memory Manager Functions:
   A classic fixed partition allocator!
*/

#include <stdlib.h>
#include <string.h>
#include <mem.h>
#include <cons.h>

#include "vm.h"
#include "kern.h"

#define MAX_PARTITION   50 /* Max available partition */

static struct {
	BYTE used;
#ifdef __WC16__
	BYTE __huge *addr;
#else
	BYTE *addr;
#endif	
	DWORD size;
} mem_table[MAX_PARTITION];

void VM_mem_init(void *base, DWORD size)
{
    register i;
    mem_table[0].used = 1;
    mem_table[0].addr = base;
    mem_table[0].size = size;
    for (i = 1; i < MAX_PARTITION; i++) mem_table[i].used = 0;
}

void *VM_alloc(DWORD s)
{
    void *p = NULL;
    int i = 0;

    while (i < MAX_PARTITION && p == NULL) {
	if (mem_table[i].used && (mem_table[i].size >= s))
		p = mem_table[i].addr;
	else i++;
    }
    if (p != NULL) {
	if (mem_table[i].size > s) {
	    mem_table[i].size -= s;
	    mem_table[i].addr += s;
	}
	else mem_table[i].used = FALSE;
    }
    return(p);
}

WORD VM_free(void *p,DWORD s)
{
    register i = 1;
    unsigned i1 = 0, i2 = 0;

    while (i < MAX_PARTITION && ((i1 == 0) || (i2 == 0)) ) {
	if (mem_table[i].used) {
	    if (mem_table[i].addr + mem_table[i].size == p) i1 = i;
	    if (mem_table[i].addr == (BYTE *)(p) + s) i2 = i;
	}
	i++;
    }
    if (i1 != 0 && i2 != 0) {
	mem_table[i1].size += mem_table[i2].size + s;
	mem_table[i2].used = FALSE;
    }
    else if (i1 == 0 && i2 != 0) {
	mem_table[i2].addr = p;
	mem_table[i2].size += s;
    }
    else if (i1 != 0 && i2 == 0) mem_table[i1].size += s;
    else {
	i = 0;
	while (i < MAX_PARTITION && (mem_table[i].used == TRUE)) i++;
	if (i == MAX_PARTITION) return(FALSE);
	mem_table[i].addr = p;
	mem_table[i].size = s;
	mem_table[i].used = TRUE;
    }
    return(TRUE);
} 

void VM_mem_dump(void)
{
    register i;
    for (i = 0; i < MAX_PARTITION; i++) {
	if (mem_table[i].used) cprintf("Entry : [%d] Addr : %p Size : %ld\n",i,mem_table[i].addr,mem_table[i].size);
    }
}
