/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:      Gerardo Lamastra                                */
/* Date:        12/7/1998                                       */
/* Revision:    Beta 2.0                                        */

/* File: VMStuff.C							*/
/* VM_* utility functions: {printf, timer_read, irq_mask, irq_unmask}	*/

#include <stdlib.h>
#include <string.h>
#include <mem.h>

#include "vm.h"
#include "cons.h"

#define bit_on(v,b)     ((v) |= (1 << (b)))
#define bit_off(v,b)    ((v) &= ~(1 << (b)))

/* PIC interrupt mask */
BYTE VM_PIC_master_mask = 0xFE;
BYTE VM_PIC_slave_mask = 0xFE;

/* This is declared in the VMCtx.Asm/S file */
/* Timer 0 counter */
#ifndef __LINUX__
extern DWORD VM_clock;
#else
extern DWORD _VM_clock;
extern DWORD VM_clock __attribute__((weak,alias("_VM_clock")));
#endif

/*
   These are declared in the VmInit[16/32].C File
*/
/* Timer 0 usec base tick */
extern DWORD VM_tick;
/* Timer 0 loaded time constant (= VM_tick * 1.197) */
extern WORD  VM_time_const;

TIME VM_std_timer_read(void)
{
    BYTE str_lsb,str_msb,isr;
    TIME result;
    /* Read Back Command on counter 0 */
    VM_out(0x43,0xD2);
    /* Read the latched value from STR */
    str_lsb = VM_in(0x40);
    str_msb = VM_in(0x40);
    /* Combine the byte values to obtain a word */
    result = ((TIME)str_msb << 8) | str_lsb;
    /* Detect a timer interrupt */
    VM_out(0x20,0x0A);
    isr = VM_in(0x20);
    /* Translate into a time value (in usec) */
    result = (TIME)(VM_time_const)-result;
    result *= VM_tick;
    result /= VM_time_const;
    if ((isr & 1) && result < ((8*VM_tick)/10)) result += VM_tick;
    return(VM_clock*VM_tick + result);
}

/* Pentium & Pentium MMX rdtsc based timer read */
TIME VM_P5_timer_read(void)
{
    return(0);
}

/* Pentium Overdrive & Pentium II rdtsc based timer read 	*/
/* The only difference is that these guys support out-of-order	*/
/* execution, then it is necessary to serialize the execution	*/
/* of the rdtsc instruction!					*/
TIME VM_P6_timer_read(void)
{
    return(0);
}

/* The default timer read function! */
static TIME (*handler)(void) = VM_std_timer_read;
TIME VM_timer_read(void)
{
    return (handler());
}

void VM_timer_set_handler(TIME (*h)(void))
{
    handler = h;
}

void VM_irq_mask(WORD irqno)
{
    /* Cannot mask timer interrupt! */
    if (irqno == 0) return;
    /* Interrupt is on master PIC */
    if (irqno < 8) {
	bit_on(VM_PIC_master_mask,irqno);
	VM_out(0x21,VM_PIC_master_mask);
    } else if (irqno < 16) {
	/* Interrupt on slave PIC */
	bit_on(VM_PIC_slave_mask,irqno-8);
	VM_out(0xA1,VM_PIC_slave_mask);
	/* If the slave PIC is completely off   */
	/* Then turn off cascading line (Irq #2)*/
	if (VM_PIC_slave_mask == 0xFF && !(VM_PIC_master_mask & 0x04)) {
	    bit_on(VM_PIC_master_mask,2);
	    VM_out(0x21,VM_PIC_master_mask);
	}
    }
}

void VM_irq_unmask(WORD irqno)
{
    /* It is a nonsense to unmask the timer interrupt */
    if (irqno == 0) return;
    /* Interrupt is on master PIC */
    if (irqno < 8) {
	bit_off(VM_PIC_master_mask,irqno);
	VM_out(0x21,VM_PIC_master_mask);
    } else if (irqno < 16) {
	/* Interrupt on slave PIC */
	bit_off(VM_PIC_slave_mask,irqno-8);
	VM_out(0xA1,VM_PIC_slave_mask);
	/* If the cascading irq line was off */
	/* Then activate it also!            */
	if (VM_PIC_master_mask & 0x04) {
	    bit_off(VM_PIC_master_mask,2);
	    VM_out(0x21,VM_PIC_master_mask);
	}
    }
}

int VM_printf(char *fmt,...)
{
    char buf[100];
    va_list parms;
    int result;
    
    va_start(parms,fmt);
    result = vsprintf(buf,fmt,parms);
    cputs(buf);
    va_end(parms);
    return(result);
}
