/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	2.0						*/

/* File: CTest.C			*/
/* Console Library test program		*/

#include <xsys.h>
#include <cons.h>
#include <mem.h>
#include <string.h>

DWORD w;
#define WAIT()	for (w = 0; w < 0x5FFFFFF; w++)

/*
    Using 16 bit system, we only need to build a compatible
    console library for HARTIK! So we just need to test that module...
    We also test the compatibility stuff for the DOS_f...() functions,
    which may be called when the system is active, is DOS is present!!
*/

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
/*			CONVDEMO					   */
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */

void ConvDemo(void)
{
    int x  = 0;
    int x1 = -12345;
    long int x2 = -123456;
    long int x3 = -1234567;
    double y1 = 1.27691234567,little = 1.23456e-100,big = 1.23456e+100;    
    float  fy1 = 2769.1234567,flittle = 1.23456e-10,fbig = 1.23456e+10;
    char try[90];
    char s1[20],s2[20];

    cputs(".|\t|..|\t|...|\t|....|\t|.....|\t|......|\t|.......|\t|........|\t|\n");
    
    cprintf("X(0): %d/%x/%u\n",x,x,x);
    cprintf("X1(-12345): %d/%x/%u\n",x1,x1,x1);
    cprintf("X2(-123456): %ld/%lx/%lu\n",x2,x2,x2);
    cprintf("X3(-1234567): %ld/%lx/%lu\n",x3,x3,x3);
    cprintf("[Double]\nY1(1.27691234567): %lg/%le/%lf\n",y1,y1,y1);
    cprintf("Little(1.234e-100): %lg/%le/%lf\n",little,little,little);
    cprintf("Big(1.234e100): %lg/%le/%lf\n",big,big,big);
    cprintf("[Float]\nY1(2769.1234567): %g/%e/%f\n",fy1,fy1,fy1);
    cprintf("Little(1.234e-10): %g/%e/%f\n",flittle,flittle,flittle);
    cprintf("Big(1.234e+10): %g/%e/%f\n",fbig,fbig,fbig);

    strcpy(try,"-1234 0xCAFF 1234 Pippero Pippo-Only5 0.123456 1e+100 1e-100");
    cprintf("Should be %s\n",try);
    x = x1 = x2 = 0;
    y1 = little = big = 0;
    sscanf(try,"%i %x %lu %s %5s %lf %lg %le",&x,&x1,&x2,s1,s2,&y1,&little,&big);
    cprintf("Gotta: %i 0x%x %lu %s %s %lf %lg %le\n",x,x1,x2,s1,s2,y1,little,big);
}

/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
/*			BIOSDEMO					   */
/* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */

void BIOSDemo(void)
{
    X_REGS16 ir,or;
    X_SREGS16 sr;
    register i;
    /* Set video mode */
    ir.h.ah = 0;
    ir.h.al = 0x12;
    X_callBIOS(0x10,&ir,&or,&sr);
    /* Put some pixels */
    
    for (i = 0; i < 200; i++) {
	ir.h.ah = 0x0C;
	ir.h.al = i % 16;
	ir.x.bx = 0;
	ir.x.dx = i+40;
	ir.x.cx = i+100;
	X_callBIOS(0x10,&ir,&or,&sr);
    }
    WAIT();
    /* Set video mode */
    ir.h.ah = 0;
    ir.h.al = 0x03;    
    X_callBIOS(0x10,&ir,&or,&sr);
}



#define MAX_DEMOS (sizeof(demo_name)/sizeof(char *))

static char *demo_name[] = {
    "1BIOS ",
    "2BIOS ",
    "1Disk ",
    "2Disk ",
    "Conv  ",
    "Irq   ",
    "String"
};

static char *demo_desc[] = {
    "Graphics switch/BIOS Demo\n",
    "Graphics+Real Mode Buffer/BIOS Demo\n",
    "Disk/BIOS read-only Demo\n",
    "Disk/BIOS r/w Demo\n",
    "Conversion(sprintf) Demo\n",
    "Irq setup Demo\n",
    "String Management Demo\n"
};

int main(int argc,char *argv[])
{
    int demo_id;
    cprintf("Hello world from the 16 bit HARTIK library\n");
    if (argc == 1) {
	cprintf("Demo available: %u\n",MAX_DEMOS);
	for (demo_id = 0; demo_id < MAX_DEMOS; demo_id++) {
	    cprintf("%s - %s",demo_name[demo_id],demo_desc[demo_id]);
	}
    } else {
	for (demo_id = 0; demo_id < MAX_DEMOS; demo_id++) {
	    if (strncmp(argv[1],demo_name[demo_id],3) == 0) break;
	}
    }
    switch (demo_id) {
	case 0 : BIOSDemo();          break;
//	case 1 : BIOSBufDemo();       break;
//	case 2 : DiskDemo(argc,argv); break;
//	case 3 : DiskDemo2();         break;
	case 4 : ConvDemo();          break;
//	case 5 : IrqDemo();           break;
//	case 6 : StrDemo();           break;
	default : cprintf("No demo selected\n"); demo_id = -1; break;
    }
    if (demo_id != -1) cprintf("Run demo %s\n",demo_name[demo_id]);
}
