# Standard library for X/COFF applications
# Makefile for GNU MAKE & GCC 2.8.0

# Standard library for X/COFF applications
# Makefile for GNU MAKE & GCC 2.8.0 

CC 	= gcc
ifdef REDIR
LD      = ld
LINK_STARTUP = x0.o k_standa.o
LINK_STARTUP1= x0.o
LINK_STARTUP2= k_standa.o
LINK_OPTS = -oformat coff-go32 -s -nostartfiles
else
LD      = gcc
LINK_STARTUP = x0.o
LINK_STARTUP1= x0.o
LINK_STARTUP2= x0.o
LINK_OPTS = -s -nostartfiles
endif
AS	= gcc
HARTIK  = ../..
INCL 	= -I$(HARTIK)/h/x86 -I$(HARTIK)/h -I.
LIBS	= $(HARTIK)/lib

C_OPT =  $(INCL) -O -finline-functions -fno-builtin -D__GNU__
A_OPT =  $(INCL) -D__GNU__

COMMON_OBJS = stdlib.o \
	random.o \
	string.o \
	sprintf.o \
	sscanf.o \
	xinfo.o \
	x1.o \
	xsystab.o \
	cons.o \
	strbase.o \
	xbios.o \
	xconv.o \
	xdosf.o \
	xdosm.o
		
GNU_S_OBJS = mem.o \
	modf.o \
	xsys0.o \
	x0.o 

ifdef REDIR
UTILS = redir2.exe gcd.exe
else
UTILS = gcd.exe
endif

GNU_C_OBJS = gnucomp.o
OBJS = $(GNU_S_OBJS) $(GNU_C_OBJS) $(COMMON_OBJS)

.PHONY : clean allclean depend info

info :
	@type ..\..\info.txt

all : $(UTILS) libhlib.a $(LINK_STARTUP)

libs : $(UTILS) libhlib.a

install : $(UTILS) libhlib.a $(LINK_STARTUP)
	copy libhlib.a ..\..\lib
	copy x0.o ..\..\lib
	copy k_standa.o ..\..\lib
	copy *.exe ..
	copy *.exe ..\..\bin

clean :
	-del *.o
	-del *.err
	-del xtest
	-del $(UTILS)
	-del libhlib.a
	-del makefile

allclean : clean 
	echo # XTN Library dependencies > deps
	cd ..\..\lib 
	del libhlib.a
	-del $(LINK_STARTUP1)
	-del $(LINK_STARTUP2)
	cd ..\xtn\lib

depend deps:
	$(CC) $(C_OPT) -M $(COMMON_OBJS:.o=.c) $(patsubst %.o,gnu/%.c,$(GNU_C_OBJS)) > deps

#
# BinUtils 
#

gcd.exe : gnu/gcd.c
	gcc -s -Wall -o gcd.o -O2 gnu/gcd.c
	coff2exe gcd.o
	del gcd.o 

redir2.exe : gnu/redir2.c
	gcc -s -Wall -o redir2.o -O2 gnu/redir2.c
	coff2exe redir2.o
	del redir2.o 

#
# Demo
#	
	
xtest : $(UTILS) xtest.o libhlib.a $(LINK_STARTUP) tick.o
	$(LD) $(LINK_OPTS) $(LINK_STARTUP) xtest.o tick.o -L$(LIBS) libhlib.a -lhm -o xtest

tick.o : gnu/tick.S
	$(AS) $(A_OPT) -c $< -o $@

#
# The library!!
#

%.o : %.c
	$(REDIR) $(CC) $(C_OPT) -c $< $(ERRORS)

include deps
	
k_standa.o : gnu/k_standa.c
	$(REDIR) $(CC) $(C_OPT) -D_USE_LIBM_MATH_H -c gnu/k_standa.c -o k_standa.o

gnucomp.o : gnu/gnucomp.c
	$(REDIR) $(CC) $(C_OPT) -D_USE_LIBM_MATH_H -c gnu/gnucomp.c -o gnucomp.o

mem.o : gnu/mem.S 
	$(REDIR) $(AS) $(A_OPT) -c gnu/mem.S -o mem.o
	
xsys0.o : gnu/xsys0.S 
	$(REDIR) $(AS) $(A_OPT) -c gnu/xsys0.S -o xsys0.o
	
x0.o : gnu/x0.S 
	$(REDIR) $(AS) $(A_OPT) -c gnu/x0.S -o x0.o 
	
modf.o : gnu/modf.s 
	$(REDIR) $(CC) $(A_OPT) -c gnu/modf.s -o modf.o
	
libhlib.a : $(OBJS)
	ar rs libhlib.a $(OBJS)
