/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 1.0/COFF					*/

/* CVS $Id: sprintf.c,v 1.5 1999/11/04 10:15:19 massy0 Exp $ */

/* File: VSprintf.C */
/* Standard library for X applications */

/* String conversion functions */

#include <stdlib.h>
#include <string.h>
#include <mem.h>
#include <math.h>

unsigned ucvt(unsigned long v,char *buffer,int base,int width,int flag)
{
    register i = 0,j;
    unsigned ret = 0,abs_base;
    unsigned long abs_v;
    char tmp[12];
    /* Extract the less significant digit */
    /* Put it into temporary buffer       */
    /* It has to be local to have 	  */
    /* reentrant functions		  */
    /*
    MG: fix to handle zero correctly
    if (v == 0) {
	*buffer++ = '0';
	*buffer = 0;
	return(1);
    }
    */
    
    /* MG: is this required? */
    /* the vsprintf() function seems to call this procedure with */
    /* this flag inverted */
    flag ^= LEFT_PAD;
    
    abs_base = (base >= 0) ? base : -base;
    if (base < 0) abs_v = ((long)(v) < 0) ? -v : v;
    else abs_v = v;
    /* Process left space-padding flags */
    if (flag & ADD_PLUS || ((base < 0) && ((long)(v) < 0))) {
	ret++;
    }
    /* MG: fix to handle zero correctly */
    if (v == 0)
        tmp[i++]='0';
    else
        while (abs_v > 0) {
	    tmp[i++] = _todigit(abs_v % abs_base);
	    abs_v = abs_v / abs_base;
        }
    ret += i;
    if ((flag & LEFT_PAD) && (flag & SPACE_PAD)) {
	j = ret;
	while (j < width) {
	    *buffer++ = ' ';
	    ret++;
	    j++;
	}
    }
    /* Add plus if wanted */
    if (base < 0) {
	if (((long)(v)) < 0) *buffer++ = '-';
        else if (flag & ADD_PLUS) *buffer++ = '+';
    } else if (flag & ADD_PLUS) *buffer++ = '+';
    /* Process left zero-padding flags */
    if ((flag & LEFT_PAD) && (flag & ZERO_PAD)) {
	j = ret;
	while (j < width) {
	    *buffer++ = '0';
	    ret++;
	    j++;
	}
    }
    /* Reverse temporary buffer into output buffer */
    /* If wanted left pad with zero/space; anyway only one at once is ok */
    for (j = i-1; j >= 0; j--) *buffer++ = tmp[j];
    if ((flag & (SPACE_PAD)) && !(flag & LEFT_PAD)) {
	/* If wanted, pad with space to specified width */
	j = ret;
	while (j < width) {
	    *buffer++ = ' ';
	    ret++;
	    j++;
	}
    }
    /* Null terminate the output buffer */
    *buffer = 0;
    return(ret);
}

unsigned dcvt(long v,char *buffer,int base,int width,int flag)
{
    return(ucvt((unsigned long)(v),buffer,-base,width,flag));
}

unsigned fcvt(double v,char *buffer,int width,int prec,int flag)
{
    double ip,fp,val;
    register int l=0;
    register int digits;
    int conv,got_a_digit,exceeded,len = 0;
    char tmp[300];
    memset(tmp,0,300);
    /* Manage Inf & NaN */
    if (_isspecial(v,buffer)) return(strlen(buffer));
    /* Check for negative value & add optional + sign */
    if (v < 0) {
	*buffer++ = '-';
	v = -v;
	len++;
    } else if (flag & ADD_PLUS) {
	*buffer++ = '+';
	len++;
    }
    /* Extract integer part & mantissa */
    fp = modf(v,&ip);
    /* Process integer part */
    digits = 0;
    if (ip >= 1.0) {
	while (ip >= 1.0) {
	    /* The 0.01 here is used to correct rounding errors*/
	    /* Ex: 1.2 --> 1.1999999999999... adjust to 1.2 */
	    /*
	    ip = ip / 10.0;
	    modf(ip,&val);
	    val = (ip - val)*10.0;
	    */
	    val = fmod(ip,10.0);
	    ip = ip / 10.0;
	    tmp[digits++] = _todigit((unsigned)(val));
	}
	len += digits;
	/* Now reverse the temporary buffer into output buffer 	*/
	/* Translate only the last 15 digits 			*/
	/* The others are beyond double precision limit!  	*/	
	for (l = digits-1; l >= max(digits-15,0); l--) *buffer++ = tmp[l];
	if (l >= 0) for (l = digits-16; l >= 0; l--) *buffer++ = '0';
	*buffer = 0;
    /* If IP == 0 -> just put in a 0 */
    } else {
	*buffer++ = '0';
	len++;
    }
    /* Process fractionary part according to width specification */
    /* If RESPECT_WIDTH is set, scan until you reach wanted precision */
    /* Else scan until you find a not 0 digit */
    if (fp > 1e-307 && len < width+1) {
	*buffer++ = '.';
	len++;
	if (flag & RESPECT_WIDTH) got_a_digit = 1;
	else got_a_digit = 0;
	exceeded = 0;
	digits = 1;
	while (!exceeded) {
	    /* The DBL_EPSILON here is used to correct rounding errors */
	    fp = (fp + DBL_EPSILON) * 10.0;
	    fp = modf(fp,&ip);
	    conv = (int)(ip);
	    if (conv != 0 && !got_a_digit) got_a_digit = 1;
	    *buffer++ = _todigit(conv);
	    len++;
	    digits++;
	    if (got_a_digit && (digits > prec)) exceeded = 1;
	    if (width < len) exceeded = 1;
	}
	/* Drop trailing zeros after decimal point */
        while (*--buffer == '0' && *(buffer-1) != '.') len--;
	buffer++;
    }
    *buffer = 0;
    return(len);
}

unsigned ecvt(double v,char *buffer,int width,int prec,int flag)
{
    register int len=0;
    long int exponent,abs_exponent;
    /* Manage Inf & NaN */
    if (_isspecial(v,buffer)) return(strlen(buffer));
    /* Check for negative value & add optional + sign */
    if (v < 0) {
	*buffer++ = '-';
	v = -v;
	len++;
    } else if (flag & ADD_PLUS) {
	*buffer++ = '+';
	len++;
    }	
    /* Zero check! */
    if (v < DBL_MIN) {
	*buffer++ = '0';
	*buffer = 0;
	return(1);
    }
    /* Evaluate the exponent */
    if (v < 1) {
	exponent = 0;
	while (v < 1.0) {
	    v *= 10.0;
	    exponent--;
	}
    }
    else {
	exponent = 0;
	while (v >= 10.0) {
	    v /= 10.0;
	    exponent++;
	}
    }
    abs_exponent = abs(exponent);
    if (abs_exponent > 99) width -= 3;
    else if (abs_exponent > 9) width -= 2;
    else if (exponent > 0) width -= 1;
    /* Now the number as IP in range [0,1] */
    /* Convert this as a fixed point format */
    len += fcvt(v,buffer,width,prec,flag | RESPECT_WIDTH);
    /* Now convert the exponent */
    if (exponent == 0) return(len);
    buffer += len;
    *buffer++ = 'e';
    len += dcvt(exponent,buffer,10,exponent/10+2,ADD_PLUS)+1;
    return(len);
}

unsigned gcvt(double v,char *buffer,int width,int prec,int flag)
{
    double v_abs;
    int exponent;
    /* Manage Inf & NaN */
    if (_isspecial(v,buffer)) return(strlen(buffer));
    /* Extract absolute value */
    if (v < 0.0) v_abs = -v;
    else v_abs = v;
    /* Zero check! */
    if (v_abs < DBL_MIN) {
	*buffer++ = '0';
	*buffer = 0;
	return(1);
    }
    exponent = 0;
    /* Evaluate exponent */
    if (v_abs < 1.0) {
	while (v_abs < 1) {
	    v_abs *= 10.0;
	    exponent--;
	}
    }
    else {
	while (v_abs >= 10.0) {
	    v_abs /= 10.0;
	    exponent++;
	}
    }
    /* Choose shortest format on exponent value */
    if (exponent > 7 || exponent < -7) 
       return(ecvt(v,buffer,width,prec,flag));
    else return(fcvt(v,buffer,width,prec,flag));
}

#define STD_SIZE	0
#define SHORT_SIZE	1
#define LONG_SIZE	2

int vsprintf(char *buf,char *fmt,va_list parms)
{
    int scanned = 0,w = 0,prec = 0,l = 0,size = 0;
    int n1 = 0;
    unsigned n2 = 0,parsing = 0,flag = 0;
    double n3 = 0.0;
    char *base = buf;
    char *sp = NULL;
    void *fp = NULL;
    while (*fmt != 0) {
	if (*fmt != '%' && !parsing) {
	    /* No token detected */
	    *buf++ = *fmt++;
	    scanned++;
	}
	else {	    
	    /* We need to make a conversion */
	    if (*fmt == '%') {
		fmt++;
		parsing = 1;
		w = 10;
		prec = 4;
		size = STD_SIZE;
		flag = 0;
	    }
	    /* Parse token */
	    switch(*fmt) {
		case '%' : *buf++ = '%';
			   scanned++;
			   parsing = 0;
			   break;
		case 'c' : *buf++ = va_arg(parms, char);
			   scanned++;
			   parsing = 0;
			   break;
		case 'i' :
	 	case 'd' : switch (size) {
		    		case STD_SIZE : n1 = va_arg(parms, int);
						break;
				case LONG_SIZE : n1 = va_arg(parms, long int);
						 break;
				case SHORT_SIZE : n1 = va_arg(parms, short int);
						  break;
			   }
			   l = dcvt(n1,buf,10,w,flag);
			   scanned += l;
			   buf += l;
			   parsing = 0;
			   break;
		case 'u' : switch (size) {
		    		case STD_SIZE : n2 = va_arg(parms, unsigned);
						break;
				case LONG_SIZE : n2 = va_arg(parms, unsigned long);
						 break;
				case SHORT_SIZE : n2 = va_arg(parms, unsigned short);
						  break;
			   }
			   l = ucvt(n2,buf,10,w,flag);
			   scanned += l;
			   buf += l;
			   parsing = 0;
			   break;
		case 'x' : switch (size) {
		    		case STD_SIZE : n2 = va_arg(parms, unsigned);
						break;
				case LONG_SIZE : n2 = va_arg(parms, unsigned long);
						 break;
				case SHORT_SIZE : n2 = va_arg(parms, unsigned short);
						  break;
			   }
			   l = ucvt(n2,buf,16,w,flag);
			   scanned += l;
			   buf += l;
			   parsing = 0;
			   break;
		case 'o' : switch (size) {
		    		case STD_SIZE : n2 = va_arg(parms, unsigned);
						break;
				case LONG_SIZE : n2 = va_arg(parms, unsigned long);
						 break;
				case SHORT_SIZE : n2 = va_arg(parms, unsigned short);
						  break;
			   }
			   l = ucvt(n2,buf,8,w,flag);
			   scanned += l;
			   buf += l;
			   parsing = 0;
			   break;
		case 'p' : fp = va_arg(parms, void *);
			   n2 = (unsigned long)(fp);
			   l = ucvt(n2,buf,16,w,0);
			   scanned += l;
			   buf += l;
			   parsing = 0;
			   break;
		case 's' : sp = va_arg(parms, char *);
		           l=0;
			   while (*sp != 0) {
			       *buf++ = *sp++;
			       l++;
			   }
			   scanned += l;
			   parsing = 0;
			   break;
		case 'f' : switch (size) {
		    		case STD_SIZE : n3 = va_arg(parms, double);
					 	break;
		    		case LONG_SIZE : n3 = va_arg(parms, double);
					 	 break;
				/*
				It seems that the compilers push a float as
				a double ... Need to check!
				*/
		    		case SHORT_SIZE : n3 = va_arg(parms, double);
					 	  break;
			   }
			   l = fcvt(n3,buf,w,prec,flag);
			   scanned += l;
			   buf += l;
			   parsing = 0;
			   break;
		case 'e' : switch (size) {
		    		case STD_SIZE : n3 = va_arg(parms, double);
					 	break;
		    		case LONG_SIZE : n3 = va_arg(parms, double);
					 	 break;
				/*
				It seems that the compilers push a float as
				a double ... Need to check!
				*/
		    		case SHORT_SIZE : n3 = va_arg(parms, double);
					 	  break;
			   }
			   l = ecvt(n3,buf,w,prec,flag);
			   scanned += l;
			   buf += l;
			   parsing = 0;
			   break;			   
		case 'g' : switch (size) {
		    		case STD_SIZE : n3 = va_arg(parms, double);
					 	break;
		    		case LONG_SIZE : n3 = va_arg(parms, double);
					 	 break;
				/*
				It seems that the compilers push a float as
				a double ... Need to check!
				*/
		    		case SHORT_SIZE : n3 = va_arg(parms, double);
					 	  break;
			   }
			   l = gcvt(n3,buf,w,prec,flag);
			   scanned += l;
			   buf += l;
			   parsing = 0;
			   break;
		case 'l' : size = LONG_SIZE;
			   break;
		case 'n' : size = SHORT_SIZE;
			   break;
		case '+' : flag |= ADD_PLUS;
			   break;
		case '-' : flag |= LEFT_PAD;
			   break;
		case '.' : parsing = 2;
			   flag |= RESPECT_WIDTH;
			   break;
		case '1' :
		case '2' :
		case '3' :
		case '4' :
		case '5' :
		case '6' :
		case '7' :
		case '8' :
		case '9' :
		case '0' : if (parsing == 1) {
		    	      w = strtou(fmt,10,&base);
			      /* MG */
			      /* if the first numeral is zero a ZERO pad is */
			      /* required */
			      /* but not if LEFT_PAD is set*/
			      if (*fmt!='0'||flag&LEFT_PAD)
				  flag |= SPACE_PAD ;
			      else
				  flag |= ZERO_PAD ;
			      fmt = base-1;
			   } else if (parsing == 2) {
			       prec = strtou(fmt,10,&base);
			       fmt = base-1;
			       parsing = 1;
			   }
			   break;
		default :  parsing = 0;
			   break;
	    }
	    fmt++;
	}
    }
    *buf = 0;
    return(scanned);
}

int sprintf(char *buf,char *fmt,...)
{
    va_list parms;
    int result;
    va_start(parms,fmt);
    result = vsprintf(buf,fmt,parms);
    va_end(parms);
    return(result);
}
