/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 1.0					*/

/* File: X1.C		*/
/* Startup code:    	*/
/* Build parameters list & make info accessible */

#include <xsys.h>
#include <xdos.h>
#include <mem.h>
#include <cons.h>

/* #define __DUMP_MEM__  */

/* We need to copy from X address space to the application space  */
/* the info structure to allow pointer access using flat model    */
/* Remember that flat model is similar to small model also if we  */
/* can see the whole memory, because it has no explicit far 	  */
/* pointers; then if we pass into _args[] the address of the      */
/* string of the n-th argument it could not be correctly accessed */
/* because it lies in a memory zone unseen from PM application.   */
/* This is due to the ELF format which has no relocation info     */
/* since the file is already relocated starting from address 0!   */
/* Then the PM application cannot see a real flat memory (segment */
/* with 0 base) but CS,DS & SS MUST have the base correctly set.  */
/* Refer to this figure:					  */
/*								  */
/* DOS Memory <- X is there					  */
/*        	      						  */
/* EXTENDED Memory -----[					  */
/* 			[					  */
/*	Address	 xxxx	[ <- Application code is here!		  */
/*			[					  */
/*	Address  yyyy	[ <- Application Data & Stack!		  */
/*								  */
/* Then	CS has xxxx base while DS & SS have yyyy base!		  */

/* As GNU C do not support far pointers explicitely, we had to 	  */
/* use a local cache for the _X_info structure			  */
struct info *_X_info = NULL;
struct info local_X_info __attribute__ ((packed));

/* Stack base address; use this to check stack overflow! 	*/
/* With Flat model I do not think we can use 386 protection 	*/
/* to detect a stack overflow; anyway Watcom C use a standard 	*/
/* function __CHK to detect it!	The compiler place it whenever  */
/* it calls a function to detect overflow			*/

DWORD _stkbase; 
DWORD _stktop;

/* This is some extra stuff we need to compile with argument	*/
/* passing and math extensions					*/
DWORD _argc;
char * _argv[MAX_ARGS];

extern void main(int argc,char *argv[]);
extern void bios_save(void);
extern void bios_restore(void);
extern void DOS_mem_init(void);

/* This is used in GNU-C to implement C++ constructors/destructors */
/* See the lib sources for more details				   */

void __main()
{
}

void __cdecl _startup(void)
{
    register i;
    void cache_X_info(void);

    /* Recopy info data from X address space 			 */
    /* We cannot use the lmemcpy function with the address space */
    /* converting function, because these need the X_info data	 */
    _X_info = &(local_X_info);
    cache_X_info();

    /* Init the DOS memory allocator */
    DOS_mem_init();
    
    _argc = _X_info->argc;
    
    /* Build parameter list */ 
    for (i = 0; i < _argc; i++) _argv[i] = _X_info->argv[i];
    
    bios_save();
    /* Call main procedure using standard C convention 	 */
    /* Remember you cannot call any console I/O function */
    /* if you do not call bios_save()			 */
    
    #ifdef __DUMP_MEM__
	cprintf("X/MEM   : %u\n",_X_info->extended_size);
	cprintf("DOS/MEM : %u\n",_X_info->dos_size);
	cprintf("X_Info Size : %u\n",sizeof(local_X_info));
	cprintf("Argc : %u/%u\n",_X_info->argc,local_X_info.argc);
	cprintf("Argv[0] : %s / %s\n",_X_info->argv[0],_argv[0]);
	cprintf("Argv[1] : %s\n",_X_info->argv[1]);
	cprintf("Argv[2] : %s\n",_X_info->argv[2]);
	cprintf("Argv[3] : %s\n",_X_info->argv[3]);
    #endif
    main(_argc,_argv);
    bios_restore();
}
