/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	2.0						*/

/* File: XBios.C		   			*/
/* System calls for X extender 			  	*/
/* BIOS Call from X (Basic Reflection Service)		*/

/* modified by Massy :
 * -- added __SAFEXBIOS__ (see config.h)
 */

#include <config.h>
#include <xsys.h>
#include <cons.h>
#include <mem.h>
#include <math.h>

/* The interface between X (and 32 bit PM) and BIOS (which runs at 16 bits */
/* It works as int86() standard library call				   */

#ifdef __SAFEXBIOS__
void X_callBIOSunsafe(int service,X_REGS16 *in,X_REGS16 *out,X_SREGS16 *s)
#else
void X_callBIOS(int service,X_REGS16 *in,X_REGS16 *out,X_SREGS16 *s)
#endif
{
    extern cache_X_info(void);
    extern update_X_info(void);
    /* Assembler gate JMP instruction */
    #ifndef __LINUX__
    extern void _x_callBIOS(void);
    #else
    extern void __x_callBIOS(void);
    extern void _x_callBIOS(void) __attribute__((weak,alias("__x_callBIOS")));
    #endif

    /* Send interrupt request & register through the X_Info structure */
    _X_info->irqno = service;
    memcpy(&(_X_info->ir),in,sizeof(X_REGS16));
    memcpy(&(_X_info->sr),s,sizeof(X_SREGS16));
    update_X_info();
    
    /* Back to RM to execute the BIOS routine */
    _x_callBIOS();

    /* Get the return register values */
    cache_X_info();
    memcpy(out,&(_X_info->or),sizeof(X_REGS16));
    memcpy(s,&(_X_info->sr),sizeof(X_SREGS16));
}

/*
 * Safe X_callBIOS (I hope)
 *
 * Added by Massy
 */

#ifdef __SAFEXBIOS__

#define MAXREG 16

static struct  {
   int when;
   void (*fun)(void);
} reg[MAXREG];

/* init table */
void xbios_initfun(void)
{
   int i;
   for (i=0;i<MAXREG;i++) {
      reg[i].when=0;
      reg[i].fun=NULL;
   }
}

/* to register a function */
int xbios_regfun(void (*fun)(void),int when)
{
   int i;
   for (i=0;i<MAXREG;i++)
      if (reg[i].fun==NULL)  {
	 reg[i].when=when;
	 reg[i].fun=fun;
	 return 0;
      }
   
   return -1;
}

/* the safe X_callBIOS */
void X_callBIOS(int service,X_REGS16 *in,X_REGS16 *out,X_SREGS16 *s)
{
   SYS_FLAGS f;
   BYTE p1,p2;
   int i;

   for (i=0;i<MAXREG;i++) {
      if (reg[i].fun!=NULL)  { 
      	if (reg[i].when==-1) (reg[i].fun)();
     } else break;
   }
   
   /* mask IRQ on controller */
   p1 = VM_in(0x21);
   p2 = VM_in(0xA1);
   VM_out(0x21,0xFF);
   VM_out(0xA1,0xFF);
   
   //f=VM_save();
   
   X_callBIOSunsafe(service,in,out,s);
   
   //VM_restore(f);
   
   /* restore */
   VM_out(0x21,p1);
   VM_out(0xA1,p2);
   
   for (i=0;i<MAXREG;i++) {
      if (reg[i].fun!=NULL)  { 
      	if (reg[i].when==1) (reg[i].fun)();
      } else break;
   }
}

#endif
