/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	2.0						*/

/* CVS $Id: xdosf.c,v 1.4 1999/07/16 09:11:06 massy Exp $ */

/* File: XDosF.C			*/
/* DOS Interface toward disk		*/

#include <mem.h>
#include <string.h>
#include <xsys.h>
#include <xdos.h>

/* define that if you want debug during 16bits reflections*/
#if 0

#include <cons.h>

int debug(char *s)
{
  cprintf("%s\n",s);
}

#else

#define debug(x)

#endif

/* Basic DOS I/O Flag */

#define DOS_READ	0
#define DOS_WRITE	1
#define DOS_RW		2

#define DOS_BUFSIZE	1024	/* I/O auxiliary buffer 	*/
#define DOS_MAXDESCR	16	/* Maximum number of DOS file 	*/

static DOS_FILE DOS_descr[DOS_MAXDESCR];
/* I Hope this array is initialized with 0... */
static BYTE busy[DOS_MAXDESCR];

/* The Last DOS Error occurred! */
static unsigned _DOS_error = 0;

unsigned DOS_error(void)
{
    unsigned v = _DOS_error;
    _DOS_error = 0;
    return(v);
}

DOS_FILE *DOS_fopen(char *name,char *mode)
{
    X_REGS16 ir,or;
    X_SREGS16 sr;
    DOS_FILE *f;
    register i;

    #ifdef __DEBUG__
        char xname[80];
    #endif

    for (i = 0; busy[i] && i < DOS_MAXDESCR; i++);
    /* Return NULL if no descriptor is available... */
    if (i == DOS_MAXDESCR) return(NULL);
    /* Otherwise, lock the descriptor & remember the index */
    f = &DOS_descr[i]; busy[i] = 1; f->index = i;
    /* Allocate a DOS buffer for the file name */
    f->n1 = DOS_alloc(80);
    strcpy(f->n2,name);
    if (mode[0] == 'r') {
	/* DOS Call: Intr 0x21
		AH = 0x3D  - Open existing file
		AL = 0,1,2 - Read,Write, R/W
	*/
	f->mode = DOS_READ;
	ir.h.ah = 0x3D;
	ir.h.al = f->mode;
    }
    else if (mode[0] == 'w') {
	/* DOS Call: Intr 0x21
		AH = 0x3C  - Create a file
		AL = File attribute [0x20 = Standard,r/w,archive]
	*/
	f->mode = DOS_WRITE;
	ir.h.ah = 0x3C;
	ir.x.cx = 0x20;
    }
    f->offset = 0;
    /* Copy th \0 also! */
    lmemcpy(f->n1,appl2linear(name),strlen(name)+1);
    #ifdef __DEBUG__
	lmemcpy(appl2linear(xname),f->n1,strlen(name)+1);
	cprintf("Name is : %s -- Mode : %d\n",xname,f->mode);
    #endif

    /* Ask DOS to open File */
    debug("calling Int0x21 func 0x3c/d");
    ir.x.dx = DOS_OFF(f->n1);
    sr.ds = DOS_SEG(f->n1);
    X_callBIOS(0x21,&ir,&or,&sr);
    f->handle = (!(or.x.cflag) ? or.x.ax : -1);
    debug("called Int0x21 func 0x3c/d");

    if (f->handle == -1) {
	/* DOS request failed! Release the used resources */
	DOS_free(f->n1,80);
	busy[i] = 0;
	_DOS_error = or.x.ax;
	return(NULL);
    }
        
    /* Allocate the DOS buffer for temporary I/O */
    f->buf = DOS_alloc(DOS_BUFSIZE);
    return(f);
}

void DOS_fclose(DOS_FILE *f)
{
    X_REGS16 ir,or;
    X_SREGS16 sr;

    if (f == NULL || busy[f->index] == 0) return;
    /* DOS Call: Intr 0x21
	    AH = 0x3E  - Close a file
	    BX = File handle
    */
    ir.h.ah = 0x3E;
    ir.x.bx = f->handle;
    X_callBIOS(0x21,&ir,&or,&sr);
    DOS_free(f->buf,DOS_BUFSIZE);
    DOS_free(f->n1,80);
    busy[f->index] = 0;
}

DWORD DOS_fread(void *abuf,DWORD size,DWORD num,DOS_FILE *f)
{
    X_REGS16 ir,or;
    X_SREGS16 sr;
    DWORD count = size*num,now = 0,chunk;
    BYTE done = 0;
    BYTE *buf = (BYTE *)(abuf);
    
    while (done == 0) {
	/* Fragment the read operation ... */
	if (count > DOS_BUFSIZE) chunk = DOS_BUFSIZE;
	else chunk = count;
	/* DOS Call: Intr 0x21
		AH = 0x3F  - Read data using a file handle
		BX = File handle
		CX = Buffer size
		DS:DX = Segment:Offset of the Buffer
	*/
	ir.h.ah = 0x3F;
	ir.x.bx = f->handle;
	ir.x.cx = chunk;
	ir.x.dx = DOS_OFF(f->buf);
	sr.ds = DOS_SEG(f->buf);
	X_callBIOS(0x21,&ir,&or,&sr);
	/* If it was OK ... */
	if (!(or.x.cflag)) {
	    /* Copy data into application buffer */
	    lmemcpy(appl2linear(buf),f->buf,or.x.ax);
	    buf += or.x.ax;
	    now += or.x.ax;
	    f->offset += or.x.ax;
	    count -= or.x.ax;
	    /*
	        Finish if we have read all the data or
		if we have read less data than how expected
	    */	
	    if (now == size*num || or.x.ax != chunk) done = 1;	    
	} else {
	    done = -1;
	    _DOS_error = or.x.ax;
	}
    }
    return(now);
}

DWORD DOS_getline(void *abuf, DWORD count, int ch, DOS_FILE *f)
{
  X_REGS16 ir,or;
  X_SREGS16 sr;
  DWORD now = 0,chunk,cread;
  BYTE done = 0;
  BYTE *buf = (BYTE *)(abuf);
  DWORD startcounter=count;
   
  debug("DOS_getline() START");

  /* why this? because we want to add always a '\0' char */
  count--;
  
  while (done == 0) {
    /* Fragment the read operation ... */
    if (count > DOS_BUFSIZE) chunk = DOS_BUFSIZE;
    else chunk = count;
    /* DOS Call: Intr 0x21
       AH = 0x3F  - Read data using a file handle
       BX = File handle
       CX = Buffer size
       DS:DX = Segment:Offset of the Buffer
       */

    debug("calling Int0x21 func 0x3f");

    ir.h.ah = 0x3F;
    ir.x.bx = f->handle;
    ir.x.cx = chunk;
    ir.x.dx = DOS_OFF(f->buf);
    sr.ds = DOS_SEG(f->buf);
    X_callBIOS(0x21,&ir,&or,&sr);

    debug("called Int0x21 func 0x3f");
         
    /* If it was OK ... */
    if (!(or.x.cflag)) {

      int delta;      
      cread=or.x.ax;
      delta=lmemchr(f->buf,ch,or.x.ax);

      if (delta==0) {
	/* char 'ch' not found... copy all buffer! */
	if (buf!=NULL) {
	  lmemcpy(appl2linear(buf),f->buf,or.x.ax);
	  buf += or.x.ax;
	}
	now += or.x.ax;
	f->offset += or.x.ax;
	count -= or.x.ax;
      }
      else {
	/* char 'ch' found... copy delta chars */
	if (buf!=NULL) {
	  lmemcpy(appl2linear(buf),f->buf,delta);
	  buf += delta;
	}
	now += delta;
	f->offset += delta;
	count -= delta;

	/* DOS Call: Intr 0x21
	   AH = 0x42  - Move File Read Write Pointer
	   AL = Method (0=beginning,1=actual,2=end)
	   BX = File handle
	   CX:DX = Buffer size
	   */
	
	debug("calling Int0x21 func 42");
	
	ir.h.ah = 0x42;
	//ir.h.al = 1;
	//ir.x.cx = ((-delta) >> 16) & 0xffff;
	//ir.x.dx = (-delta) & 0xffff;
	ir.h.al=0;
	ir.x.cx=((f->offset)>>16)&0xffff;
	ir.x.dx=(f->offset)&0xffff; 
	ir.x.bx = f->handle;
	
	X_callBIOS(0x21,&ir,&or,&sr);

	debug("called Int0x21 func 42");

	if (or.x.cflag) {
	  _DOS_error = or.x.ax;
	  done=-1;     
	} else
	  done=1;
      }
      /*
	Finish if we have read all the data or
	if we have read less data than how expected
	*/	
      if ((now == startcounter || cread != chunk || count==0 )
	  && done==0) done = 1;
    } else {
      done = -1;
      _DOS_error = or.x.ax;
    }
  }
  
  if (done>0) *(buf+1)='\0';

  debug("DOS_getline() END");
  return(now);
}

int DOS_fgetc(char *ptr, DOS_FILE *f)
{
  X_REGS16 ir,or;
  X_SREGS16 sr;
    
  /* DOS Call: Intr 0x21
     AH = 0x3F  - Read data using a file handle
     BX = File handle
     CX = Buffer size
     DS:DX = Segment:Offset of the Buffer
     */
  ir.h.ah = 0x3F;
  ir.x.bx = f->handle;
  ir.x.cx = sizeof(char);
  ir.x.dx = DOS_OFF(f->buf);
  sr.ds = DOS_SEG(f->buf);
  X_callBIOS(0x21,&ir,&or,&sr);
  
  /* If it was not OK ... */
  if (or.x.cflag) {
    _DOS_error = or.x.ax;
    return -1;
  }

  lmemcpy(appl2linear(ptr),f->buf,sizeof(char));
  f->offset += or.x.ax;

  return 0;
}

static inline int isspace(int ch)
{
  return ch==' '||ch=='\t'||ch=='\n'||ch=='\r';
}

static inline int isint(int ch)
{
  return (ch>='0'&&ch<='9')||ch=='+'||ch=='-';
}

static inline int isfloat(int ch)
{
  return isint(ch)||ch=='e'||ch=='E'||ch=='.';
}

int DOS_fgeti(int *ptr, DOS_FILE *f)
{
  char str[256],*s;
  char ch;
  int res;
  
  /* Well, this can be improved more */
  
  ch=' ';
  while (isspace(ch)) {
    res=DOS_fgetc(&ch,f);
    if (res!=0) return -1;
  }

  s=str;
  while (isint(ch)) {
    *s++=ch;
    if (s-str>=sizeof(str)-1) return -2;
    res=DOS_fgetc(&ch,f);
    if (res!=0) return -1;
  }
  *s='\0';

  sscanf(str,"%i",ptr);
  return 0;
}

int DOS_fgetf(double *ptr, DOS_FILE *f)
{
  char str[256],*s;
  char ch;
  int res;
  
  /* Well, this can be improved more */
  
  ch=' ';
  while (isspace(ch)) {
    res=DOS_fgetc(&ch,f);
    if (res!=0) return -1;
  }

  s=str;
  while (isfloat(ch)) {
    *s++=ch;
    if (s-str>=sizeof(str)-1) return -2;
    res=DOS_fgetc(&ch,f);
    if (res!=0) return -1;
  }
  *s='\0';

  sscanf(str,"%f",ptr);
  return 0;
}

	      
DWORD DOS_fwrite(void *abuf,DWORD size,DWORD num,DOS_FILE *f)
{
    X_REGS16 ir,or;
    X_SREGS16 sr;
    DWORD count = size*num,now = 0,chunk;
    BYTE done = 0;
    BYTE *buf = (BYTE *)(abuf);
    
    while (done == 0) {
	/* Fragment the write operation ... */
	if (count > DOS_BUFSIZE) chunk = DOS_BUFSIZE;
	else chunk = count;
	/* Copy data from application buffer */
	lmemcpy(f->buf,appl2linear(buf), chunk);
	/* DOS Call: Intr 0x21
		AH = 0x40 - Write data using a file handle
		BX = File handle
		CX = Buffer size
		DS:DX = Segment:Offset of the Buffer
	*/
	ir.h.ah = 0x40;
	ir.x.bx = f->handle;
	ir.x.cx = chunk;
	ir.x.dx = DOS_OFF(f->buf);
	sr.ds = DOS_SEG(f->buf);
	X_callBIOS(0x21,&ir,&or,&sr);
	/* If it was OK ... */
	if (!(or.x.cflag)) {
	    f->offset += or.x.ax;
	    count -= or.x.ax;
	    buf += or.x.ax;
	    now += or.x.ax;
	    if (now == size*num || or.x.ax != chunk) done = 1;	    
	} else {
	    done = -1;
	    _DOS_error = or.x.ax;
	}
    }
    return(now);
}

