/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 2.0 					*/

/* File: XDosM.C				*/
/* This file contains the DOS memory allocator 	*/

#include <mem.h>
#include <xsys.h>
#include <xdos.h>
#include <cons.h>

/*
    We do not use the standard K&R pointer based memory allocator!
    This is because we cannot do pointer tricks easily with the GCC
    which only handles explicitely 32 bit non segmented address space.
    So we cannot use far pointers, and we cannot chain buckets of memory
    which are located in low memory area, beyond the accessibility
    of the ELF address space.
    A static allocation is so preferred; this should not give trouble
    since DOS memory is only used for reflection related purposes
*/

#define MAX_PARTITION   50 /* Max available partition */

static struct {
	BYTE 	 used;              
	LIN_ADDR addr;
	DWORD 	 size;                     
} mem_table[MAX_PARTITION];

void DOS_dump_mem(void)
{
    register i;
    for (i = 0; i < MAX_PARTITION; i++) {
	if (mem_table[i].used) cprintf("(%d) Addr : %ld   Size : %lu/%lx\n",i,mem_table[i].addr,mem_table[i].size,mem_table[i].size);
    }
}

void DOS_mem_init(void)
{
    register i;
    mem_table[0].used = TRUE;
    mem_table[0].addr = _X_info->dos_base;
    mem_table[0].size = _X_info->dos_size;
    for (i = 1; i < MAX_PARTITION; i++)	mem_table[i].used = FALSE;
}

LIN_ADDR DOS_alloc(DWORD s)
{
    LIN_ADDR p = 0;
    int i = 0;

    while (i < MAX_PARTITION && p == NULL) {
	if (mem_table[i].used && (mem_table[i].size >= s))
		p = mem_table[i].addr;
	else i++;
    }
    if (p != 0) {
	if (mem_table[i].size > s) {
	    mem_table[i].size -= s;
	    mem_table[i].addr += s;
	}
	else mem_table[i].used = FALSE;
    }
    return(p);
}

int DOS_free(LIN_ADDR p,DWORD s)
{
    register i = 1;
    unsigned i1 = 0, i2 = 0;

    while (i < MAX_PARTITION && ((i1 == 0) || (i2 == 0)) ) {
	if (mem_table[i].used) {
	    if (mem_table[i].addr + mem_table[i].size == p) i1 = i;
	    if (mem_table[i].addr == p + s) i2 = i;
	}
	i++;
    }
    if (i1 != 0 && i2 != 0) {
	mem_table[i1].size += mem_table[i2].size + s;
	mem_table[i2].used = FALSE;
    }
    else if (i1 == 0 && i2 != 0) {
	mem_table[i2].addr = p;
	mem_table[i2].size += s;
    }
    else if (i1 != 0 && i2 == 0) mem_table[i1].size += s;
    else {
	i = 0;
	while (i < MAX_PARTITION && (mem_table[i].used == TRUE)) i++;
	if (i == MAX_PARTITION) return(FALSE);
	mem_table[i].addr = p;
	mem_table[i].size = s;
	mem_table[i].used = TRUE;
    }
    return(TRUE);
} 
