/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	2.0						*/

/* File: XSysTab.C		   			  */
/* System calls for X extender 			  	  */
/* These function provide access to system tabels GDT/IDT */

#include <xsys.h>
#include <cons.h>
#include <mem.h>
#include <math.h>

/* We usually need to set-up an interrupt handler; to do this we have */
/* to fill a GATE structure & copy it into the Interrupt Descriptor   */
/* Table or IDT; its phisical address is given by IDT_base	      */

void IDT_place(BYTE num,void (__cdecl *handler)(void))
{
    DWORD IDT_addr = _X_info->IDT_base + num*sizeof(GATE);    
    DWORD offset = (DWORD)(handler);
    GATE g;
    g.sel = PM_APPLCODE_SEL;
    g.access = INT_GATE386;
    g.dword_cnt = 0;
    g.offset_lo = offset & 0xFFFF;
    offset >>= 16;
    g.offset_hi = offset & 0xFFFF;
    offset = appl2linear(&g);
    lmemcpy(IDT_addr,offset,sizeof(g));
}

/* Ok; the GDT is managed in the same way as the IDT		    */
/* When a descriptor is cleared using it will cause a SEGMENT FAULT */
/* to refer such descriptor					    */

void GDT_place(WORD sel,DWORD base,DWORD lim,BYTE acc,BYTE gran)
{
    union gdt_entry x;
    DWORD GDT_addr = _X_info->GDT_base + sel;
    DWORD offset = appl2linear(&x);
    x.d.base_lo = (base & 0x0000FFFF);
    x.d.base_med = ((base & 0x00FF0000) >> 16);
    x.d.base_hi = ((base & 0xFF000000) >> 24);
    x.d.access = acc;
    x.d.lim_lo = (lim & 0xFFFF);
    x.d.gran = (gran | ((lim >> 16) & 0x0F));    
    lmemcpy(GDT_addr,offset,sizeof(union gdt_entry));
}

/* This function is used to read & format the descriptor data */
/* Anyway is better to use the hw 386 instruction to modify   */
/* a descriptor rather than reading,modifying & updating with */
/* this high level functions!				      */

DWORD GDT_read(WORD sel,DWORD *lim,BYTE *acc,BYTE *gran)
{
    union gdt_entry x;
    DWORD GDT_addr = _X_info->GDT_base + sel;
    DWORD offset = appl2linear(&x);
    DWORD base;
    lmemcpy(offset,GDT_addr,sizeof(union gdt_entry));
    base = x.d.base_hi;
    base <<= 8;
    base |= x.d.base_med;
    base <<= 16;
    base |= x.d.base_lo;
    if (lim != NULL) {
	*lim = (x.d.gran & 0x0F);
	*lim <<= 16;
	*lim |= x.d.lim_lo;
    }
    if (acc != NULL) *acc = x.d.access;
    if (gran != NULL) *gran = x.d.gran & 0xF0;
    return(base);
}
