; Project: 	HARTIK 3.0      				
; Description:  Hard Real TIme Kernel for 386 & higher machines 
; Author:	Gerardo Lamastra			
; Date:		9/5/96 						
; Revision:	Beta 1.4

; Module: X_A20.Asm
; A20 hardware specific functions

; Remember that WC calling convention uses registers to pass arguments
; and use an underscore postfix on the name, while the standard __cdecl
; convention use an underscore prefix on the name

; Beta 1.4: Modified the A20_query function, using only 8086 like
; 	    registers; when XMS was not active, the system would 
;	    be stuck on certain machines; this now has been fixed
; 	    (I hope definitevely!)

.MODEL C,SMALL

_TEXT		SEGMENT BYTE PUBLIC 'code' USE16
		assume cs:_TEXT
		
		PUBLIC A20_AT_enable_
		PUBLIC A20_AT_disable_
		PUBLIC A20_PS2_enable_
		PUBLIC A20_PS2_disable_
		PUBLIC A20_query_
		
wait8042	PROC NEAR
		xor 	cx,cx	; Used to wait for 8042 ready
wait8042loop:	in 	al,064H	;
		and	al,2	;
		loopnz	wait8042loop
		ret
wait8042	ENDP

; The handlers get the value in AX; if it is 1 then A20 is set
; else A20 is reset; We need different handlers for different machines
; We provide the handlers for PS2 & truly AT compatible machines

_A20_AT		PROC 
		cli
		push 	cx
		push	bx
		mov	bx,ax
		
		call 	wait8042	; Syncronize 8042
		jnz	a20_error_AT	
		mov	al,0d1h		; Select command mode
		out	064h,al
		call	wait8042	; Syncronize 8042
		jnz	a20_error_AT
		mov	al,bl		; Write command 
		out	060H,al
		call	wait8042
		jnz	a20_error_AT
		mov	al,0ffh		; Reset 8042 controller
		out	64h,al
		call	wait8042
		jnz	a20_error_AT	; Check for errors
		mov	ax,1
		pop	bx
		pop	cx
		sti
		ret
a20_error_AT:	xor	ax,ax
		pop	bx
		pop	cx
		sti
		ret
_A20_AT		ENDP

A20_AT_enable_	PROC
		mov	ax,0DFH
		jmp	_A20_AT
A20_AT_enable_	ENDP

A20_AT_disable_	PROC
		mov	ax,0DDH
		jmp	_A20_AT
A20_AT_disable_ ENDP

_A20_PS2	PROC 
		cli
		test	ax,1
		jz	_a20_off_PS2
		
_a20_on_PS2:	in	al,092h		; Read PS2 command port
		or	al,2		; Activate the A20 bit
		out	92h,al		; Store back command port value
		xor	cx,cx		; Wait for a while until A20
wait_a20_on:	in	al,092h		; turns on 
		test	al,2
		loopz	wait_a20_on
		jz	a20_error_PS2
		mov	ax,1
		ret
_a20_off_PS2:	in	al,092h		; Ok same as before			
		and	al,0fdH		; Just mask off A20 bit now
		out	92h,al
		xor	cx,cx
wait_a20_off:	in	al,092h
		test	al,2
		loopnz	wait_a20_off
		jz	a20_error_PS2
		mov	ax,1
		sti
		ret
a20_error_PS2:	xor	ax,ax
		sti
		ret
_A20_PS2	ENDP

A20_PS2_enable_	PROC
		mov	ax,1
		jmp	_A20_PS2
A20_PS2_enable_ ENDP

A20_PS2_disable_ PROC
		 mov	ax,0
		 jmp	_A20_PS2
A20_PS2_disable_ ENDP

A20_query_	PROC 
		push	ds
		push	es
		push	bx
		mov	ax,0
		mov	ds,ax
		mov	ax,0ffffh
		mov	es,ax
		mov	bx,word ptr ds:[0]
		mov	word ptr ds:[0],1234
		cmp	word ptr es:[16],1234
		je	a20_is_off
		mov	word ptr ds:[0],4321
		cmp	word ptr es:[16],4321
		je	a20_is_off
		mov	word ptr ds:[0],bx
		mov	ax,1
		jmp	a20_query_end
a20_is_off:	mov	ax,0
a20_query_end:	pop	bx
		pop	es
		pop	ds
		ret
A20_query_	ENDP

_TEXT		ends

END

