/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 1.0					*/

/* File: X_COFF.C	*/
/* COFF loader support   */

/* Well I suppose some more testing is necessary for this... 	*/
/* We have just done some basic tests, but never tried to    	*/
/* compile HARTIK application under GNU C !			*/

#include <i86.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <io.h>
#include <fcntl.h>

#include <malloc.h>
#include <conio.h>

#include "x86\hw386.h"
#include "x_mem.h"
#include "x_coff.h"
#include "x_stub.h"

//#define __DEBUG_COFF__

/* File descriptor for RM loading using DOS */
static int fd;

/* Coff main header & program headers; refer to X_COFF.H for details */
static FILHDR fh;
static AOUTHDR oh;
static long num_sections;
static long section_seek;
static long symptr;
static long num_symbols;
static SCNHDR sh[3];

/* This is the entry point offset */
static DWORD entry_offset;

/* This is where the application will be loaded!   */
/* Allocate the needed memory through x_MM_getarea */

static DWORD exec_space;

void COFF_read_headers(void)
{
    read(fd,&fh,sizeof(fh));
    #ifdef __DEBUG_COFF__
	printf("Header values:\n");
	printf("f_magic     : %x\n",fh.f_magic);
	printf("f_nscns     : %d\n",fh.f_nscns);
	printf("f_timdat    : %ld\n",fh.f_timdat);
	printf("f_symptr    : %ld\n",fh.f_symptr);
	printf("f_nsyms     : %ld\n",fh.f_nsyms);
	printf("f_opthdr    : %d\n",fh.f_opthdr);
	printf("Flags settings:\n");
	if (fh.f_flags & F_RELFLG) printf("Relocation info : No\n");
	else printf("Relocation info : Yes\n");
	if (fh.f_flags & F_EXEC) printf("Executable : Yes\n");
	else printf("Executable : No\n");
	if (fh.f_flags & F_LNNO) printf("Line numbers : No\n");
	else printf("Line numbers : Yes\n");
	if (fh.f_flags & F_LSYMS) printf("Local symbols : No\n");
	else printf("Local symbols : Yes\n");
    #endif
    num_sections = fh.f_nscns;
    num_symbols = fh.f_nsyms;
    symptr = fh.f_symptr;
    #ifdef __DEBUG_COFF__
	getch();
    #endif
    if (fh.f_opthdr) {
	read(fd,&oh,sizeof(oh));
	#ifdef __DEBUG_COFF__
	    printf("\nOptional header values:\n");
	    printf("Magic           : %x\n",oh.magic);
	    printf("Vstamp          : %d\n",oh.vstamp);
	    printf("Tsize           : %x\n",oh.tsize);
	    printf("Dsize           : %x\n",oh.dsize);
	    printf("Bsize           : %x\n",oh.bsize);
	    printf("Entry           : %x\n",oh.entry);
	    printf("Text_start      : %x\n",oh.text_start);
	    printf("Data_start      : %x\n",oh.data_start);
	#endif
    }
    section_seek = sizeof(FILHDR) + fh.f_opthdr;
    #ifdef __DEBUG_COFF__
	printf("AOUTHDR  size : %d\n",sizeof(oh));
	getch();
    #endif
}

void COFF_read_section(void)
{
    int i;
    if (num_sections != 3) {
	printf("Error: expected only 3 sections\n");
	exit(-1);
    }
    for(i = 0; i < 3; i++) {
	lseek(fd,section_seek,0);
	read(fd,&sh[i],sizeof(SCNHDR));
	section_seek += sizeof(SCNHDR);
	#ifdef __DEBUG_COFF__
	    printf("\nSections #%d info:\n",i);
	    printf("Name            : %s\n",sh[i].s_name);
	    printf("S_paddr         : %lx\n",sh[i].s_paddr);
	    printf("S_vaddr         : %lx\n",sh[i].s_vaddr);
	    printf("S_size          : %lx\n",sh[i].s_size);
	    printf("S_scnptr        : %lx\n",sh[i].s_scnptr);
	    printf("S_relptr        : %lx\n",sh[i].s_relptr);
	    printf("S_lnnoptr       : %lx\n",sh[i].s_lnnoptr);
	    printf("S_nreloc        : %x\n",sh[i].s_nreloc);
	    printf("S_nlnno         : %x\n",sh[i].s_nlnno);
	    printf("S_flags         : %lx\n",sh[i].s_flags);
	    getch();
	#endif
    }
}

void COFF_load_rawdata(void)
{
    register i,j;

    DWORD buf_linaddr = LINEARIZE(load_buf);
    DWORD needed_mem,chunks,residual,where_to_place;
    
    /* The first thing to do is to allocate a bunch of memory 		*
     * which will be used by the executable			  	*
     * This means we need to set up these objects:		  	*
     * 1) allocate the memory obtaining its first address		*
     * 1) a selector for CS based at previous address			*
     * 2) a selector for DS based at previous address			*
     *									*
     * The memory model assumed in COFF is basically flat with 		*
     * virtual address starting from 0! Anyway the COFF exe starts  	*
     * at some arbitrary point in this linear space! We MUST place  	*
     * it just there as no relocation is performed.			*
     * Then we just have this memory layout:				*
     *									*
     * +-----------	0						*
     * |		Unused!						*
     * +------------	usually the entry point is here 		*
     * |		.text section					*
     * |								*
     * +------------							*
     * | 		unused memory of last text page			*
     * +------------ 							*
     * |		.data these sections are joined together	*
     * | 		.bss						*
     * |								*
     * +------------ usually they are page aligned			*/

    needed_mem = sh[2].s_vaddr + sh[2].s_size;
    #ifdef __DEBUG_COFF__
        printf("Needed memory : %lu/%lx\n",needed_mem,needed_mem);
    #endif
    exec_space = x_MM_getarea(needed_mem,EXTENDED_PART);
    X_info.exec_base = exec_space;
    if (exec_space == 0) {
	 printf("Error : Not enough memory to load the COFF executable\n");
	 exit(-1);
    }

    /* .text & .data section 			*/
    /* These must be loaded from disk image 	*/
    for (i = 0; i < 2; i++) {
	chunks = sh[i].s_size / BUFSIZE;
	residual = sh[i].s_size % BUFSIZE;
	lseek(fd,sh[i].s_scnptr,0);
	where_to_place = exec_space + (DWORD)(sh[i].s_vaddr);
	for (j = 0; j < chunks; j++) {
	    memset(load_buf,0,BUFSIZE);
	    read(fd,load_buf,BUFSIZE);
	    x_memcpy(where_to_place,buf_linaddr,BUFSIZE);
	    where_to_place += BUFSIZE;
	}
	if (residual) {
	    memset(load_buf,0,BUFSIZE);
	    read(fd,load_buf,residual);
	    x_memcpy(where_to_place,buf_linaddr,residual);
	    where_to_place += residual;
	}
    }
    /* .bss section 				 */
    /* .bss section is always zero-filled memory */
    if (sh[2].s_size != 0) {
	where_to_place = exec_space + sh[2].s_vaddr;
	x_memset(where_to_place,0,sh[2].s_size);
    }

    ins_descr(PM_APPLCODE_SEL,exec_space,0xFFFFF,CODE_ACCESS,GRAN_32B);
    ins_descr(PM_APPLDATA_SEL,exec_space,0xFFFFF,DATA_ACCESS,GRAN_32B);
    /* Now save the entry offset relative to exec_space */
    X_info.exec_offset = entry_offset = oh.entry;
}

void COFF_spawn(char *application)
{
    int ret;
    fd = open(application,O_RDONLY | O_BINARY);
    if (fd == NULL) {
	printf("Error : COFF Spawning failed\n\n");
	exit(-1);
    }
    COFF_read_headers();
    COFF_read_section();
    COFF_load_rawdata();
    close(fd);
    /* Set the memory related fields of info structure */
    X_info.dos_base = MM.dos_base;
    X_info.dos_size = MM.dos_size;
    X_info.extended_base = MM.extended_base;
    X_info.extended_size = MM.extended_size;
    /* Jump to the PM application */
    #ifdef __COFF_DEBUG__
    	printf("Want to execute the application ?\n");
        if (getch() != 'y') exit(1);
    #endif
    ret = x_call32(PM_APPLCODE_SEL,entry_offset);
    /* When the PM application terminates it will return here! */
}
