/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:      Gerardo Lamastra                                */
/* Date:        9/5/96                                          */
/* Revision:    Beta 1.4                                        */

/* File: COFF.H                                  */
/* Header file for COFF executable file handling */

/* This is mainly extracted from the book:       */
/* Understanding & using COFF                    */
/* Author: Gintaras R. Gircys                    */
/* Editor: Tim O'Reilly                          */

/* Some basic typedefs... */

typedef unsigned long  COFF_Dword;
typedef unsigned short COFF_Word;

/* We have also included the GNU aout header from the GNU C/C++ */
/* compiler for MS-DOS, the DJGPP suite                         */

#pragma pack(1);

typedef struct external_filehdr {
	COFF_Word f_magic;	/* magic number                 */
	COFF_Word f_nscns;	/* number of sections           */
	COFF_Dword f_timdat;	/* time & date stamp            */
	COFF_Dword f_symptr;	/* file pointer to symtab       */
	COFF_Dword f_nsyms;	/* number of symtab entries     */
	COFF_Word f_opthdr;	/* sizeof(optional hdr)         */
	COFF_Word f_flags;	/* flags                        */
} FILHDR;

/* Bits for f_flags:
 *      F_RELFLG        relocation info stripped from file
 *      F_EXEC          file is executable (no unresolved external references)
 *      F_LNNO          line numbers stripped from file
 *      F_LSYMS         local symbols stripped from file
 *      F_AR32WR        file has byte ordering of an AR32WR machine (e.g. vax)
 */

#define F_RELFLG        (0x0001)
#define F_EXEC          (0x0002)
#define F_LNNO          (0x0004)
#define F_LSYMS         (0x0008)

#define I386MAGIC       0x14c
#define I386AIXMAGIC    0x175
#define I386BADMAG(x)   (((x).f_magic!=I386MAGIC)&&(x).f_magic!=I386AIXMAGIC)

#define FILHSZ  sizeof(FILHDR)

typedef struct {
      COFF_Word magic;		/* type of file                         */
      COFF_Word vstamp;         /* version stamp                        */
      COFF_Dword tsize;         /* text size in bytes, padded to FW bdry*/
      COFF_Dword dsize;         /* initialized data "  "                */
      COFF_Dword bsize;         /* uninitialized data "   "             */
      COFF_Dword entry;         /* entry pt.                            */
      COFF_Dword text_start;    /* base of text used for this file */
      COFF_Dword data_start;    /* base of data used for this file */
} AOUTHDR;

typedef struct gnu_aout {
    COFF_Dword info;
    COFF_Dword tsize;
    COFF_Dword dsize;
    COFF_Dword bsize;
    COFF_Dword symsize;
    COFF_Dword entry;
    COFF_Dword txrel;
    COFF_Dword dtrel;
} GNU_AOUT;

#define AOUTSZ (sizeof(AOUTHDR))

#define OMAGIC          0404    /* object files, eg as output */
#define ZMAGIC          0413    /* demand load format, eg normal ld output */
#define STMAGIC         0401    /* target shlib */
#define SHMAGIC         0443    /* host   shlib */

typedef struct external_scnhdr {
	char   s_name[8];       /* section name                 	*/
	COFF_Dword s_paddr; 	/* physical address, aliased s_nlib 	*/
	COFF_Dword s_vaddr;	/* virtual address              	*/
	COFF_Dword s_size;      /* section size                 	*/
	COFF_Dword s_scnptr;    /* file ptr to raw data for section 	*/
	COFF_Dword s_relptr;    /* file ptr to relocation       	*/
	COFF_Dword s_lnnoptr;   /* file ptr to line numbers     	*/
	COFF_Word s_nreloc;     /* number of relocation entries 	*/
	COFF_Word s_nlnno; 	/* number of line number entries	*/
	COFF_Dword s_flags; 	/* flags                        	*/
} SCNHDR;

#define SCNHSZ  sizeof(SCNHDR)

/* Names of "special" sections */

#define _TEXT   ".text"
#define _DATA   ".data"
#define _BSS    ".bss"
#define _COMMENT ".comment"
#define _LIB ".lib"

/* s_flags "type" */

#define STYP_TEXT        (0x0020)       /* section contains text only */
#define STYP_DATA        (0x0040)       /* section contains data only */
#define STYP_BSS         (0x0080)       /* section contains bss only */

#define DEFAULT_DATA_SECTION_ALIGNMENT 4
#define DEFAULT_BSS_SECTION_ALIGNMENT 4
#define DEFAULT_TEXT_SECTION_ALIGNMENT 4

/* For new sections we havn't heard of before */
#define DEFAULT_SECTION_ALIGNMENT 4

void COFF_spawn(char *name);
