/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 1.5					*/

/* File: X_ELF.C	*/
/* ELF loader support   */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <io.h>

#include "x86\hw386.h"
#include "x_mem.h"
#include "x_elf.h"
#include "x_stub.h"

/* File descriptor for RM loading using DOS */
static int fd;
/* Elf main header & program headers; refer to X_ELF.H for details */
static Elf32_Ehdr fh;
static Elf32_Phdr ph[ACTIVE_PH];

/* Human readable translation of used constants */
#ifdef __DEBUG__
   static char *elf_htypes[6] = {"None","Rel","Exec","Dyn","Core","Unknown" };
   static char *elf_mach[10]  = {"None","M32","Sparc","80386","68000 series",
			       	"","88000 series","I860","Mips","Unknown" };
   static char *elf_ptypes[8] = {"Null","Loadable","Dynamic","Interpreted",
    				"Note","Shared library","Program header",
				"Unknown"};
#endif

/* This is the entry point offset */
static DWORD entry_offset;

/* This is where the application will be loaded!   */
/* Allocate the needed memory through x_MM_getarea */

static DWORD exec_space;

void ELF_read_headers(void)
{
    register i;
    /* Read main header */
    read(fd,&fh,sizeof(fh));
    /* Go to the beginning of program headers table */
    lseek(fd,fh.e_phoff,0);
    /* Get all the necessary program headers */
    if (fh.e_phnum > ACTIVE_PH) {
	printf("Error : Too much program headers found\n");
	printf("Recomplie with ACTIVE_PH greater & check .ELF file\n");
	exit(-1);
    }
    for (i = 0; i < fh.e_phnum; i++) 
	read(fd,&ph[i],sizeof(Elf32_Phdr));
}

void ELF_read_rawdata(void)
{
    WORD curr_alignment;
    WORD alignment;
    DWORD needed_mem,chunks,residual,where_to_place;
    DWORD buf_linaddr = LINEARIZE(load_buf);
    register i,j;
    int last_section;
    
    /* Evaluate needed memory summing the p_memsz of each program */
    /* Header; consider required alignment in calculation	  */
    /* Then allocate it into extended memory			  */

    /* BUG FIX!							  */
    /* Well this is not totally true however; we MUST cope with   */
    /* strange virtual address assignment performed by linker	  */
    /*
    needed_mem = 0;
    for (i = 0; i < fh.e_phnum; i++) {
	needed_mem += ph[i].p_memsz;
	curr_alignment = needed_mem % ph[i].p_align;
	if (curr_alignment != 0) needed_mem += ph[i].p_align-curr_alignment;
    }
    */
    /* Check for unique aligment between segment; different would */
    /* sound really strange!					  */
    last_section = fh.e_phnum-1;
    alignment = ph[0].p_align;
    for (i = 1; i < last_section; i++) {
	if (alignment != ph[i].p_align) {
	    printf("Error : Different alignment between segments\n");
	    exit(-1);
	}
    }
    /* Evaluate needed memory as the program is loaded from virtual */
    /* address 0; then we need an amount given by the virtual addr  */
    /* of the last section + the memory required from the last	    */
    /* section							    */
    needed_mem = ph[last_section].p_vaddr + ph[last_section].p_memsz;
    curr_alignment = needed_mem % ph[last_section].p_align;
    if (curr_alignment != 0) needed_mem += ph[last_section].p_align-curr_alignment;

    exec_space = x_MM_getarea(needed_mem,EXTENDED_PART);
    if (exec_space == 0) {
	printf("Error : Not enough memory to load the ELF executable\n");
	exit(-1);
    }

    /* This represent the base of accessible area for ELF application */
    /* Any phisical address must be relocated when accessing memory   */
    /* This is particularly true if we allocate some memory; to use   */
    /* a pointer to access that memory we have to subtract exec_space */
    /* to the phisical address, since the address space for the       */
    /* application is based with this constant!			      */
    /* The PM memory manager MUST know this constant, so we pass it   */
    /* using the info structure				 	      */
    X_info.exec_base = exec_space;
    
    /* Each program header can be splitted into two sections; the first   */
    /* is the binary image of program header which we must load from file */
    /* Its size is given by p_filesz field; then there is a zero filled   */
    /* area representing BSS (or uninitialized data); 			  */

    /* The loaded portion of the program header is loaded first into a    */
    /* temporary buffer in conventional dos memory, then using x_memcpy   */
    /* it is transferred into the extended memory where it will run!      */
    /* The zero-filled portion is set up directly into extended memory	  */
    /* using the x_memset function					  */
    
    for (i = 0; i < fh.e_phnum; i++) {
	chunks = ph[i].p_filesz / BUFSIZE;
	residual = ph[i].p_filesz % BUFSIZE;
	lseek(fd,ph[i].p_offset,0);
	where_to_place = exec_space + (DWORD)(ph[i].p_vaddr);
	for (j = 0; j < chunks; j++) {
	    memset(load_buf,0,BUFSIZE);
	    read(fd,load_buf,BUFSIZE);
	    x_memcpy(where_to_place,buf_linaddr,BUFSIZE);
	    where_to_place += BUFSIZE;
	}
	if (residual) {
	    memset(load_buf,0,BUFSIZE);
	    read(fd,load_buf,residual);
	    x_memcpy(where_to_place,buf_linaddr,residual);
	    where_to_place += residual;
	}
	residual = ph[i].p_memsz - ph[i].p_filesz;
	x_memset(where_to_place,0,residual);
    }

    /* Now we must build some entries into GDT to enable the application  */
    /* to run; the ELF executable assumes all address as flat linear      */
    /* address starting from 0; so we have to set up at least a data      */
    /* selector & code selector which are correctly based with the actual */
    /* address from which we have loaded the application		  */
    /* The start-up code of the application will set the stack selector   */
    /* According to the stack requirements of application itself	  */
    
    ins_descr(PM_APPLCODE_SEL,exec_space,0xFFFFF,CODE_ACCESS,GRAN_32B);
    ins_descr(PM_APPLDATA_SEL,exec_space,0xFFFFF,DATA_ACCESS,GRAN_32B);

    /* Now save the entry offset relative to exec_space */
    X_info.exec_offset = entry_offset = fh.e_entry;
}

void ELF_spawn(char *application)
{
    int ret;
    fd = open(application,O_RDONLY | O_BINARY);
    if (fd == -1) {
	printf("Error : ELF Spawning failed\n\n");
	exit(-1);
    }
    ELF_read_headers();
    ELF_read_rawdata();
    close(fd);
    /* Set the memory related fields of info structure */
    X_info.dos_base = MM.dos_base;
    X_info.dos_size = MM.dos_size;
    X_info.extended_base = MM.extended_base;
    X_info.extended_size = MM.extended_size;
    /* Jump to the PM application */
    ret = x_call32(PM_APPLCODE_SEL,entry_offset);
    /* When the PM application terminates it will return here! */
}

