/* Project: 	HARTIK 3.0      				*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 1.4					*/

/* File: ELF.H					 */
/* Header file for .ELF executable file handling */

/* This file contains *ONLY* the #defines needed to the EXECUTION */
/* It does not contains any reloc stuff 			  */

/* The main structure of an ELF executable is as following 	*/
/* ELF Header							*/
/* Program Header[0]						*/
/* Program Header[1]						*/
/* .................						*/
/* Program Header[N]						*/
/* Segment [0]							*/
/* Segment [1]							*/
/* .................						*/
/* Segment [N]							*/

/* These are the basic ELF data types 		*/
/* From Portable Formats Specification 1.1 	*/
 
typedef unsigned long Elf32_Addr;
typedef unsigned short Elf32_Half;
typedef unsigned long Elf32_Off;
typedef unsigned long Elf32_Sword;
typedef unsigned long Elf32_Word;

/* The ELF header: this is the first thing in an ELF file */

#define EI_NIDENT	16

typedef struct {
    unsigned char e_ident[EI_NIDENT];
    Elf32_Half	e_type;
    Elf32_Half	e_machine;
    Elf32_Word	e_version;
    Elf32_Addr	e_entry;
    Elf32_Off	e_phoff;
    Elf32_Off	e_shoff;
    Elf32_Word	e_flags;
    Elf32_Half	e_ehsize;
    Elf32_Half	e_phentsize;
    Elf32_Half	e_phnum;
    Elf32_Half	e_shentsize;
    Elf32_Half	e_shnum;
    Elf32_Half	e_shstrndx;
} Elf32_Ehdr;

#define	ET_NONE		0	/* Type values */
#define ET_REL		1
#define	ET_EXEC		2
#define ET_DYN		3
#define ET_CORE		4
#define ET_LOPROC	0xff00
#define ET_HIPROC	0xffff

#define EM_NONE		0	/* Machine types */
#define EM_M32		1
#define EM_SPARC	2
#define EM_386		3
#define EM_68K		4
#define EM_88K		5
#define EM_860		7
#define EM_MIPS		8

#define EV_NONE		0	/* version types */
#define EV_CURRENT	1

#define EI_MAG0		0	/* Magic position stuff */
#define EI_MAG1		1
#define EI_MAG2		2
#define EI_MAG3		3
#define EI_CLASS	4
#define EI_DATA		5
#define EI_VERSION	6
#define EI_PAD		7

#define ELFMAG0		0x7f	/* Magic numbers to identify ELF */
#define ELFMAG1		'E'
#define ELFMAG2		'L'
#define ELFMAG3		'F'

#define ELFCLASSNONE	0	/* Elf Class 32 bit/64 bit*/
#define ELFCLASS32	1
#define ELFCLASS64	2

#define ELFDATANONE	0 /* Elf data storage: */
#define ELFDATA2LSB	1 /* LSB : 0x01020304 -> 04030201 [Left = 0 Addr] */
#define ELFDATA2MSB	2 /* MSB : 0x01020304 -> 01020304 [Left = 0 Addr] */

/* Intel 386 standards are:		*/
/* e_ident[EI_CLASS] = ELFCLASS32	*/
/* e_ident[EI_DATA]  = ELFDATA2LSB 	*/

/* ELF Program header */
	
typedef struct {
    Elf32_Word	p_type;
    Elf32_Off	p_offset;
    Elf32_Addr	p_vaddr;
    Elf32_Addr	p_paddr;
    Elf32_Word	p_filesz;
    Elf32_Word	p_memsz;
    Elf32_Word	p_flags;
    Elf32_Word	p_align;
} Elf32_Phdr;

#define PT_NULL		0	/* Type */
#define PT_LOAD		1
#define PT_DYNAMIC	2
#define PT_INTERP	3
#define PT_NOTE		4
#define PT_SHLIB	5
#define PT_PHDR		6
#define PT_LOPROC	0x70000000
#define PT_HIPROC	0x7fffffff

/* Support max. 8 different program headers; this value is totally */
/* arbitrary; I observed that usually thw Watcom linker defines    */
/* only 3 program headers; they are one for code, one for data and */
/* one for stack						   */
#define ACTIVE_PH 	8

/* Load & execute the .ELF file	*/
void ELF_spawn(char *application);
