/* Project:     HARTIK 3.0                                      */
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:      Gerardo Lamastra                                */
/* Date:        9/5/96                                          */
/* Revision:    Beta 1.4                                        */

/* Module: X_Mem.C                                               */
/* X Memory manager                                              */

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include "x86\hw386.h"
#include "x_xms.h"
#include "x_mem.h"

//#define __STUB__
//#define __DEBUG__

struct mem_manager_info MM;

static WORD xms_handle = 0,dos_handle = 0;
// MG
static WORD largest,total;

unsigned XMemQuery(void);
#pragma aux XMemQuery = \
	"mov ax,08800h" \
	"int 015H"    \
	value [ax];

unsigned A20_test(WORD (* en)(void),WORD (* dis)(void),char *what)
{
    int s1,s2,r;
    
    s1 = A20_query();
    #ifdef __DEBUG__
	printf("Testing : %s\n",what);
	printf("[%s] S1 : %d\n",what,s1);
    #endif
    if (!s1) r = en();
    else r = dis();
    s2 = A20_query();
    if (s1 == s2) return(0);
    #ifdef __DEBUG__
	printf("[%s] S2 : %d\n",what,s2);
    #endif
    if (!s2) r = en();
    else r = dis();
    #ifdef __DEBUG__
	printf("[%s] Test result : %s\n",what,s1 != s2 ? "OK" : "BAD");
    #endif
    return(1);
}

void dos_remover(void)
{
    _dos_freemem(dos_handle);
}

void ext_remover(void)
{
    if (xms_handle != 0) {
	XMS_EMB_unlock(xms_handle);
	XMS_EMB_free(xms_handle);
    }
    if (MM.a20_original_status != MM.a20_query()) {
	if (MM.a20_original_status) MM.a20_on();
	else MM.a20_off();
    }
    if (MM.a20_original_status != MM.a20_query())
       printf("Warning! Cannot restore A20 status\n");
}

void x_MM_init(void)
{
    /* First allocate all availabel DOS memory and */
    /* Calculate the linear address of the block   */
    _dos_allocmem(0xFFFF,&largest);
    _dos_allocmem(largest,&dos_handle);
    MM.dos_base = (DWORD)(dos_handle) << 4;
    MM.dos_size = (DWORD)(largest) << 4;
    /* Post the DOS deallocator which must be called on exit */
    atexit(dos_remover);
    #ifdef __DEBUG__
	printf("Dos allocation OK\n");
    #endif
    /* Check for XMS driver */
    if (XMS_init()) {
	#ifdef __DEBUG__
	    printf("XMS detected\n");
	#endif
	MM.mode = MM_XMS;
	/* Set the A20 status analyzer */
	MM.a20_query = XMS_A20_query;
	/* Save original A20 status */
	MM.a20_original_status = MM.a20_query();
	/* Assign correct A20 handlers according XMS 3.0 specs */
	if (XMS_HMA_request()) {
	    MM.a20_on = XMS_A20_global_enable;
	    MM.a20_off = XMS_A20_global_disable;
	    XMS_HMA_release();
	} else {
	    MM.a20_on = XMS_A20_local_enable;
	    MM.a20_off = XMS_A20_local_disable;
	}
	/* Allocate the available extended memory */
	XMS_EMB_query(&largest,&total);
	if (largest != total) printf("Warning: XMS Memory is fragmented [Tot:%lu,Larg:%lu]\n",total,largest);
	if (!XMS_EMB_alloc(largest,&xms_handle)) {
	    printf("Error : %s\n",XMS_parse_error());
	    exit(-1);
	}
	if (!XMS_EMB_lock(xms_handle,&MM.extended_base)) {
	    XMS_EMB_free(xms_handle);
	    printf("Error : %s\n",XMS_parse_error());
	    exit(-1);
	}
	MM.extended_size = (DWORD)(largest) * 1024;
	#ifdef __DEBUG__
	    printf("XMS OK\n");
	#endif
    } else {
	/* No XMS detected; select correct A20 toggling functions */
	MM.mode = MM_RAW;
	/* Set the A20 status analyzer */
	MM.a20_query = A20_query;
	/* Save original A20 status */
	MM.a20_original_status = MM.a20_query();
	/* Check PS/2 protocol */
	if (A20_test(A20_PS2_enable,A20_PS2_disable,"Ps/2")) {
	    MM.a20_on = A20_PS2_enable;
	    MM.a20_off = A20_PS2_disable;
	    MM.technique = A20_PS2_TECH;
	}
	/* Check AT protocol */
	else if (A20_test(A20_AT_enable,A20_AT_disable,"AT")) {
	    MM.a20_on = A20_AT_enable;
	    MM.a20_off = A20_AT_disable;
	    MM.technique = A20_AT_TECH;
	/* Add here other protocol checks... */
	} else {
	    printf("Error : Could not toggle A20\n");
	    exit(-1);
	}
	/* In raw mode we do not consider HMA! So we have all memory */
	/* which lies beyond the 1 MB barrier                        */
	MM.extended_base = 0x100000;
	/* Use BIOS int 15H to evaluate extended memory size         */
	MM.extended_size = (DWORD)(XMemQuery())*1024;
    }
    /* Post the extended memory deallocator */
    atexit(ext_remover);
    /* Enable A20 */
    if (!MM.a20_on()) {
	printf("Error : could not enable A20\n");
	exit(-1);
    }
}

/* The x_MM_getarea is a very raw function which is used only   */
/* to reserve some memory for the application loading           */
/* The real partition manager works in 32 bit, inherithing      */
/* the informations requires from the RM memory manager         */

DWORD x_MM_getarea(DWORD size,BYTE where)
{
    DWORD ret;
    if (where == DOS_PART) {
	if (size > MM.dos_size) return(0);
	ret = MM.dos_base;
	MM.dos_base += size;
	MM.dos_size -= size;
    } else {
	if (size > MM.extended_size) return(0);
	ret = MM.extended_base;
	MM.extended_base += size;
	MM.extended_size -= size;
    }
    return(ret);
}

#ifdef __STUB__

void main()
{
    const char *modestr[] = {"Unavailable","Only Dos","Raw","XMS"};
    const char *techstr[] = {"","AT Style","PS/2 Style"};
    x_MM_init();
    printf("MM Dump after init\n");
    printf("Mode                : %s\n",modestr[MM.mode]);
    if (MM.mode == MM_RAW)
       printf("Technique           : %s\n",techstr[MM.technique]);
    printf("A20 original status : %s\n",MM.a20_original_status ? "On" : "Off");
    printf("A20 actual   status : %s\n",MM.a20_query() ? "On" : "Off");
    printf("Dos memory          : %lu Bytes (From linear address %lu)\n",MM.dos_size,MM.dos_base);
    printf("Ext.memory          : %lu Bytes (From linear address %lu)\n",MM.extended_size,MM.extended_base);
    printf("XMemQuery reports   : %x Block (1K)\n",XMemQuery());
}

#endif
