/* Project: 	HARTIK 3.0     					*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 1.4					*/

/* Module: X_Mem.H						 */
/* C Header file for memory manager interface	 		 */

/* We use this variables as starting address for the system partitions   */
/* XTN will setup 2 partitions: 					 */ 
/* Dos partition : This is the < 640KB free memory allocated from DOS 	 */
/* Extended partition : This is the Extended Memory (>1024KB) available  */

/* XTN will get Dos memory using the DOS allocation functions and will 	 */
/* release it upon exit; The extended memory will be allocated through 	 */
/* XMS services (if available) or in raw mode using the A20 enabling 	 */
/* functions to access all memory beyond 1 MB. 				 */
/* The application can be loaded in extended or conventional memory 	 */
/* The mode field specify if XMS or RAW allocation is currently used     */

#define MM_UNAVAILABLE	0
#define MM_ONLYDOS	1
#define	MM_RAW		2
#define MM_XMS		3

#define A20_AT_TECH	1
#define A20_PS2_TECH	2

struct mem_manager_info {
	WORD mode;
	WORD technique;	
	WORD (*a20_query)(void);
	WORD (*a20_on)(void);
	WORD (*a20_off)(void);
	WORD a20_original_status;
	DWORD dos_base;
	DWORD dos_size;
	DWORD extended_base;
	DWORD extended_size;
};

extern struct mem_manager_info MM;

/* These functions are written in assembler (module X_A20.Asm) and */
/* are specific for single architecture; currently only AT & PS/2  */
/* A20 management is supported					   */ 
/* They return 1/0 on success/failure				   */
/* The query function returns 1 if detect A20 enabled, 0 otherwise */

WORD A20_AT_enable(void);
WORD A20_AT_disable(void);
WORD A20_PS2_enable(void);
WORD A20_PS2_disable(void);
WORD A20_query(void);

/* Initialize MM & install the automatic release_mem-on-exit function */
void x_MM_init(void);
/* Obtain an usable memory area in a specific partition */
#define DOS_PART	1
#define EXTENDED_PART	2	
DWORD x_MM_getarea(DWORD size,BYTE where);

