; Project: 	HARTIK 3.0      				
; Description:  Hard Real TIme Kernel for 386 & higher machines 
; Author:	Gerardo Lamastra			
; Date:		9/5/96 						
; Revision:	Beta 2.0

; Module: X_Switch.Asm
; RM <-> PM Switching functions & 32 Bit stubs for PM functions

;
; Warning! These settings are duplicated in hw386.h
; They *MUST* be obviously the same thing!
;

.386P
.MODEL SMALL,C

;
; Use X standard GDT selectors
;

INCLUDE x86\sel.inc

;
; These macros are used to transform a real mode segment:offset address
; into a flat linear address; the second one assumes that we are dealing 
; with a segment name only
;

x_lin@		MACRO s,o
		xor	eax,eax
		mov 	ax,s
		shl	eax,4
		add	eax,OFFSET o
		ENDM

x_lin0@		MACRO s,o
		xor	eax,eax
		mov 	ax,SEG s
		shl	eax,4
		add	eax,OFFSET o
		ENDM

;
; When we are in a USE16 segment we MUST add the 066h prefix to the push
; & pop operation in order to use the DWORD alignment when accessing
; the stack
;

pushd		MACRO x
		DB 	066h
		push	x
		ENDM

popd		MACRO x
		DB 	066h
		pop	x
		ENDM

;
; Just for debugging purpose, these macro display a char on text screen
; The first MUST be used while in PM, the last ONLY for RM
; Hopefully stack should be active!
;
		
pm_watch@	MACRO x
		push	eax
		push	ds
		push	edi
		mov	ax,X_VIDEO_SEL
		mov	ds,ax
		mov	edi,0
		mov	byte ptr ds:[edi],'x'
		pop	edi
		pop	ds
		pop	eax
		ENDM

rm_watch@	MACRO x
		push	ax
		push	es
		push	di
		mov	ax,0b800h
		mov	es,ax
		mov	di,4
		mov	byte ptr es:[di],'x'
		pop	di
		pop	es
		pop	ax
		ENDM

;
; These variables are declared in X_Stub.C
;

EXTRN 	_GDT : BYTE		; System tables
EXTRN	_IDT : BYTE		;

EXTRN	_real_pic1_mask : BYTE	; PIC masks 
EXTRN	_real_pic2_mask : BYTE	; 1:Master 2:Slave
EXTRN	_real_data : WORD	; DS,SS,ES for real mode
EXTRN	_real_code : WORD	; CS for real mode
EXTRN	_PM_esp : DWORD		; Stack top for PM

;
; This function is implemented in X_Stub.C
;

EXTRN	x_bios32_ : NEAR

;
; Use PUBLIC variables to debug purposes; it is easier to watch them
; using the symbolic debugger;
; Pay attention!  WD (Ver ) has troubles in handling extended register
; if you are debugging a RM application (DOS type) using extended register
; settings
;

_DATA           SEGMENT WORD PUBLIC  'DATA'
		PUBLIC _x_sp
		PUBLIC _old_esi
		PUBLIC _old_edi
		PUBLIC _old_eax
		PUBLIC _old_ebx
		PUBLIC _old_ecx
		PUBLIC _old_edx
		PUBLIC _old_ip
		PUBLIC _old_cs
		PUBLIC _old_ebp
		PUBLIC _old_flags
		PUBLIC gdtr_base
		PUBLIC gdtr_lim
		PUBLIC gdtr_image
		PUBLIC idtr_base
		PUBLIC idtr_lim
		PUBLIC idtr_image
		PUBLIC _old_GDTR
		PUBLIC _old_IDTR
		PUBLIC _x_sp
		
_x_sp		dw 	(0) ; Old values for SP,GDTR,IDTR & CR0
_old_GDTR	dp 	(0) ; Restored when X returns to RM
_old_IDTR	dp 	(0)
_old_CR0	dd	(0)

_x_esp		dd	(0) ; Stack top for BIOS call switch
_x_ss		dw	(0)
_x_cr0		dd	(0) ; Cr0 Setting to return to PM

_old_eax	dd	(0) ; The 32 bit regs are passed during 
_old_ebx	dd	(0) ; the RM->PM switch
_old_ecx	dd	(0)
_old_edx	dd	(0)
_old_esi	dd	(0)
_old_edi	dd	(0)

_old_ebp	dd	(0) ; These regs are saved when you enter in PM
_old_ip		dw	(0) ; and restored when you come back to RM to
_old_cs		dw	(0) ; close PM application 
_old_flags	dw	(0) ; 

gdtr_image      label	pword ; Used to assemble the GDTR & IDTR
gdtr_lim	dw 	(0)
gdtr_base	dd 	(0)
idtr_image      label 	pword
idtr_lim        dw 	(0)
idtr_base	dd 	(0)
_DATA		ends

_TEXT		SEGMENT BYTE PUBLIC 'code' USE16
		assume cs:_TEXT,ds:_DATA,es:_DATA
		
		PUBLIC _x_call32
		PUBLIC _x_memcpy
		PUBLIC _x_memcmp
		PUBLIC _x_memset
		
; Used to insert a gate into the gdt/idt; the table is specified
; giving the table offset into the dx regsiter
; di  : gate index
; dx  : table offset
; ax  : selector \
;                  called address
; ebx : offset   /
; ch  : gate type
; cl  : word/dwords (only for call gate 16/32 bit)

ins_gate	PROC	
		add	di,dx
		mov 	es:[di],bx   ; bx: low offset address
		add 	di,2         ; 
		mov 	es:[di],ax   ; ax: selector
		add 	di,2
		and 	cl,00011111b ; can max transfer 32 dwords/words
		mov 	es:[di],cx   ; cx: num. of words & type
		add 	di,2
		shr 	ebx,16       ; now bx: hi hi offset address
		mov 	es:[di],bx   ; insert in the descriptor
		ret
ins_gate	ENDP

;
; 32 bit memory management functions:
; memcpy,memset,memcmp; they all use 4G linear addressing
;

_x_memcpy       PROC	
		push	si
		push	di
		push    bp
                mov     bp,sp
                mov     edi,dword ptr +8[bp]
		mov	esi,dword ptr +12[bp]
		mov	ecx,dword ptr +16[bp]
		x_lin0@	_TEXT32,__x_memcpy
		pushd	eax
		push	X_FLATCODE_SEL
		call	_x_call32
		add	sp,6
                pop     bp
		pop	di
		pop	si
                ret
_x_memcpy	ENDP

_x_memcmp	PROC	
		push	si
		push	di
		push    bp
                mov     bp,sp
                mov     edi,dword ptr +8[bp]
		mov	esi,dword ptr +12[bp]
		mov	ecx,dword ptr +16[bp]
		x_lin0@	_TEXT32,__x_memcmp
		pushd	eax
		push	X_FLATCODE_SEL
		call	_x_call32
		add	sp,6
                pop     bp
		pop	di
		pop	si
                ret
_x_memcmp	ENDP

_x_memset	PROC	
		push	si
		push	di
		push    bp
                mov     bp,sp
                mov     edi,dword ptr +8[bp]
		mov	bx,word ptr +12[bp]
		mov	ecx,dword ptr +14[bp]
		x_lin0@	_TEXT32,__x_memset
		pushd	eax
		push	X_FLATCODE_SEL
		movzx	eax,bx
		call	_x_call32
		add	sp,6
                pop     bp
		pop	di
		pop	si
                ret
_x_memset	ENDP

;
; This starts up the pm procedure whose address is passed onto stack top
; The C proto is void x_call32(WORD selector,DWORD offset)
; If you use X_FLATCODE_SEL the offset is a true linear address
; Otherwise you can specify any selector (obviously correctly assigned)
; and an offset relative to the segment base
; The general registers are all passed to the protected mode procedure;
; EBP,ESP & EFlags are saved and restored when you return to RM
; The Segment registers are saved/restored also
;

_x_call32	PROC 

		push	ax		; Save AX to avoid clobbering it		
		pushf			; Save the flags; restore them when 
		pop	ax		; return to RM
		mov	_old_flags,ax
		cli			; Ignore HW-INT
		pop	ax		;
		mov	_old_eax,eax	; 
		mov	_old_ebx,ebx	; Used to pass the regs
		mov	_old_ecx,ecx	; to the PM routine
		mov	_old_edx,edx	;
		mov	_old_esi,esi
		mov	_old_edi,edi
		
		mov	_old_ebp,ebp	; These one 
		pop	ax		; are saved but not (of course)
		mov	_old_ip,ax	; passed to PM
		
		mov	dx,021h		; mask all hw interrupts
		mov	al,0ffh		; 
		out	dx,al
		mov	dx,0a1h
		out	dx,al

		db	066H		; Save the old GDTR/IDTR
		sgdt	_old_gdtr	; and prepare the new register
		db	066H		; images
		sidt	_old_idtr	;
		
		mov 	gdtr_lim,(256*8-1) 
		x_lin@	DS,_GDT
		mov	gdtr_base,eax	
			
		mov 	idtr_lim,(256*8-1)	
		x_lin@	DS,_IDT
		mov	idtr_base,eax
		
		mov	_x_sp,sp	; This is the correct stack setting
					; The C calling sequence will clean
					; up stack when you return there!
					
		mov	ax,ds		; Insert the X_PM_ENTRY_GATE selector
		mov	es,ax		; in the GDT; use the stack arguments
		mov	dx,offset _GDT	; in the following order:
		pop	ax		; This is the SELECTOR
		popd	ebx		; This is the OFFSET
		mov 	di,X_PM_ENTRY_GATE
		mov 	cl,0		; 0 dword to tranfer
		mov 	ch,10001100B	; Call Gate for 386
		call 	ins_gate	; Insert gate in GDT
		
		db   	066h		; with override (32 bit argument)
		lgdt 	gdtr_image	; load the gdt
					; int disabled
		db	066h		; now it is safe to load
		lidt	idtr_image	; the new idt
							
		mov 	eax,cr0		; switch to pm 
		mov	_old_CR0,eax	; store old CR0 value 
		or 	eax,1
		mov 	cr0,eax		
		
		; This stuff here is necessary as WASM could mess
		; everything if you refer PM objects from a RM application
		; 066H Prefix for 32 bit argument
		; 0EAH is the JMP opcode
		
		db 	066h		  ; jmp using a direct jump
		db 	0eah		  ; using a 32 bit argument
		dd 	offset __x_call32 ; to the pm stub used for the
		dw 	X_CODE32_SEL	  ; switch
_x_call32	endp

rm_stub		LABEL 	FAR 
		mov 	ax,X_DATA_SEL	; CS is now set for RM
		mov 	ds,ax		; fix the other segment
		mov 	es,ax		; registers! 
		mov 	ss,ax
		mov 	fs,ax
		mov 	gs,ax
					; back to RM
		mov 	eax,_old_CR0	; Use 386 switch capability!
		mov 	cr0,eax		; & empty prefetch queue
		
		jmp	empty_prefetch
		
empty_prefetch	LABEL	FAR
		mov	ax,_real_data		; Recover the real mode 
		mov	ds,ax			; Segments 
		mov	es,ax
		mov	ss,ax
					
		mov	al,_real_pic1_mask	; restore old PIC masks!
		mov	dx,021h
		out	dx,al
		mov	al,_real_pic2_mask
		mov	dx,0a1h
		out	dx,al
		mov 	sp,_x_sp		; Recover RM stack pointer
		db	066h			; Recover old GDT & IDT 
		lgdt 	_old_gdtr		; Settings 		
		db	066h
		lidt 	_old_idtr	
		push	_old_flags		; Set up the IRET context
		push	_real_code
		push	_old_ip	
		mov	eax,_old_eax	; Get the register used to
		pushd	eax		; pass PM info to RM application
		mov	ebx,_old_ebx
		mov	ecx,_old_ecx
		mov	edx,_old_edx
		mov	esi,_old_esi
		mov	edi,_old_edi
		mov	ebp,_old_ebp
		mov 	ax,_real_data	; Now recover other seg regs!
		mov 	es,ax		; 
		mov	fs,ax		; These are usually unused in RM 
		mov 	gs,ax		; Anyway fix them with a RM valid 
					; Segment value
		popd	eax		; Recover the old eax!
		iret			; Back to the caller

callBIOS_stub	LABEL 	FAR
		mov 	ax,X_DATA_SEL	; CS is now set for RM
		mov 	ds,ax		; fix the other segment
		mov 	es,ax		; registers! 
		mov 	ss,ax
		mov 	fs,ax
		mov 	gs,ax
					; back to RM
		mov 	eax,_old_CR0	; Use 386 switch capability!
		mov 	cr0,eax		; & empty prefetch queue		
		jmp	empty_2
		
empty_2		LABEL	FAR
		mov	ax,_real_data	; Recover the real mode 
		mov	ds,ax		; Segments 
		mov	es,ax
		mov	ss,ax
		mov 	sp,_x_sp	; Recover RM stack pointer
		db	066h		; Recover old GDT & IDT 
		lgdt 	_old_gdtr	; Settings 		
		db	066h
		lidt 	_old_idtr
		mov 	ax,_real_data	; Now recover other seg regs!
		mov 	es,ax		; 
		mov	fs,ax		; These are usually unused in RM 
		mov 	gs,ax		; Anyway fix them with a RM valid 
					; Segment value
		
		call	x_bios32_	; Transfer the control to C code
					; which implement the real mode
					; bios service call
					
					; Now we return to PM ...
		db   	066h		; With override (32 bit argument)
		lgdt 	gdtr_image	; load the gdt
					; int disabled
		db	066h		; now it is safe to load
		lidt	idtr_image	; the new idt
							
		mov 	eax,_x_cr0	; Switch to pm 
		mov 	cr0,eax		; using original CR0 settings
		
		; This stuff here is necessary as WASM could mess
		; everything if you refer PM objects from a RM application
		; 066H Prefix for 32 bit argument
		; 0EAH is the JMP opcode
		
		db 	066h		  ; jmp using a direct jump
		db 	0eah		  ; using a 32 bit argument
		dd 	offset __backto32 ; to the pm stub used for the
		dw 	X_CODE32_SEL	  ; switch
_TEXT		ends

; The x_code32 contains all the 32 bit code; there are 2 class of functions:
; 1) protected mode code for some real mode functions
; 2) protected mode code for the X-API functions accessed from PM using an
;    interrupt gate
;

_TEXT32		SEGMENT PARA PUBLIC 'code32' USE32
		ASSUME DS:_DATA,CS:_TEXT32
		
		; These functions must be visible externally
		; to ensure correct settings for PM selectors
		; which are defined in X_Stub.C 
		
		; __x_call32 is also used to mark the beginning
		; of _TEXT32 segment which contains all PM code 
		; accesible from X RM interface!
		
		PUBLIC __x_call32,__x_ret16,__x_callBIOS
		
__x_call32	LABEL 	FAR

		mov 	ax,NULL_SEL	; fs & gs are null
		mov 	fs,ax
		mov 	gs,ax
		
		; This SS setting here sound strange but is necessary because
		; of small model assumption; in fact the assembler refers
		; the variables placed into _DATA segment by mean of SS
		; segment register; then is necessary that ALSO SS is loaded
		; with the correct selector for accessing datas!		
		
		mov 	ax,X_DATA_SEL	; set up pm ds,es, ...
		mov 	ds,ax		; 		
		mov 	es,ax		;		
		mov	ss,ax		; 
		mov 	esp,_PM_esp	; set up ESP for PM
		mov	eax,_old_eax	; pass the general regs
		mov	ebx,_old_ebx	; to protected mode procedure!
		mov	ecx,_old_ecx	; Here SS is used to access
		mov	edx,_old_edx	; the _old... vars!
		mov	esi,_old_esi
		mov	edi,_old_edi		
		sti			; Interrupt again!
		db 	0eah		; Jmp using the call gate
		dd 	0		; to the entry point
		dw 	X_PM_ENTRY_GATE	;
		; no ret needed!

__x_ret16	LABEL	FAR		; This function MUST be accessed
		cli			; through a gate if you are calling
		push	X_DATA_SEL	; it from an application procedure!
		pop	ds		; Switch to X kernel data without
		push	X_DATA_SEL	; clobbering eax
		pop	ss
		
		mov	_old_eax,eax	; These regs are passed to
		mov	_old_ebx,ebx	; the RM procedure!
		mov	_old_ecx,ecx	; USE DS Overrides
		mov	_old_edx,edx
		mov	_old_esi,esi
		mov	_old_edi,edi
		
		db 	066h		; jmp using a direct jump
		db 	0eah		; using a 16 bit argument
		dw 	offset rm_stub
		dw 	X_CODE16_SEL

__x_callBIOS	LABEL	FAR		; This function MUST be accessed
		cli			; through a gate if you are calling
					; it from an application procedure!		
		pushad			; Save everything
		push	ds
		push	ss
		push	es
		push	fs
		push	gs
		
		mov	ax,X_DATA_SEL	; Switch to X kernel data in DS
		mov	ds,ax		; 
		mov	ax,ss		; Store SS in AX
		push	ds		; Switch to X data also in SS
		pop	ss
		
		mov	_x_ss,ax	; Save PM stack pointer
		mov	_x_esp,esp	; OK wether wasm use SS or DS!
		mov	eax,cr0		; Save CR0 settings
		mov	_x_cr0,eax
		
		db 	066h		; jmp using a direct jump
		db 	0eah		; using a 16 bit argument
		dw 	offset callBIOS_stub
		dw 	X_CODE16_SEL
		
__backto32	PROC	FAR		; Execution resumes here		
		mov 	ax,X_DATA_SEL	; Recover stack pointer
		mov	ds,ax		;
		mov	ss,ax		;
		 
		mov 	esp,_x_esp	; 
		mov	ax,_x_ss	;
		mov	ss,ax		
		
		pop	gs		; Restore everything
		pop	fs		; And go back to application...
		pop	es
		pop	ss
		pop	ds
		popad
		sti
		retf
__backto32	ENDP
		
__x_memcpy	LABEL	FAR
		mov	ax,X_FLATDATA_SEL
		mov	ds,ax
		mov	es,ax
		cld
		rep	movsb
		db	0eah		; Direct gate jmp to RM
		dd	0		;
		dw	X_RM_BACK_GATE

__x_memcmp	LABEL	FAR
		mov	ax,X_FLATDATA_SEL
		mov	ds,ax
		mov	es,ax
		cld
		repe	cmpsb
		jz	__equal
		mov	eax,0
		jmp	__differnt
__equal:	mov	eax,1
__differnt:	db	0eah		; Direct gate jmp
		dd	0		;
		dw	X_RM_BACK_GATE

__x_memset	LABEL	FAR
		push	eax
		mov	ax,X_FLATDATA_SEL
		mov	ds,ax
		mov	es,ax
		pop	eax
		cld
		rep	stosb
		db	0eah		; Direct gate jmp
		dd	0		;
		dw	X_RM_BACK_GATE

; Insert here other __... stub functions defining PM code
; which can be called from X using x_call32()

_TEXT32		ENDS

end
