/* Project: 	HARTIK 3.0     					*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 1.4					*/

/* X_XMS.C: XMS interface implementation */
/* Following the XMS 3.0 standard available at ftp.microsoft.com */

#include <i86.h>
#include <stdio.h>
#include "x_xms.h"

static union REGS in,out;
static struct SREGS seg;
static void ((__far * __far XMS_hook))(void);
static WORD _AX,_BX,_CX,_DX;

char XMS_error = 1;

static void __far _noop(void)
{
 XMS_error = 1;
}

void read_regs(void);
#pragma aux read_regs = \
	"mov _AX,ax" \
	"mov _BX,bx" \
	"mov _CX,cx" \
	"mov _DX,dx";

void XMS_calldriver(void);
#pragma aux XMS_calldriver = \
	"push es" \
	"mov ax,seg XMS_hook"\
	"mov es,ax"  \
	"mov ax,_AX" \
	"mov bx,_BX" \
	"mov cx,_CX" \
	"mov dx,_DX" \
	"call dword ptr es:XMS_hook" \
	"mov _AX,ax" \
	"mov _BX,bx" \
	"mov _CX,cx" \
	"mov _DX,dx" \
	"pop es" \
	modify [ax bx cx dx];
		

WORD XMS_init(void)
{
 XMS_hook = &(_noop);
 in.x.ax = 0x4300;
 int86(0x2f,&in,&out);
 if (out.h.al != 0x80) return(0);
 in.x.ax = 0x4310;
 int86x(0x2f,&in,&out,&seg);
 XMS_hook = MK_FP(seg.es,out.x.bx);
 XMS_error = 0;
 return(1);
}

char *XMS_parse_error(void)
{
 switch (XMS_error) {
     case 0x00 : return("No error");
     case 0x01 : return("Driver (HIMEM.SYS) not loaded");
     case 0x80 : return("Function not implemented");
     case 0x81 : return("Found VDISK device");
     case 0x82 : return("A20 error");
     case 0x8e : return("General driver error");
     case 0x8f : return("Unrecoverable driver error");
     case 0x90 : return("HMA does not exist");
     case 0x91 : return("HMA already allocated");
     case 0x92 : return("Too low memory required for HMA allocation");
     case 0x93 : return("HMA not allocated");
     case 0x94 : return("A20 still on");
     case 0xa0 : return("All extended memory allocated");
     case 0xa1 : return("All handles are used");
     case 0xa2 : return("Handle invalid");
     case 0xa3 : return("Source handle is invalid");
     case 0xa4 : return("Source offset is invalid");
     case 0xa5 : return("Destination handle is invalid");
     case 0xa6 : return("Destination offset is invalid");
     case 0xa7 : return("Length is invalid");
     case 0xa8 : return("Move with invalid overlap");
     case 0xa9 : return("Parity error");
     case 0xaa : return("Block not locked");
     case 0xab : return("Block locked");
     case 0xac : return("Block lock count overflow");
     case 0xad : return("Lock failure");
     case 0xb0 : return("Only smaller UMB available");
     case 0xb1 : return("No UMBs available");
     case 0xb2 : return("UMB segment number invalid");
     default : return("Unknown error code");
 }
}

WORD XMS_version(WORD *ver)
{
 read_regs();
 _AX = 0;
 XMS_calldriver();
 if (ver != NULL) *ver = _AX;
 return(_DX);
}

WORD XMS_HMA_request(void)
{
 read_regs();
 _AX = 0x0100;
 _DX = 0xFFFF;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX);
}

WORD XMS_HMA_release(void)
{
 read_regs();
 _AX = 0x0200;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX); 
}

WORD XMS_A20_global_enable(void)
{
 read_regs();
 _AX = 0x0300;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX);
}

WORD XMS_A20_local_enable(void)
{
 read_regs();
 _AX = 0x0500;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX);
}

WORD XMS_A20_global_disable(void)
{
 read_regs();
 _AX = 0x0400;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX);
}

WORD XMS_A20_local_disable(void)
{
 read_regs();
 _AX = 0x0600;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX);
}

WORD XMS_A20_query(void)
{
 read_regs();
 _AX = 0x0700;
 XMS_calldriver();
 XMS_error = _BX;
 return(_AX);
}

WORD XMS_EMB_query(WORD *largest,WORD *total)
{
 read_regs();
 _AX = 0x0800;
 XMS_calldriver();
 XMS_error = _BX;
 *largest = _AX;
 *total = _DX;
 XMS_error = 0;
 return(*largest);
}

WORD XMS_EMB_alloc(WORD size,WORD *handle)
{
 read_regs();
 _AX = 0x0900;
 _DX = size;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 *handle = _DX;
 return(_AX);
}

WORD XMS_EMB_free(WORD handle)
{
 read_regs();
 _AX = 0x0A00;
 _DX = handle;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX);
}

WORD XMS_EMB_lock(WORD handle,DWORD *addr)
{
 read_regs();
 _AX = 0x0C00;
 _DX = handle;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 *addr = ((DWORD )(_BX) | (((DWORD )(_DX)) << 16));
 return(_AX);
}

WORD XMS_EMB_unlock(WORD handle)
{
 read_regs();
 _AX = 0x0D00;
 _DX = handle;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX); 
}

WORD XMS_EMB_handlequery(WORD handle,WORD *locks,WORD *size,WORD *freehandles)
{
 read_regs();
 _AX = 0x0E00;
 _DX = handle;
 XMS_calldriver();
 if (locks != NULL) *locks = _BX >> 8;
 if (size != NULL) *size = _DX;
 if (freehandles != NULL) *freehandles = (_BX & 0xFF);
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX); 
}

WORD XMS_EMB_realloc(WORD handle,WORD size)
{
 read_regs();
 _AX = 0x0F00;
 _BX = size;
 _DX = handle;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX); 
}

WORD XMS_UMB_alloc(WORD *size,WORD *handle)
{
 read_regs();
 _AX = 0x1000;
 _DX = *size;
 XMS_calldriver();
 if (_AX == 0) {
     XMS_error = _BX;
     *handle = 0;
     *size = _DX;
 }
 else {
     XMS_error = 0;
     *handle = _BX;
     *size = _DX;
 }
 return(_AX);
}

WORD XMS_UMB_free(WORD handle)
{
 read_regs();
 _AX = 0x1100;
 _DX = handle;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX); 
 
}

WORD XMS_UMB_realloc(WORD handle,WORD size)
{
 read_regs();
 _AX = 0x1200;
 _BX = size;
 _DX = handle;
 XMS_calldriver();
 if (_AX == 0) XMS_error = _BX;
 else XMS_error = 0;
 return(_AX);
}

/* This is for debugging purposes only! */

#ifdef __DEBUG__

void process_error()
{
 if (!XMS_error) return;
 printf("XMS Error (%x): %s\n",XMS_error,XMS_parse_error());
 exit(-1);
}

void main()
{
 WORD handle,locks,size;
 WORD largest,total,version,hma_present,a20_status;
 WORD EMB_op_res,freehandles,UMB_op_res;
 WORD long phaddr;
 if (!XMS_init()) process_error();

 puts("Querying version....");
 hma_present = XMS_version(&version);
 process_error();
 printf("XMS Driver version   : %x.%x\n",version >> 8,version & 0xff);
 printf("HMA                  : %s\n",hma_present ? "Present" : "Absent");

 puts("Querying A20 status....");
 a20_status = XMS_A20_query();
 process_error(); 
 printf("A20 status            : %d %s\n",a20_status,a20_status ? "On" : "Off");

 puts("Toggling A20....(Global mode)");
 if (a20_status) XMS_A20_global_disable();
 else XMS_A20_global_enable();
 process_error();
 printf("A20 status            : %s\n",XMS_A20_query() ? "On" : "Off");
 if (a20_status) XMS_A20_global_enable();
 else XMS_A20_global_disable();
 printf("A20 status            : %s\n",XMS_A20_query() ? "On" : "Off");

 puts("Toggling A20....(Local mode)");
 if (a20_status) XMS_A20_local_disable();
 else XMS_A20_local_enable();
 process_error();
 printf("A20 status            : %s\n",XMS_A20_query() ? "On" : "Off");
 if (a20_status) XMS_A20_local_enable();
 else XMS_A20_local_disable();
 printf("A20 status            : %s\n",XMS_A20_query() ? "On" : "Off");

 puts("Querying extended memory info....");
 XMS_EMB_query(&largest,&total);
 process_error();
 printf("Largest EMB           : %u KB\n",largest);
 printf("Total Extended memory : %u KB\n",total);

 puts("Getting the extended memory....");
 EMB_op_res = XMS_EMB_alloc(largest,&handle);
 process_error();
 printf("I had handle          : %u\n",handle);

 puts("Locking memory to obtain phisical address...");
 EMB_op_res = XMS_EMB_lock(handle,&phaddr);
 process_error();
 printf("Extended memory base address is : %lx Hex\n",phaddr);
 
 puts("Querying info on my handle....");
 EMB_op_res = XMS_EMB_handlequery(handle,&locks,&size,&freehandles);
 process_error();
 printf("Handle locks          : %u\n",locks);
 printf("Handle memory size    : %u\n",size);
 printf("Free system handles   : %u\n",freehandles);

 puts("Unlocking & Releasing memory....");
 EMB_op_res = XMS_EMB_unlock(handle);
 process_error();
 EMB_op_res = XMS_EMB_free(handle);
 process_error();

 puts("Trying UMB allocation....");
 size = 0xFFFF;
 UMB_op_res = XMS_UMB_alloc(&size,&handle);
 if (!UMB_op_res && XMS_error == 0xB0)
    UMB_op_res = XMS_UMB_alloc(&size,&handle);
 process_error();
 printf("Largest available UMB : %lu bytes\n",(WORD long)(size*16));
 printf("UMB address           : %lx\n",(WORD long)(handle * 4));
 
 puts("Freeing UMB....");
 UMB_op_res = XMS_UMB_free(handle);
 process_error();
 
 exit(1);
}
#endif
