/* Project: 	HARTIK 3.0     					*/
/* Description: Hard Real TIme Kernel for 386 & higher machines */
/* Author:	Gerardo Lamastra				*/
/* Date:	9/5/96 						*/
/* Revision:	Beta 1.4					*/

/* X_XMS.H: XMS Interface header file */

/* Init: return 1 on succesful init 0 otherwise  */
/* Version: return 1/0 if HMA is present or not; */
/*          ver is 0x<maior><minor>		 */

/* These pseudo-typedefs are only used to shorten declarations */

#ifndef WORD
#define WORD unsigned short
#endif

#ifndef DWORD
#define DWORD unsigned long
#endif
  
WORD XMS_init(void);
WORD XMS_version(WORD *ver);

/* HMA request/release function 		 */
/* These functions are meant to be used in an    */
/* application which will release HMA upon exit! */

WORD XMS_HMA_request(void);
WORD XMS_HMA_release(void);

/* Enable/Disable A20 to provide access to EMB & HMA */

WORD XMS_A20_global_enable(void);
WORD XMS_A20_local_enable(void);
WORD XMS_A20_global_disable(void);
WORD XMS_A20_local_disable(void);
WORD XMS_A20_query(void);

/* EMB (Memory over 1MB + 64KB[HMA] management functions 		*/
/* To access phisical memory you *MUST* lock it 		   	*/
/* Release all memory on exit! No protection scheme implemented to 	*/
/* prevent unfreeing of EMBs 					   	*/

WORD XMS_EMB_query(WORD *largest,WORD *total);
WORD XMS_EMB_alloc(WORD size,WORD *handle);
WORD XMS_EMB_free(WORD handle);
WORD XMS_EMB_lock(WORD handle,DWORD *addr);
WORD XMS_EMB_unlock(WORD handle);
WORD XMS_EMB_handlequery(WORD handle,WORD *locks,WORD *size,WORD *freehandles);
WORD XMS_EMB_realloc(WORD handle,WORD size);

/* UMB (memory between 640KB and 1MB) management functions 	 */
/* These functions are only usable if the UMB manager is present */
/* Generally the UMB manager is EMM386.EXE			 */

WORD XMS_UMB_alloc(WORD *size,WORD *handle);
WORD XMS_UMB_free(WORD handle);
WORD XMS_UMB_realloc(WORD handle,WORD size);

/* XMS_errors, #defines & human-string id for errors */

#define XMS_OK				0
#define XMS_ABSENT			1
#define XMS_FUNCTION_NOT_PRESENT	0x80
#define XMS_VDISK_FOUND			0x81
#define XMS_A20_ERROR			0x82
#define XMS_GENERAL_ERROR		0x8e
#define XMS_UNRECOVERABLE_ERROR		0x8f
#define XMS_HMA_NOT_PRESENT		0x90
#define XMS_HMA_ALLOCATED		0x91
#define XMS_HMA_LOW_MEM			0x92
#define XMS_HMA_NOT_ALLOCATED		0x93
#define XMS_A20_STILL_ON		0x94
#define XMS_OUT_OF_EMB			0xa0
#define XMS_OUT_OF_HANDLES		0xa1
#define XMS_HANDLE_INVALID		0xa2
#define XMS_SOURCE_HANDLE_INVALID	0xa3
#define XMS_SOURCE_OFFSET_INVALID	0xa4
#define XMS_DEST_HANDLE_INVALID		0xa5
#define XMS_DEST_OFFSET_INVALID		0xa6
#define XMS_LEN_INVALID			0xa7
#define XMS_OVERLAP_INVALID		0xa8
#define XMS_PARITY			0xa9
#define XMS_BLOCK_UNLOCKED		0xaa
#define XMS_BLOCK_LOCKED		0xab
#define XMS_LOCK_OVERFLOW		0xac
#define XMS_LOCK_FAILURE		0xad
#define XMS_UMB_SMALLER_ONLY		0xb0
#define XMS_UMB_NOT_AVAILABLE		0xb1
#define XMS_UMB_SEG_INVALID		0xb2

extern char XMS_error;
char *XMS_parse_error(void);


